﻿
-- Changing fields with non-printing characters only to NULL in SourceContacts

UPDATE SourceContacts SET SourceCompany = NULL   WHERE SourceCompany IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(SourceCompany, '?', ''), '"', ''), ',', ''), ' ', '')) = '';
GO
UPDATE SourceContacts SET SourceJobTitle = NULL  WHERE SourceJobTitle IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(SourceJobTitle, '?', ''), '"', ''), ',', ''), ' ', '')) = '';
GO
UPDATE SourceContacts SET SourceFirstName = NULL WHERE SourceFirstName IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(SourceFirstName, '?', ''), '"', ''), ',', ''), ' ', '')) = '';
GO
UPDATE SourceContacts SET SourceLastName = NULL  WHERE SourceLastName IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(SourceLastName, '?', ''), '"', ''), ',', ''), ' ', '')) = '';
GO

-- Changing fields with non-printing characters only to NULL in Contacts

UPDATE ContactCompany SET Company = NULL    WHERE Company IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(Company, '?', ''), '"', ''), ',', ''), ' ', '')) = '';
GO
UPDATE ContactJobTitle SET JobTitle = NULL  WHERE JobTitle IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(JobTitle, '?', ''), '"', ''), ',', ''), ' ', '')) = '';
GO
UPDATE ContactNames SET FirstName = NULL    WHERE FirstName IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(FirstName, '?', ''), '"', ''), ',', ''), ' ', '')) = '';
GO
UPDATE ContactNames SET LastName = NULL     WHERE LastName IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(LastName, '?', ''), '"', ''), ',', ''), ' ', '')) = '';
GO

-- Setting ContactID for new source contacts if SourceFirstName, SourceLastName, SourceJobTitle, and SourceCompany are the same (FirstName and LastName may be not unique)

INSERT INTO SourceContactID
    ( [Source]
    , SourceID
    , ContactID
    )
SELECT
    s.[Source]
    , s.SourceID
    , v.ContactID
FROM
    SourceContacts s
    INNER JOIN SourceContacts t ON
        t.[Source] = s.[Source]
        AND t.SourceFirstName = s.SourceFirstName
        AND t.SourceLastName = s.SourceLastName
        AND t.SourceJobTitle = s.SourceJobTitle
        AND t.SourceCompany = s.SourceCompany
    LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
    LEFT OUTER JOIN SourceContactID v ON v.[Source] = t.[Source] AND v.SourceID = t.SourceID
WHERE
    t.MergeDate < s.MergeDate
    AND v.ContactID IS NOT NULL    
    AND d.ContactID IS NULL;
GO

-- =====
-- Discovering existing contacts by unique FirstName and LastName in the same source (latest merged data are used)

INSERT INTO SourceContactID
    ( [Source]
    , SourceID
    , ContactID
    )
SELECT
    t.[Source]
    , t.SourceID
    , d.ContactID
FROM
    (
        -- Selects unique FirstName, LastName in the last merged data in every Source
        SELECT
            s.[Source]
            , MAX(s.SourceID) AS SourceID
            , MAX(d.ContactID) AS ContactID
            , s.SourceFirstName
            , s.SourceLastName
        FROM
            SourceContacts s
            INNER JOIN (
                -- Selects the latest MergeDate for each FirstName, LastName in every Source
                SELECT
                    s.[Source]
                    , s.SourceFirstName
                    , s.SourceLastName
                    , MAX(s.MergeDate) AS LastMergeDate
                FROM
                    SourceContacts s
                GROUP BY
                    s.[Source]
                    , s.SourceFirstName
                    , s.SourceLastName
            ) t ON t.[Source] = s.[Source] AND t.SourceFirstName = s.SourceFirstName AND t.SourceLastName = s.SourceLastName
                AND t.LastMergeDate = s.MergeDate
            LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
        GROUP BY
            s.[Source]
            , s.SourceFirstName
            , s.SourceLastName
        HAVING
            COUNT(*) = 1
    ) t
    INNER JOIN SourceContacts s ON t.[Source] = s.[Source] AND t.SourceFirstName = s.SourceFirstName AND t.SourceLastName = s.SourceLastName
        AND t.SourceID <> s.SourceID    -- Source is the same and SourceID is different
    LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
    LEFT OUTER JOIN SourceContactID v ON v.[Source] = t.[Source] AND v.SourceID = t.SourceID
WHERE
    d.ContactID IS NOT NULL
    AND t.ContactID IS NULL
    AND v.SourceID IS NULL;
GO

-- Discovering existing contacts by unique Emails in all sources
-- Existing duplicated emails in Contacts must be resolved manually

INSERT INTO SourceContactID
    ( [Source]
    , SourceID
    , ContactID
    )
SELECT
    s.[Source]
    , s.SourceID
    , t.ContactID
FROM
    (
        -- Selects unique Emails with ContactID
        SELECT
            s.SourceEmail
            , MAX(s.ContactID) AS ContactID
        FROM
            (
            SELECT
                s.SourceEmail
                , s.ContactID
            FROM
                (
                    -- Select Emails and SecondaryEmails with ContactID
                    SELECT
                        s.SourceEmail
                        , d.ContactID
                    FROM
                        SourceContacts s
                        INNER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
                    WHERE
                        d.ContactID IS NOT NULL
                        AND s.SourceEmail IS NOT NULL
                    UNION
                    SELECT
                        s.SourceSecondaryEmail AS SourceEmail
                        , d.ContactID
                    FROM
                        SourceContacts s
                        INNER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
                    WHERE
                        d.ContactID IS NOT NULL
                        AND s.SourceSecondaryEmail IS NOT NULL
                    UNION
                    SELECT
                        s.Email AS SourceEmail
                        , s.ID AS ContactID
                    FROM
                        ContactEmail s
                    WHERE
                        s.Email IS NOT NULL
                ) s
            GROUP BY
                s.SourceEmail
                , s.ContactID
            ) s
        GROUP BY
            s.SourceEmail
        HAVING
            COUNT(*) = 1
    ) t
    INNER JOIN SourceContacts s ON s.SourceEmail = t.SourceEmail
    LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
WHERE
    d.SourceID IS NULL;
GO

INSERT INTO SourceContactID
    ( [Source]
    , SourceID
    , ContactID
    )
SELECT
    s.[Source]
    , s.SourceID
    , t.ContactID
FROM
    (
        -- Selects unique Emails with ContactID
        SELECT
            s.SourceEmail
            , MAX(s.ContactID) AS ContactID
        FROM
            (
            SELECT
                s.SourceEmail
                , s.ContactID
            FROM
                (
                    -- Select Emails and SecondaryEmails with ContactID
                    SELECT
                        s.SourceEmail
                        , d.ContactID
                    FROM
                        SourceContacts s
                        INNER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
                    WHERE
                        d.ContactID IS NOT NULL
                        AND s.SourceEmail IS NOT NULL
                    UNION
                    SELECT
                        s.SourceSecondaryEmail AS SourceEmail
                        , d.ContactID
                    FROM
                        SourceContacts s
                        INNER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
                    WHERE
                        d.ContactID IS NOT NULL
                        AND s.SourceSecondaryEmail IS NOT NULL
                    UNION
                    SELECT
                        s.Email AS SourceEmail
                        , s.ID AS ContactID
                    FROM
                        ContactEmail s
                    WHERE
                        s.Email IS NOT NULL
                ) s
            GROUP BY
                s.SourceEmail
                , s.ContactID
            ) s
        GROUP BY
            s.SourceEmail
        HAVING
            COUNT(*) = 1
    ) t
    INNER JOIN SourceContacts s ON s.SourceSecondaryEmail = t.SourceEmail
    LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
WHERE
    d.SourceID IS NULL;
GO

-- Discovering existing contacts by unique mobile phones in all sources
-- Existing duplicated mobile phones in Contacts must be resolved manually
-- Phone numbers are normalized to digits

INSERT INTO SourceContactID
    ( [Source]
    , SourceID
    , ContactID
    )
SELECT
    s.[Source]
    , s.SourceID
    , t.ContactID
FROM
    SourceContacts s
    INNER JOIN (
        -- Selects unique mobile phones with ContactID
        SELECT
            s.SourceMobile
            , MAX(s.ContactID) AS ContactID
        FROM
            (
            SELECT
                s.SourceMobile
                , s.ContactID
            FROM
                (
                    SELECT
                        REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.SourceMobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '') AS SourceMobile
                        , d.ContactID
                    FROM
                        SourceContacts s
                        LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
                    WHERE
                        d.ContactID IS NOT NULL
                        AND s.SourceMobile IS NOT NULL
                    UNION
                    SELECT
                        REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.Mobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '') AS SourceMobile
                        , s.ID AS ContactID
                    FROM
                        ContactMobile s
                    WHERE
                        s.Mobile IS NOT NULL
                ) s
            GROUP BY
                s.SourceMobile
                , s.ContactID
            ) s
        GROUP BY
            s.SourceMobile
        HAVING
            COUNT(*) = 1
    ) t ON t.SourceMobile = REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.SourceMobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '')
    LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
WHERE
    d.SourceID IS NULL;
GO
-- =====

-- Adding new contacts identified by email (SourceID in Contacts)

INSERT INTO Contacts
    ( SourceID
    )
SELECT
    s.SourceEmail
FROM
    SourceContacts s
    LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
    LEFT OUTER JOIN Contacts c ON c.SourceID = s.SourceEmail
WHERE
    s.IsJunk IS NULL
    AND s.SourceEmail IS NOT NULL
GROUP BY
    s.SourceEmail
HAVING
    MAX(d.ContactID) IS NULL
    AND MAX(c.SourceID) IS NULL;
GO

-- Adding new contacts identified by secondary email (SourceID in Contacts)

INSERT INTO Contacts
    ( SourceID
    )
SELECT
    s.SourceSecondaryEmail
FROM
    SourceContacts s
    LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
    LEFT OUTER JOIN Contacts c ON c.SourceID = s.SourceSecondaryEmail
WHERE
    s.IsJunk IS NULL
    AND s.SourceEmail IS NULL
    AND s.SourceSecondaryEmail IS NOT NULL
GROUP BY
    s.SourceSecondaryEmail
HAVING
    MAX(d.ContactID) IS NULL
    AND MAX(c.SourceID) IS NULL;
GO

-- Adding new contacts identified by email (ID and Email in ContactEmail)

INSERT ContactEmail
    ( ID
    , Email
    )
SELECT
    c.ID
    , s.SourceEmail
FROM
    SourceContacts s
    LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
    INNER JOIN Contacts c ON c.SourceID = s.SourceEmail
    LEFT OUTER JOIN ContactEmail e ON e.ID = c.ID
GROUP BY
    c.ID
    , s.SourceEmail
HAVING
    MAX(e.ID) IS NULL;
GO

-- Adding new contacts identified by secondary email (ID and Email in ContactEmail)

INSERT ContactEmail
    ( ID
    , Email
    )
SELECT
    c.ID
    , s.SourceSecondaryEmail
FROM
    SourceContacts s
    LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
    INNER JOIN Contacts c ON c.SourceID = s.SourceSecondaryEmail
    LEFT OUTER JOIN ContactEmail e ON e.ID = c.ID
GROUP BY
    c.ID
    , s.SourceSecondaryEmail
HAVING
    MAX(e.ID) IS NULL;
GO

-- =====
-- Discovering existing contacts by unique Emails in all sources
-- Existing duplicated emails in Contacts must be resolved manually

INSERT INTO SourceContactID
    ( [Source]
    , SourceID
    , ContactID
    )
SELECT
    s.[Source]
    , s.SourceID
    , t.ContactID
FROM
    (
        -- Selects unique Emails with ContactID
        SELECT
            s.SourceEmail
            , MAX(s.ContactID) AS ContactID
        FROM
            (
            SELECT
                s.SourceEmail
                , s.ContactID
            FROM
                (
                    -- Select Emails and SecondaryEmails with ContactID
                    SELECT
                        s.SourceEmail
                        , d.ContactID
                    FROM
                        SourceContacts s
                        INNER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
                    WHERE
                        d.ContactID IS NOT NULL
                        AND s.SourceEmail IS NOT NULL
                    UNION
                    SELECT
                        s.SourceSecondaryEmail AS SourceEmail
                        , d.ContactID
                    FROM
                        SourceContacts s
                        INNER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
                    WHERE
                        d.ContactID IS NOT NULL
                        AND s.SourceSecondaryEmail IS NOT NULL
                    UNION
                    SELECT
                        s.Email AS SourceEmail
                        , s.ID AS ContactID
                    FROM
                        ContactEmail s
                    WHERE
                        s.Email IS NOT NULL
                ) s
            GROUP BY
                s.SourceEmail
                , s.ContactID
            ) s
        GROUP BY
            s.SourceEmail
        HAVING
            COUNT(*) = 1
    ) t
    INNER JOIN SourceContacts s ON s.SourceEmail = t.SourceEmail
    LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
WHERE
    d.SourceID IS NULL;
GO

INSERT INTO SourceContactID
    ( [Source]
    , SourceID
    , ContactID
    )
SELECT
    s.[Source]
    , s.SourceID
    , t.ContactID
FROM
    (
        -- Selects unique Emails with ContactID
        SELECT
            s.SourceEmail
            , MAX(s.ContactID) AS ContactID
        FROM
            (
            SELECT
                s.SourceEmail
                , s.ContactID
            FROM
                (
                    -- Select Emails and SecondaryEmails with ContactID
                    SELECT
                        s.SourceEmail
                        , d.ContactID
                    FROM
                        SourceContacts s
                        INNER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
                    WHERE
                        d.ContactID IS NOT NULL
                        AND s.SourceEmail IS NOT NULL
                    UNION
                    SELECT
                        s.SourceSecondaryEmail AS SourceEmail
                        , d.ContactID
                    FROM
                        SourceContacts s
                        INNER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
                    WHERE
                        d.ContactID IS NOT NULL
                        AND s.SourceSecondaryEmail IS NOT NULL
                    UNION
                    SELECT
                        s.Email AS SourceEmail
                        , s.ID AS ContactID
                    FROM
                        ContactEmail s
                    WHERE
                        s.Email IS NOT NULL
                ) s
            GROUP BY
                s.SourceEmail
                , s.ContactID
            ) s
        GROUP BY
            s.SourceEmail
        HAVING
            COUNT(*) = 1
    ) t
    INNER JOIN SourceContacts s ON s.SourceSecondaryEmail = t.SourceEmail
    LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
WHERE
    d.SourceID IS NULL;
GO
-- =====

-- Adding new contacts identified by mobile phones with empty emails (SourceID in Contacts)
-- Phone numbers are normalized to digits

INSERT INTO Contacts
    ( SourceID
    )
SELECT
    REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.SourceMobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '') AS SourceMobile2
FROM
    SourceContacts s
    LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
    LEFT OUTER JOIN Contacts c ON c.SourceID = REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.SourceMobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '')
WHERE
    s.IsJunk IS NULL
    AND s.SourceEmail IS NULL
    AND s.SourceSecondaryEmail IS NULL
    AND s.SourceMobile IS NOT NULL
GROUP BY
    REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.SourceMobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '')
HAVING
    MAX(d.ContactID) IS NULL
    AND MAX(c.SourceID) IS NULL;
GO

-- Adding new contacts identified by mobile phones with empty emails (ID and Mobile in ContactMobile)

INSERT ContactMobile
    ( ID
    , Mobile
    )
SELECT
    c.ID
    , MIN(s.SourceMobile) AS Mobile
FROM
    SourceContacts s
    LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
    INNER JOIN Contacts c ON c.SourceID = REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.SourceMobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '')
    LEFT OUTER JOIN ContactMobile m ON m.ID = c.ID
GROUP BY
    c.ID
    , REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.SourceMobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '')
HAVING
    MAX(m.ID) IS NULL;
GO

-- =====
-- Discovering existing contacts by unique mobile phones in all sources
-- Existing duplicated mobile phones in Contacts must be resolved manually
-- Phone numbers are normalized to digits

INSERT INTO SourceContactID
    ( [Source]
    , SourceID
    , ContactID
    )
SELECT
    s.[Source]
    , s.SourceID
    , t.ContactID
FROM
    SourceContacts s
    INNER JOIN (
        -- Selects unique mobile phones with ContactID
        SELECT
            s.SourceMobile
            , MAX(s.ContactID) AS ContactID
        FROM
            (
            SELECT
                s.SourceMobile
                , s.ContactID
            FROM
                (
                    SELECT
                        REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.SourceMobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '') AS SourceMobile
                        , d.ContactID
                    FROM
                        SourceContacts s
                        LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
                    WHERE
                        d.ContactID IS NOT NULL
                        AND s.SourceMobile IS NOT NULL
                    UNION
                    SELECT
                        REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.Mobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '') AS SourceMobile
                        , s.ID AS ContactID
                    FROM
                        ContactMobile s
                    WHERE
                        s.Mobile IS NOT NULL
                ) s
            GROUP BY
                s.SourceMobile
                , s.ContactID
            ) s
        GROUP BY
            s.SourceMobile
        HAVING
            COUNT(*) = 1
    ) t ON t.SourceMobile = REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.SourceMobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '')
    LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
WHERE
    d.SourceID IS NULL;
GO
-- =====

-- Updating LinkedIn API contacts
-- LinkedIn API data do not contain emails
-- Data are linked using first and last names if the names are unique in LinkedIn CSV data

INSERT INTO SourceContactID
    ( [Source]
    , SourceID
    , ContactID
    )
SELECT
    s.[Source]
    , s.SourceID
    , t.ContactID
FROM
    SourceContacts s
    INNER JOIN (
        SELECT
            s.SourceFirstName
            , s.SourceLastName
            , MAX(d.ContactID) AS ContactID
        FROM
            SourceContacts s
            LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
        WHERE
            s.[Source] = 'LinkedInCSV'
        GROUP BY
            s.SourceFirstName
            , s.SourceLastName
        HAVING
            COUNT(*) = 1
    ) t ON t.SourceFirstName = s.SourceFirstName AND t.SourceLastName = s.SourceLastName
    LEFT OUTER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
WHERE
    s.[Source] = 'LinkedIn'
    AND t.ContactID IS NOT NULL
    AND d.SourceID IS NULL;
GO

-- Inserting names for new contacts

INSERT ContactNames (ID, FirstName, LastName)
SELECT
    d.ContactID
    , MIN(s.SourceFirstName) AS FirstName
    , MIN(s.SourceLastName) AS LastName
FROM
    SourceContacts s
    INNER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
    LEFT OUTER JOIN ContactNames c ON c.ID = d.ContactID
WHERE
    s.SourceFirstName IS NOT NULL
GROUP BY
    d.ContactID
HAVING
    MAX(c.ID) IS NULL
    AND d.ContactID IS NOT NULL;
GO

-- Inserting mobile numbers if not exist

INSERT ContactMobile (ID, Mobile)
SELECT
    d.ContactID
    , MIN(s.SourceMobile) AS Mobile
FROM
    SourceContacts s
    INNER JOIN SourceContactID d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID
    LEFT OUTER JOIN ContactMobile c ON c.ID = d.ContactID
WHERE
    s.SourceMobile IS NOT NULL
GROUP BY
    d.ContactID
HAVING
    MAX(c.ID) IS NULL;
GO

-- Inserting new countries from SourceContacts

INSERT Countries
    (CountryCode, CountryName)
SELECT
    s.SourceCountryCode AS CountryCode
    , MIN(t.SourceLocationName) AS CountryName              -- Min LocationName for all with min length
FROM
    (
        SELECT
            s.SourceCountryCode
            , MIN(LEN(s.SourceLocationName)) AS [Length]    -- Min LocationName length
        FROM
            SourceContacts s
        WHERE
            s.SourceCountryCode IS NOT NULL
        GROUP BY
            s.SourceCountryCode                             -- All country codes
    ) s

    INNER JOIN (
        SELECT
            s.SourceCountryCode
            , s.SourceLocationName                          -- Default country name as LocationName
            , LEN(s.SourceLocationName) AS [Length]         -- with minimal length
        FROM
            SourceContacts s
        GROUP BY
            s.SourceCountryCode
            , s.SourceLocationName
    ) t ON t.SourceCountryCode = s.SourceCountryCode AND t.[Length] = s.[Length]

    LEFT OUTER JOIN Countries c ON c.CountryCode = s.SourceCountryCode
WHERE
    c.CountryCode IS NULL
GROUP BY
    s.SourceCountryCode;
GO

-- Inserting new domains into Websites from SourceContacts if domains have 3 or more contacts

INSERT Websites
    (Domain, Website)
SELECT
    c.Domain AS Domain
    , 'www.' + c.Domain AS Website
FROM
    (
        SELECT
            LOWER(SUBSTRING(c.SourceEmail, CHARINDEX('@', c.SourceEmail) + 1, 255)) AS Domain
        FROM
            SourceContacts c
        WHERE
            c.SourceEmail IS NOT NULL
            AND CHARINDEX('@', c.SourceEmail) > 1
        GROUP BY
            LOWER(SUBSTRING(c.SourceEmail, CHARINDEX('@', c.SourceEmail) + 1, 255))
        HAVING
            COUNT(*) > 2
    ) c
    LEFT OUTER JOIN Websites w ON w.Domain = c.Domain
WHERE
    w.Domain IS NULL;
GO

-- Inserting  new domains into Websites from Contacts if domains have 3 or more contacts

INSERT Websites
    (Domain, Website)
SELECT
    c.Domain AS Domain
    , 'www.' + c.Domain AS Website
FROM
    (
        SELECT
            LOWER(SUBSTRING(c.Email, CHARINDEX('@', c.Email) + 1, 255)) AS Domain
        FROM
            ContactEmail c
        WHERE
            c.Email IS NOT NULL
            AND CHARINDEX('@', c.Email) > 1
        GROUP BY
            LOWER(SUBSTRING(c.Email, CHARINDEX('@', c.Email) + 1, 255))
        HAVING
            COUNT(*) > 2
    ) c
    LEFT OUTER JOIN Websites w ON w.Domain = c.Domain
WHERE
    w.Domain IS NULL;
GO
