-- Social Contact Database for Microsoft SQL Server
-- July 15, 2014

-- This SQL code must be executed after SaveToDB Framework install

CREATE SCHEMA [etl71]
GO
CREATE SCHEMA [dbo71]
GO
CREATE SCHEMA [xls71]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo71].[Contacts](
    [ID] [int] IDENTITY(1,1) NOT NULL,
    [ContactStatus] [tinyint] NULL,
    [ContactCategory] [nvarchar](255) NULL,
    [FirstName] [nvarchar](255) NULL,
    [LastName] [nvarchar](255) NULL,
    [Email] [nvarchar](255) NULL,
    [Mobile] [nvarchar](255) NULL,
    [Phone] [nvarchar](255) NULL,
    [Country] [nvarchar](255) NULL,
    [Website] [nvarchar](255) NULL,
    [Company] [nvarchar](255) NULL,
    [JobTitle] [nvarchar](255) NULL,
    [String1] [nvarchar](255) NULL,
    [String2] [nvarchar](255) NULL,
    [String3] [nvarchar](255) NULL,
    [Number1] [float] NULL,
    [Number2] [float] NULL,
    [Number3] [float] NULL,
 CONSTRAINT [PK_Contacts_dbo71] PRIMARY KEY CLUSTERED 
(
    [ID] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]

GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO
CREATE TABLE [dbo71].[Countries](
    [CountryCode] [char](2) NOT NULL,
    [CountryName] [nvarchar](255) NULL,
    [Checked] [tinyint] NULL,
 CONSTRAINT [PK_Countries_dbo71] PRIMARY KEY CLUSTERED 
(
    [CountryCode] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY],
 CONSTRAINT [IX_Countries_CountryName] UNIQUE NONCLUSTERED 
(
    [CountryName] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]

GO
SET ANSI_PADDING OFF
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO
CREATE TABLE [dbo71].[SourceContacts](
    [Source] [nvarchar](50) NOT NULL,
    [SourceID] [nvarchar](50) NOT NULL,
    [MergeDate] [datetime] NULL,
    [IsJunk] [tinyint] NULL,
    [ContactID] [int] NULL,
    [SourceFirstName] [nvarchar](255) NULL,
    [SourceLastName] [nvarchar](255) NULL,
    [SourceEmail] [nvarchar](255) NULL,
    [SourceSecondaryEmail] [nvarchar](255) NULL,
    [SourceMobile] [nvarchar](255) NULL,
    [SourcePhone] [nvarchar](255) NULL,
    [SourceCompany] [nvarchar](255) NULL,
    [SourceJobTitle] [nvarchar](255) NULL,
    [SourceHeadline] [nvarchar](255) NULL,
    [SourceIndustry] [nvarchar](255) NULL,
    [SourceLocationName] [nvarchar](255) NULL,
    [SourceCountryCode] [char](2) NULL,
    [SourceProfileUrl] [nvarchar](255) NULL,
 CONSTRAINT [PK_SourceContacts_dbo71] PRIMARY KEY CLUSTERED 
(
    [Source] ASC,
    [SourceID] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]

GO
SET ANSI_PADDING OFF
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo71].[Websites](
    [ID] [smallint] IDENTITY(1,1) NOT NULL,
    [Domain] [nvarchar](255) NULL,
    [Website] [nvarchar](255) NULL,
    [Comment] [nvarchar](255) NULL,
    [Checked] [tinyint] NULL,
 CONSTRAINT [PK_ChangeWebsites_dbo71] PRIMARY KEY CLUSTERED 
(
    [ID] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY],
 CONSTRAINT [IX_ChangeWebsites_Domain_dbo71] UNIQUE NONCLUSTERED 
(
    [Domain] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]

GO
ALTER TABLE [dbo71].[SourceContacts]  WITH CHECK ADD  CONSTRAINT [FK_SourceContacts_Contacts_dbo71] FOREIGN KEY([ContactID])
REFERENCES [dbo71].[Contacts] ([ID])
ON UPDATE CASCADE
ON DELETE SET NULL
GO
ALTER TABLE [dbo71].[SourceContacts] CHECK CONSTRAINT [FK_SourceContacts_Contacts_dbo71]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     1.0, 2013-12-13
-- Description: Discovers existing contacts
-- =============================================

CREATE PROCEDURE [etl71].[uspDiscoverExistingContacts]
AS
BEGIN

-- Discovering existing contacts by unique FirstName and LastName in the same source
-- FirstName and LastName are equals to existing contact, and FirstName and LastName are unique in the last merged data in a one source
-- Discovers changes of email, mobile phones, and other data used as SourceID
-- Unique FirstName and LastName in Contacts is not exact criteria as new persons can be in new data

UPDATE dbo71.SourceContacts
SET
    ContactID = d.ContactID
FROM
    dbo71.SourceContacts s
    INNER JOIN (
        -- Selects new contacts with ContactID from contacts with the same unique FirstName, LastName
        SELECT
            d.[Source]
            , d.SourceID
            , s.ContactID
        FROM
            (
                -- Selects unique FirstName, LastName in the last merged data of each Source
                SELECT
                    s.[Source]
                    , MAX(s.SourceID) AS SourceID
                    , MAX(s.ContactID) AS ContactID
                    , s.SourceFirstName
                    , s.SourceLastName
                FROM
                    dbo71.SourceContacts s
                    INNER JOIN (
                        -- Selects LastMergeDate for each FirstName, LastName in each Source
                        SELECT
                            s.[Source]
                            , s.SourceFirstName
                            , s.SourceLastName
                            , MAX(s.MergeDate) AS LastMergeDate
                        FROM
                            dbo71.SourceContacts s
                        GROUP BY
                            s.[Source]
                            , s.SourceFirstName
                            , s.SourceLastName
                    ) d ON d.[Source] = s.[Source] AND d.SourceFirstName = s.SourceFirstName AND d.SourceLastName = s.SourceLastName
                        AND d.LastMergeDate = s.MergeDate
                GROUP BY
                    s.[Source]
                    , s.SourceFirstName
                    , s.SourceLastName
                HAVING
                    COUNT(*) = 1
            ) d
            INNER JOIN dbo71.SourceContacts s ON d.[Source] = s.[Source] AND d.SourceFirstName = s.SourceFirstName AND d.SourceLastName = s.SourceLastName
                AND d.SourceID <> s.SourceID
        WHERE
            s.ContactID IS NOT NULL
            AND d.ContactID IS NULL
    ) d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID


-- Discovering existing contacts by unique Emails in all sources
-- This code prevents email duplication in Contacts
-- Existing duplicated emails in Contacts must be resolved manually

UPDATE dbo71.SourceContacts
SET
    ContactID = d.ContactID
FROM
    dbo71.SourceContacts s
    INNER JOIN (
        -- Selects unique Emails with ContactID
        SELECT
            s.SourceEmail
            , MAX(s.ContactID) AS ContactID
        FROM
            (
                -- Select Emails and SecondaryEmails with ContactID
                SELECT
                    s.SourceEmail
                    , s.ContactID
                FROM
                    dbo71.SourceContacts s
                WHERE
                    s.ContactID IS NOT NULL
                    AND s.SourceEmail IS NOT NULL
                UNION
                SELECT
                    s.SourceSecondaryEmail AS SourceEmail
                    , s.ContactID
                FROM
                    dbo71.SourceContacts s
                WHERE
                    s.ContactID IS NOT NULL
                    AND s.SourceSecondaryEmail IS NOT NULL
                UNION
                SELECT
                    s.Email AS SourceEmail
                    , s.ID AS ContactID
                FROM
                    dbo71.Contacts s
                WHERE
                    s.Email IS NOT NULL
            ) s
        GROUP BY
            s.SourceEmail
        HAVING
            COUNT(DISTINCT s.ContactID) = 1
    ) d ON d.SourceEmail = s.SourceEmail OR d.SourceEmail = s.SourceSecondaryEmail
WHERE
    s.ContactID IS NULL
        
-- Discovering existing contacts by unique mobile phones in all sources
-- This code prevents mobile phone duplication in Contacts
-- Existing duplicated mobile phones in Contacts must be resolved manually
-- Phone numbers are normalized to digits

UPDATE dbo71.SourceContacts
SET
    ContactID = d.ContactID
FROM
    dbo71.SourceContacts s
    INNER JOIN (
        -- Selects unique mobile phones with ContactID
        SELECT
            s.SourceMobile
            , MAX(s.ContactID) AS ContactID
        FROM
            (
                SELECT
                    REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.SourceMobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '') AS SourceMobile
                    , s.ContactID
                FROM                    
                    dbo71.SourceContacts s
                WHERE
                    s.ContactID IS NOT NULL
                    AND s.SourceMobile IS NOT NULL
                UNION
                SELECT
                    REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.Mobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '') AS SourceMobile
                    , s.ID AS ContactID
                FROM                    
                    dbo71.Contacts s
                WHERE
                    s.Mobile IS NOT NULL
            ) s
        GROUP BY
            s.SourceMobile
        HAVING
            COUNT(DISTINCT s.ContactID) = 1
    ) d ON d.SourceMobile = REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.SourceMobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '')
WHERE
    s.ContactID IS NULL

END


GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     1.0, 2013-12-13
-- Description: Updating contacts
-- =============================================

CREATE PROCEDURE [etl71].[uspUpdateContacts]
AS
BEGIN

SET NOCOUNT ON

SET ANSI_WARNINGS OFF

-- ============================================================================
-- Clearing incorrect data
-- ============================================================================

-- Changing empty fields to NULL in SourceContacts

UPDATE dbo71.SourceContacts SET SourceCompany = NULL   WHERE SourceCompany IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(SourceCompany, '?', ''), '"', ''), ',', ''), ' ', '')) = ''
UPDATE dbo71.SourceContacts SET SourceJobTitle = NULL  WHERE SourceJobTitle IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(SourceJobTitle, '?', ''), '"', ''), ',', ''), ' ', '')) = ''
UPDATE dbo71.SourceContacts SET SourceFirstName = NULL WHERE SourceFirstName IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(SourceFirstName, '?', ''), '"', ''), ',', ''), ' ', '')) = ''
UPDATE dbo71.SourceContacts SET SourceLastName = NULL  WHERE SourceLastName IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(SourceLastName, '?', ''), '"', ''), ',', ''), ' ', '')) = ''

-- Changing empty fields to NULL in Contacts

UPDATE dbo71.Contacts SET Company = NULL    WHERE Company IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(Company, '?', ''), '"', ''), ',', ''), ' ', '')) = ''
UPDATE dbo71.Contacts SET JobTitle = NULL   WHERE JobTitle IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(JobTitle, '?', ''), '"', ''), ',', ''), ' ', '')) = ''
UPDATE dbo71.Contacts SET FirstName = NULL  WHERE FirstName IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(FirstName, '?', ''), '"', ''), ',', ''), ' ', '')) = ''
UPDATE dbo71.Contacts SET LastName = NULL   WHERE LastName IS NOT NULL AND RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(LastName, '?', ''), '"', ''), ',', ''), ' ', '')) = ''

-- ============================================================================
-- Linking and inserting new contacts from SourceContacts
-- ============================================================================

-- Setting ContactID for new source contacts if FirstName, LastName, JobTitle, and Company are the same

UPDATE dbo71.SourceContacts
SET
    ContactID = t.ContactID
FROM
    dbo71.SourceContacts s
    INNER JOIN (
        SELECT
            s.[Source]
            , s.SourceID
            , t.ContactID
        FROM
            dbo71.SourceContacts s
            INNER JOIN dbo71.SourceContacts t ON t.SourceFirstName = s.SourceFirstName AND t.SourceLastName = s.SourceLastName
                AND t.[Source] = s.[Source]
        WHERE
            t.MergeDate < s.MergeDate
            AND t.SourceJobTitle = s.SourceJobTitle
            AND t.SourceCompany = s.SourceCompany
            AND s.ContactID IS NULL
    ) t ON t.[Source] = s.[Source] AND t.[SourceID] = s.[SourceID]

-- Discovering existing contacts

EXEC etl71.uspDiscoverExistingContacts

-- Adding new contacts identified by email
-- The first available source is used to add a contact with unique Email

INSERT INTO dbo71.Contacts
    ( ContactStatus
    , FirstName
    , LastName
    , Email
    , Mobile
    , Phone
    , Company
    , JobTitle
    )
SELECT
    2 AS ContactStatus                  -- New record to verify and change the status to 3 or 0
    , s.SourceFirstName AS FirstName
    , s.SourceLastName AS LastName
    , s.SourceEmail AS Email
    , s.SourceMobile AS Mobile
    , s.SourcePhone AS Phone
    , s.SourceCompany AS Company
    , s.SourceJobTitle AS JobTitle
FROM
    dbo71.SourceContacts s
    INNER JOIN (
        -- Selects unique min Source and SourceID for new contacts where SourceID = SourceEmail
        SELECT
            MIN(s.[Source]) AS [Source]
            , s.SourceID
        FROM
            dbo71.SourceContacts s
        WHERE
            s.ContactID IS NULL
            AND s.IsJunk IS NULL
            AND s.SourceEmail IS NOT NULL
        GROUP BY
            s.SourceID
    ) d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID

-- Discovering existing contacts after adding new contacts by Email

EXEC etl71.uspDiscoverExistingContacts

-- Adding new contacts identified by mobile phone
-- The first available source is used to add a contact with a unique mobile phone and an empty Email
-- Phone numbers are normalized to digits

INSERT INTO dbo71.Contacts
    ( ContactStatus
    , FirstName
    , LastName
    , Email
    , Mobile
    , Phone
    , Company
    , JobTitle
    )
SELECT
    2 AS ContactStatus                  -- New record to verify and change the status to 3 or 0
    , s.SourceFirstName AS FirstName
    , s.SourceLastName AS LastName
    , s.SourceEmail AS Email
    , s.SourceMobile AS Mobile
    , s.SourcePhone AS Phone
    , s.SourceCompany AS Company
    , s.SourceJobTitle AS JobTitle
FROM
    dbo71.SourceContacts s
    INNER JOIN (
        -- Selects Source and SourceID for mobile phone
        SELECT
            s.[Source]
            , MIN(s.SourceID) AS SourceID
            , s.SourceMobile
        FROM
            dbo71.SourceContacts s
            INNER JOIN (
                -- Selects min source for mobile phone
                SELECT
                    MIN(s.[Source]) AS [Source]
                    , REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.SourceMobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '') AS SourceMobile
                FROM
                    dbo71.SourceContacts s
                WHERE
                    s.ContactID IS NULL
                    AND s.IsJunk IS NULL
                    AND s.SourceEmail IS NULL
                    AND s.SourceMobile IS NOT NULL
                GROUP BY
                    REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.SourceMobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '')
            ) d ON d.[Source] = s.[Source] AND d.SourceMobile = REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(s.SourceMobile, '-', ''), '.', ''), '(', ''), ')', ''), ' ', ''), '+', '')
        GROUP BY
            s.[Source]
            , s.SourceMobile
    ) d ON d.[Source] = s.[Source] AND d.SourceID = s.SourceID

-- Discovering existing contacts after adding new contacts by mobile phone

EXEC etl71.uspDiscoverExistingContacts

-- ============================================================================
-- Updating LinkedIn API contacts
-- ============================================================================

-- LinkedIn API data do not contain emails
-- Data are linked using first and last names if the names are unique in LinkedIn CSV data

UPDATE dbo71.SourceContacts
SET
    ContactID = l.ContactID
FROM
    dbo71.SourceContacts s
    INNER JOIN (
        SELECT
            l.SourceFirstName
            , l.SourceLastName
            , MAX(l.ContactID) AS ContactID
        FROM
            dbo71.SourceContacts l
        WHERE
            l.[Source] = 'LinkedInCSV'
        GROUP BY
            l.SourceFirstName
            , l.SourceLastName
        HAVING
            COUNT(*) = 1
    ) l ON l.SourceFirstName = s.SourceFirstName AND l.SourceLastName = s.SourceLastName
WHERE
    s.[Source] = 'LinkedIn'
    AND s.ContactID IS NULL

-- ============================================================================
-- Updating phones
-- ============================================================================

UPDATE dbo71.Contacts
SET
    ContactStatus = 2
    , Phone = s.SourcePhone
FROM
    dbo71.Contacts c
    INNER JOIN dbo71.SourceContacts s ON s.ContactID = c.ID
WHERE
    c.Phone IS NULL
    AND s.SourcePhone IS NOT NULL

-- Updating mobile phones

UPDATE dbo71.Contacts
SET
    ContactStatus = 2
    , Mobile = s.SourceMobile
FROM
    dbo71.Contacts c
    INNER JOIN dbo71.SourceContacts s ON s.ContactID = c.ID
WHERE
    c.Mobile IS NULL
    AND s.SourceMobile IS NOT NULL

-- ============================================================================
-- Inserting new countries into the Countries table
-- ============================================================================

-- Inserting new countries from SourceContacts

INSERT dbo71.Countries
    (CountryCode, CountryName)
SELECT
    s.SourceCountryCode AS CountryCode
    , MIN(t.SourceLocationName) AS CountryName              -- Min LocationName for all with min length
FROM
    (
        SELECT
            s.SourceCountryCode
            , MIN(LEN(s.SourceLocationName)) AS [Length]    -- Min LocationName length
        FROM
            dbo71.SourceContacts s
        WHERE
            s.SourceCountryCode IS NOT NULL
        GROUP BY
            s.SourceCountryCode                             -- All country codes
    ) s

    INNER JOIN (
        SELECT
            s.SourceCountryCode
            , s.SourceLocationName                          -- Default country name as LocationName
            , LEN(s.SourceLocationName) AS [Length]         -- with minimal length
        FROM
            dbo71.SourceContacts s
        GROUP BY
            s.SourceCountryCode
            , s.SourceLocationName
    ) t ON t.SourceCountryCode = s.SourceCountryCode AND t.[Length] = s.[Length]

    LEFT OUTER JOIN dbo71.Countries c ON c.CountryCode = s.SourceCountryCode
WHERE
    c.CountryCode IS NULL
GROUP BY
    s.SourceCountryCode

-- ============================================================================
-- Updating contact countries
-- ============================================================================

-- Updating contact Country

UPDATE dbo71.Contacts
SET
    Country = t.CountryName
FROM
    dbo71.Contacts c
    INNER JOIN dbo71.SourceContacts s ON s.ContactID = c.ID
    INNER JOIN dbo71.Countries t ON t.CountryCode = s.SourceCountryCode
WHERE
    c.Country IS NULL

-- ============================================================================
-- Inserting new domains into the Websites table
-- ============================================================================

-- Inserting websites from SourceContacts

INSERT dbo71.Websites
    (Domain, Website)
SELECT
    c.Domain AS Domain
    , 'www.' + c.Domain AS Website
FROM
    (
        SELECT
            LOWER(SUBSTRING(c.SourceEmail, CHARINDEX('@', c.SourceEmail) + 1, 255)) AS Domain
        FROM
            dbo71.SourceContacts c
        WHERE
            c.SourceEmail IS NOT NULL
            AND CHARINDEX('@', c.SourceEmail) > 1
        GROUP BY
            LOWER(SUBSTRING(c.SourceEmail, CHARINDEX('@', c.SourceEmail) + 1, 255))
        HAVING
            COUNT(*) > 2
    ) c
    LEFT OUTER JOIN dbo71.Websites w ON w.Domain = c.Domain
WHERE
    w.ID IS NULL

-- Inserting websites from Contacts

INSERT dbo71.Websites
    (Domain, Website)
SELECT
    c.Domain AS Domain
    , 'www.' + c.Domain AS Website
FROM
    (
        SELECT
            LOWER(SUBSTRING(c.Email, CHARINDEX('@', c.Email) + 1, 255)) AS Domain
        FROM
            dbo71.Contacts c
        WHERE
            c.Email IS NOT NULL
            AND CHARINDEX('@', c.Email) > 1
        GROUP BY
            LOWER(SUBSTRING(c.Email, CHARINDEX('@', c.Email) + 1, 255))
        HAVING
            COUNT(*) > 2
    ) c
    LEFT OUTER JOIN dbo71.Websites w ON w.Domain = c.Domain
WHERE
    w.ID IS NULL

-- ============================================================================
-- Updating contact websites
-- ============================================================================

-- Updating contact websites by domain if websites are not in the Websites table

UPDATE dbo71.Contacts
SET
    ContactStatus = 2
    , Website = 'www.' + LOWER(SUBSTRING(c.email, CHARINDEX('@', c.email) + 1, 255))
FROM
    dbo71.Contacts c
    LEFT OUTER JOIN dbo71.Websites w ON w.Domain = LOWER(SUBSTRING(c.email, CHARINDEX('@', c.email) + 1, 255))
WHERE
    c.Website IS NULL           -- Empty website
    AND c.Email IS NOT NULL
    AND w.ID IS NULL            -- Website not in Websites

-- Updating contact websites as domains from the Websites table

UPDATE dbo71.Contacts
SET
    ContactStatus = 2
    , Website = w.Website
FROM
    dbo71.Contacts c
    INNER JOIN dbo71.Websites w ON w.Domain = LOWER(SUBSTRING(c.email, CHARINDEX('@', c.email) + 1, 255))
WHERE
    (c.Website IS NULL OR c.Website = 'www.' + w.Domain)    -- Default website by domain
    AND COALESCE(c.Website, '') <> COALESCE(w.Website, '')

END

GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     1.0, 2013-12-13
-- Description: Contacts for country
-- =============================================

CREATE PROCEDURE [xls71].[uspContactsByCountryName]
    @CountryName nvarchar(255) = NULL
AS
BEGIN

SELECT
    c.*
FROM
    xls71.viewContacts c
WHERE
    c.Country = @CountryName
END

GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     1.0, 2013-12-13
-- Description: Contacts for domain
-- =============================================

CREATE PROCEDURE [xls71].[uspContactsByDomain]
    @Domain nvarchar(255) = NULL
AS
BEGIN

DECLARE @Website nvarchar(255)

SELECT @Website = Website FROM dbo71.Websites WHERE Domain = @Domain

SELECT
    c.*
FROM
    xls71.viewContacts c
WHERE
    c.Domain = @Domain
    OR c.Website = @Website

END
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     1.0, 2013-12-13
-- Description: Updating contact ContactStatus as confirmed (3)
-- =============================================

CREATE PROCEDURE [xls71].[uspExcelEvent_Actions_dbo71_Contacts_SetAsConfirmed]
AS
BEGIN

UPDATE dbo71.Contacts
SET
    ContactStatus = 3
WHERE
    ContactStatus = 2

END
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     1.0, 2013-12-13
-- Description: Sets Checked state to 1
-- =============================================

CREATE PROCEDURE [xls71].[uspExcelEvent_Actions_dbo71_Countries_SetChecked]
AS
BEGIN

UPDATE dbo71.Countries
SET
    Checked = 1
WHERE
    Checked IS NULL

END
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     1.0, 2013-12-13
-- Description: Sets Checked state to 1
-- =============================================

CREATE PROCEDURE [xls71].[uspExcelEvent_Actions_dbo71_Websites_SetChecked]
AS
BEGIN

UPDATE dbo71.Websites
SET
    Checked = 1
WHERE
    Checked IS NULL

END
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     1.0, 2013-12-13
-- Description: Source contacts for contact ID
-- =============================================

CREATE PROCEDURE [xls71].[uspSourceContactsByContactID]
    @ID int = NULL
AS
BEGIN

SELECT
    s.*
FROM
    xls71.viewSourceContacts s
WHERE
    s.ContactID = @ID
    OR (s.ContactID IS NULL AND @ID IS NULL)
UNION ALL
SELECT
    s.*
FROM
    xls71.viewSourceContacts s
    INNER JOIN dbo71.Contacts c ON c.LastName = s.SourceLastName
WHERE
    s.ContactID IS NULL
    AND c.ID = @ID

END
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     1.0, 2013-12-13
-- Description: Contacts view
-- =============================================

CREATE VIEW [xls71].[viewContacts]
AS

SELECT
     c.ID
    , c.ContactStatus
    , CASE WHEN dn.FirstName IS NULL THEN NULL ELSE DoubleNameCount END AS IsDoubleName
    , c.ContactCategory
    , c.FirstName
    , c.LastName
    , c.Email
    , c.Mobile
    , c.Phone
    , c.Country
    , LOWER(SUBSTRING(c.email, CHARINDEX('@', c.email) + 1, 255)) AS Domain
    , c.Website
    , c.Company
    , c.JobTitle
    , l2.SourceHeadline AS LinkedInHeadline
    , l2.SourceIndustry AS Industry
    , l2.SourceLocationName AS LocationName
    , l2.SourceProfileUrl AS LinkedInProfileUrl
    , g2.SourceProfileUrl AS GooglePlusProfileUrl
    , c.String1
    , c.String2
    , c.String3
    , c.Number1
    , c.Number2
    , c.Number3
FROM
    dbo71.Contacts c

    LEFT OUTER JOIN (
        SELECT
            c.FirstName
            , c.LastName
            , COUNT(*) AS DoubleNameCount
        FROM
            dbo71.Contacts c
        WHERE
            c.FirstName IS NOT NULL
        GROUP BY
            c.FirstName
            , c.LastName
        HAVING
            COUNT(*) > 1
    ) dn ON dn.FirstName = c.FirstName AND dn.LastName = c.LastName

    LEFT OUTER JOIN (
        SELECT
            s.[Source]
            , MIN(s.[SourceID]) AS SourceID
            , s.ContactID
        FROM
            dbo71.SourceContacts s
        WHERE
            s.[Source] = 'LinkedIn'
        GROUP BY
            s.[Source]
            , s.ContactID
    ) s2 ON s2.ContactID = c.ID

    LEFT OUTER JOIN dbo71.SourceContacts l2 ON l2.[Source] = s2.[Source] AND l2.SourceID = s2.SourceID

    LEFT OUTER JOIN (
        SELECT
            s.ContactID
            , MIN(s.SourceProfileUrl) AS SourceProfileUrl
        FROM
            dbo71.SourceContacts s
        WHERE
            s.[Source] = 'GoogleCSV'
            AND s.SourceProfileUrl IS NOT NULL
        GROUP BY
            s.ContactID
    ) g2 ON g2.ContactID = c.ID

GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     1.0, 2013-12-13
-- Description: Countries view
-- =============================================

CREATE VIEW [xls71].[viewCountries]
AS

SELECT
    c.*
    , l.ContactCount
    , CASE WHEN c.Checked IS NULL THEN 1 ELSE NULL END AS HasErrors
FROM
    dbo71.Countries c
    LEFT OUTER JOIN (
        SELECT
            c.Country
            , COUNT(*) AS ContactCount
        FROM
            dbo71.Contacts c
        GROUP BY
            c.Country
    ) l ON l.Country = c.CountryName

GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     1.0, 2013-12-13
-- Description: SourceContacts view
-- =============================================

CREATE VIEW [xls71].[viewSourceContacts]
AS

SELECT
    s.*
FROM
    dbo71.SourceContacts s

GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     1.0, 2013-12-13
-- Description: Websites view
-- =============================================

CREATE VIEW [xls71].[viewWebsites]
AS

SELECT
    w.*
    , s.SourceDomainContactCount
    , c.DomainContactCount
    , t.WebsiteContactCount
    , t.ContactCategoryCount
    , CASE WHEN t.CompanyCount > 1 THEN 1 WHEN t.ContactCategoryCount > 1 THEN 1 WHEN w.Checked IS NULL THEN 1 ELSE NULL END AS HasErrors
FROM
    dbo71.Websites w

    LEFT OUTER JOIN (
        SELECT
            LOWER(SUBSTRING(c.SourceEmail, CHARINDEX('@', c.SourceEmail) + 1, 255)) AS Domain
            , COUNT(*) AS SourceDomainContactCount
        FROM
            dbo71.SourceContacts c
        GROUP BY
            LOWER(SUBSTRING(c.SourceEmail, CHARINDEX('@', c.SourceEmail) + 1, 255))
    ) s ON s.Domain = w.Domain

    LEFT OUTER JOIN (
        SELECT
            LOWER(SUBSTRING(c.Email, CHARINDEX('@', c.Email) + 1, 255)) AS Domain
            , COUNT(*) AS DomainContactCount
        FROM
            dbo71.Contacts c
        GROUP BY
            LOWER(SUBSTRING(c.Email, CHARINDEX('@', c.Email) + 1, 255))
    ) c ON c.Domain = w.Domain

    LEFT OUTER JOIN (
        SELECT
            c.Website
            , COUNT(*) AS WebsiteContactCount
            , COUNT(DISTINCT c.Company) AS CompanyCount
            , COUNT(DISTINCT c.ContactCategory) AS ContactCategoryCount
        FROM
            dbo71.Contacts c
        GROUP BY
            c.Website
    ) t ON t.Website = w.Website

GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     1.0, 2013-12-13
-- Description: Exporting contacts to Outlook CSV
-- =============================================

CREATE VIEW [xls71].[viewExportOutlookCSV]
AS

SELECT
    NULL AS [Title]
    , c.FirstName AS [First Name]
    , NULL AS [Middle Name]
    , c.LastName AS [Last Name]
    , NULL AS [Suffix]
    , c.Company AS [Company]
    , NULL AS [Department]
    , c.JobTitle AS [Job Title]
    , NULL AS [Business Street]
    , NULL AS [Business Street 2]
    , NULL AS [Business Street 3]
    , NULL AS [Business City]
    , NULL AS [Business State]
    , NULL AS [Business Postal Code]
    , c.Country AS [Business Country/Region]
    , NULL AS [Home Street]
    , NULL AS [Home Street 2]
    , NULL AS [Home Street 3]
    , NULL AS [Home City]
    , NULL AS [Home State]
    , NULL AS [Home Postal Code]
    , NULL AS [Home Country/Region]
    , NULL AS [Other Street]
    , NULL AS [Other Street 2]
    , NULL AS [Other Street 3]
    , NULL AS [Other City]
    , NULL AS [Other State]
    , NULL AS [Other Postal Code]
    , NULL AS [Other Country/Region]
    , NULL AS [Assistant''s Phone]
    , NULL AS [Business Fax]
    , c.Phone AS [Business Phone]
    , NULL AS [Business Phone 2]
    , NULL AS [Callback]
    , NULL AS [Car Phone]
    , NULL AS [Company Main Phone]
    , NULL AS [Home Fax]
    , NULL AS [Home Phone]
    , NULL AS [Home Phone 2]
    , NULL AS [ISDN]
    , c.Mobile AS [Mobile Phone]
    , NULL AS [Other Fax]
    , NULL AS [Other Phone]
    , NULL AS [Pager]
    , NULL AS [Primary Phone]
    , NULL AS [Radio Phone]
    , NULL AS [TTY/TDD Phone]
    , NULL AS [Telex]
    , NULL AS [Account]
    , NULL AS [Anniversary]
    , NULL AS [Assistant''s Name]
    , NULL AS [Billing Information]
    , NULL AS [Birthday]
    , NULL AS [Business Address PO Box]
    , NULL AS [Categories]
    , NULL AS [Children]
    , NULL AS [Directory Server]
    , c.Email AS [E-mail Address]
    , NULL AS [E-mail Type]
    , NULL AS [E-mail Display Name]
    , NULL AS [E-mail 2 Address]
    , NULL AS [E-mail 2 Type]
    , NULL AS [E-mail 2 Display Name]
    , NULL AS [E-mail 3 Address]
    , NULL AS [E-mail 3 Type]
    , NULL AS [E-mail 3 Display Name]
    , NULL AS [Gender]
    , NULL AS [Government ID Number]
    , NULL AS [Hobby]
    , NULL AS [Home Address PO Box]
    , NULL AS [Initials]
    , NULL AS [Internet Free Busy]
    , NULL AS [Keywords]
    , NULL AS [Language]
    , NULL AS [Location]
    , NULL AS [Manager''s Name]
    , NULL AS [Mileage]
    , NULL AS [Notes]
    , NULL AS [Office Location]
    , NULL AS [Organizational ID Number]
    , NULL AS [Other Address PO Box]
    , NULL AS [Priority]
    , NULL AS [Private]
    , NULL AS [Profession]
    , NULL AS [Referred By]
    , NULL AS [Sensitivity]
    , NULL AS [Spouse]
    , c.ContactCategory AS [User 1]
    , NULL AS [User 2]
    , NULL AS [User 3]
    , NULL AS [User 4]
    , c.Website AS [Web Page]
FROM
    dbo71.Contacts c


GO

INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'ae',N'United Arab Emirates',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'af',N'Afghanistan',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'ar',N'Argentina',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'at',N'Austria',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'au',N'Australia',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'bd',N'Bangladesh',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'be',N'Belgium',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'bg',N'Bulgaria',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'bh',N'Bahrain',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'br',N'Brazil',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'bs',N'Bahamas',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'by',N'Belarus',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'ca',N'Canada',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'ch',N'Switzerland',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'cl',N'Chile',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'cn',N'China',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'cy',N'Cyprus',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'cz',N'Czech Republic',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'de',N'Germany',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'dk',N'Denmark',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'dz',N'Algeria',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'eg',N'Egypt',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'es',N'Spain',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'et',N'Ethiopia',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'fi',N'Finland',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'fr',N'France',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'gb',N'United Kingdom',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'gh',N'Ghana',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'gr',N'Greece',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'hk',N'Hong Kong',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'hr',N'Croatia',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'hu',N'Hungary',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'id',N'Indonesia',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'ie',N'Ireland',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'il',N'Israel',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'in',N'India',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'ir',N'Iran',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'it',N'Italy',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'jm',N'Jamaica',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'jp',N'Japan',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'ke',N'Kenya',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'kh',N'Cambodia',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'kr',N'Korea',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'kz',N'Kazakhstan',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'lb',N'Lebanon',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'li',N'Liechtenstein',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'lk',N'Sri Lanka',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'lt',N'Lithuania',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'lu',N'Luxembourg',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'ma',N'Morocco',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'mm',N'Myanmar',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'mx',N'Mexico',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'my',N'Malaysia',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'nc',N'New Caledonia',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'ng',N'Nigeria',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'ni',N'Nicaragua',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'nl',N'Netherlands',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'no',N'Norway',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'np',N'Nepal',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'nz',N'New Zealand',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'oo',N'Other',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'pa',N'Panama',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'ph',N'Philippines',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'pk',N'Pakistan',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'pl',N'Poland',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'pt',N'Portugal',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'qa',N'Qatar',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'ro',N'Romania',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'ru',N'Russian Federation',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'sa',N'Saudi Arabia',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'sc',N'Seychelles',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'sd',N'Sudan',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'se',N'Sweden',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'sg',N'Singapore',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'si',N'Slovenia',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'sk',N'Slovak Republic',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'sy',N'Syria',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'th',N'Thailand',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'tr',N'Turkey',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'tt',N'Trinidad and Tobago',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'tz',N'Tanzania',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'ua',N'Ukraine',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'ug',N'Uganda',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'us',N'United States',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'vn',N'Vietnam',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'za',N'South Africa',1);
INSERT INTO [dbo71].[Countries] ([CountryCode],[CountryName],[Checked]) VALUES (N'zm',N'Zambia',1);

INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'126.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'aol.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'att.net',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'bk.ru',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'btinternet.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'comcast.net',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'cox.net',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'cs.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'd.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'europe.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'free.fr',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'gmail.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'gmx.de',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'gmx.net',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'googlemail.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'hotbox.ru',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'hotmail.co.uk',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'hotmail.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'inbox.ru',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'land.ru',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'libero.it',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'list.ru',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'live.cn',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'live.co.uk',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'live.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'live.com.au',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'live.de',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'live.it',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'mac.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'mail.ru',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'me.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'msn.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'nm.ru',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'oncallmail.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'outlook.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'pochta.ru',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'rambler.ru',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'seznam.cz',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'sympatico.ca',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'teleworm.us',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'tiscali.it',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'tut.by',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'ukr.net',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'uol.com.br',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'verizon.net',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'virgin.net',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'y7mail.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'ya.ru',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.ca',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.co.in',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.co.jp',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.co.uk',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.com.au',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.com.br',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.com.cn',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.com.vn',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.de',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.es',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.fr',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.ie',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.in',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.nl',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.no',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yahoo.pt',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yandex.ru',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'ymail.com',NULL,N'Public emails',1);
INSERT INTO [dbo71].[Websites] ([Domain],[Website],[Comment],[Checked]) VALUES (N'yopmail.com',NULL,N'Public emails',1);

INSERT INTO [dbo01].[QueryList] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_TYPE],[INSERT_PROCEDURE],[UPDATE_PROCEDURE],[DELETE_PROCEDURE]) VALUES (N'{FileName=google.csv};CodePage={CodePage=1251}',N'Google - CSV',N'TEXT',N'dbo71.SourceContacts',N'dbo71.SourceContacts',N'dbo71.SourceContacts');
INSERT INTO [dbo01].[QueryList] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_TYPE],[INSERT_PROCEDURE],[UPDATE_PROCEDURE],[DELETE_PROCEDURE]) VALUES (N'{FileName=linkedin_connections_export_microsoft_outlook.csv};CodePage={CodePage=1251}',N'LinkedIn - CSV',N'TEXT',N'dbo71.SourceContacts',N'dbo71.SourceContacts',N'dbo71.SourceContacts');
INSERT INTO [dbo01].[QueryList] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_TYPE],[INSERT_PROCEDURE],[UPDATE_PROCEDURE],[DELETE_PROCEDURE]) VALUES (N'{FileName=outlook.csv};CodePage={CodePage=1251}',N'Outlook - CSV',N'TEXT',N'dbo71.SourceContacts',N'dbo71.SourceContacts',N'dbo71.SourceContacts');
INSERT INTO [dbo01].[QueryList] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_TYPE],[INSERT_PROCEDURE],[UPDATE_PROCEDURE],[DELETE_PROCEDURE]) VALUES (N'{FileName=YahooAddressBook.htm};CodePage={CodePage=65001}',N'Yahoo! - HTM',N'TEXT',N'dbo71.SourceContacts',N'dbo71.SourceContacts',N'dbo71.SourceContacts');
INSERT INTO [dbo01].[QueryList] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_TYPE],[INSERT_PROCEDURE],[UPDATE_PROCEDURE],[DELETE_PROCEDURE]) VALUES (N'http://api.linkedin.com/v1/people/~/connections',N'LinkedIn - API',N'HTTP',N'dbo71.SourceContacts',N'dbo71.SourceContacts',N'dbo71.SourceContacts');
INSERT INTO [dbo01].[QueryList] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_TYPE],[INSERT_PROCEDURE],[UPDATE_PROCEDURE],[DELETE_PROCEDURE]) VALUES (N'http://social.yahooapis.com/v1/user/me/contacts',N'Yahoo! - API',N'HTTP',N'dbo71.SourceContacts',N'dbo71.SourceContacts',N'dbo71.SourceContacts');
INSERT INTO [dbo01].[QueryList] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_TYPE],[INSERT_PROCEDURE],[UPDATE_PROCEDURE],[DELETE_PROCEDURE]) VALUES (N'http://www.google.com/m8/feeds/contacts/default/base?alt=json{}',N'Google - API',N'HTTP',N'dbo71.SourceContacts',N'dbo71.SourceContacts',N'dbo71.SourceContacts');
INSERT INTO [dbo01].[QueryList] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_TYPE],[INSERT_PROCEDURE],[UPDATE_PROCEDURE],[DELETE_PROCEDURE]) VALUES (N'xls71',N'viewContacts',N'VIEW',N'dbo71.Contacts',N'dbo71.Contacts',N'dbo71.Contacts');
INSERT INTO [dbo01].[QueryList] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_TYPE],[INSERT_PROCEDURE],[UPDATE_PROCEDURE],[DELETE_PROCEDURE]) VALUES (N'xls71',N'viewSourceContacts',N'VIEW',N'dbo71.SourceContacts',N'dbo71.SourceContacts',N'dbo71.SourceContacts');
INSERT INTO [dbo01].[QueryList] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_TYPE],[INSERT_PROCEDURE],[UPDATE_PROCEDURE],[DELETE_PROCEDURE]) VALUES (N'xls71',N'viewWebsites',N'VIEW',N'dbo71.Websites',N'dbo71.Websites',N'dbo71.Websites');
INSERT INTO [dbo01].[QueryList] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_TYPE],[INSERT_PROCEDURE],[UPDATE_PROCEDURE],[DELETE_PROCEDURE]) VALUES (N'xls71',N'viewCountries',N'VIEW',N'dbo71.Countries',N'dbo71.Countries',N'dbo71.Countries');
INSERT INTO [dbo01].[QueryList] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_TYPE],[INSERT_PROCEDURE],[UPDATE_PROCEDURE],[DELETE_PROCEDURE]) VALUES (N'xls71',N'uspContactsByDomain',N'PROCEDURE',N'dbo71.Contacts',N'dbo71.Contacts',N'dbo71.Contacts');
INSERT INTO [dbo01].[QueryList] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_TYPE],[INSERT_PROCEDURE],[UPDATE_PROCEDURE],[DELETE_PROCEDURE]) VALUES (N'xls71',N'uspContactsByCountryName',N'PROCEDURE',N'dbo71.Contacts',N'dbo71.Contacts',N'dbo71.Contacts');
INSERT INTO [dbo01].[QueryList] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_TYPE],[INSERT_PROCEDURE],[UPDATE_PROCEDURE],[DELETE_PROCEDURE]) VALUES (N'xls71',N'uspSourceContactsByContactID',N'PROCEDURE',N'dbo71.SourceContacts',N'dbo71.SourceContacts',N'dbo71.SourceContacts');

INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'http',N'Google - API',NULL,N'Actions',N'http://contacts.google.com',N'Open Google Contacts page',N'HTTP',NULL,11,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'http',N'LinkedIn - API',NULL,N'Actions',N'http://www.linkedin.com/people/connections?trk=nav_responsive_tab_network',N'Open LinkedIn Connections page',N'HTTP',NULL,11,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'http',N'LinkedIn - API',NULL,N'ContextMenu',N'{SourceProfileUrl}',N'Open LinkedIn Profile',N'HTTP',NULL,12,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'http',N'Yahoo! - API',NULL,N'Actions',N'http://contacts.yahoo.com/',N'Open Yahoo! Contacts page',N'HTTP',NULL,11,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'Google - CSV',NULL,N'Actions',N'http://contacts.google.com',N'Open Google Contacts export page',N'HTTP',NULL,11,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'Google - CSV',NULL,N'Actions',N'{WorkbookPath}',N'Open working directory',N'CMD',NULL,12,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'Google - CSV',NULL,N'Actions',N'notepad.exe {FileName}',N'Open source file',N'CMD',NULL,13,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'LinkedIn - CSV',NULL,N'Actions',N'http://www.linkedin.com/people/export-settings',N'Open LinkedIn export page',N'HTTP',NULL,11,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'LinkedIn - CSV',NULL,N'Actions',N'{WorkbookPath}',N'Open working directory',N'CMD',NULL,12,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'LinkedIn - CSV',NULL,N'Actions',N'notepad.exe {FileName}',N'Open source file',N'CMD',NULL,13,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'Outlook - CSV',NULL,N'Actions',N'{WorkbookPath}',N'Open working directory',N'CMD',NULL,12,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'Outlook - CSV',NULL,N'Actions',N'notepad.exe {FileName}',N'Open source file',N'CMD',NULL,13,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'Yahoo! - HTM',NULL,N'Actions',N'http://contacts.yahoo.com/',N'Open Yahoo! Contacts export page',N'HTTP',NULL,11,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'Yahoo! - HTM',NULL,N'Actions',N'{WorkbookPath}',N'Open working directory',N'CMD',NULL,12,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'Yahoo! - HTM',NULL,N'Actions',N'{FileName}',N'Open source file',N'CMD',NULL,13,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'Actions',N'etl71',N'uspUpdateContacts',N'PROCEDURE',N'_Reload',11,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewSourceContacts',NULL,N'Actions',N'etl71',N'uspUpdateContacts',N'PROCEDURE',N'_Reload',11,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'Contacts',NULL,N'Actions',N'etl71',N'uspUpdateContacts',N'PROCEDURE',N'_Reload',11,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'SourceContacts',NULL,N'Actions',N'etl71',N'uspUpdateContacts',N'PROCEDURE',N'_Reload',11,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewWebsites',NULL,N'Actions',N'etl71',N'uspUpdateContacts',N'PROCEDURE',N'_Reload',10,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewWebsites',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,20,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewWebsites',NULL,N'Actions',N'xls71',N'uspExcelEvent_Actions_dbo71_Websites_SetChecked',N'PROCEDURE',N'_Reload',21,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewWebsites',NULL,N'ContextMenu',N'http://{Website}',N'Open website {Website}',N'HTTP',NULL,11,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'Websites',NULL,N'Actions',N'etl71',N'uspUpdateContacts',N'PROCEDURE',N'_Reload',10,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'Websites',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,20,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'Websites',NULL,N'Actions',N'xls71',N'uspExcelEvent_Actions_dbo71_Websites_SetChecked',N'PROCEDURE',N'_Reload',21,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'Websites',NULL,N'ContextMenu',N'http://{Website}',N'Open website {Website}',N'HTTP',NULL,11,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewCountries',NULL,N'Actions',N'etl71',N'uspUpdateContacts',N'PROCEDURE',N'_Reload',11,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewCountries',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,20,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewCountries',NULL,N'Actions',N'xls71',N'uspExcelEvent_Actions_dbo71_Countries_SetChecked',N'PROCEDURE',N'_Reload',21,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'Countries',NULL,N'Actions',N'etl71',N'uspUpdateContacts',N'PROCEDURE',N'_Reload',11,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'Countries',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,20,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'Countries',NULL,N'Actions',N'xls71',N'uspExcelEvent_Actions_dbo71_Countries_SetChecked',N'PROCEDURE',N'_Reload',21,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,20,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'Actions',N'xls71',N'uspExcelEvent_Actions_dbo71_Contacts_SetAsConfirmed',N'PROCEDURE',N'_Reload',22,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'ContextMenu',N'http://{Website}',N'Open website {Website}',N'HTTP',NULL,11,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'ContextMenu',N'{LinkedInProfileUrl}',N'Open LinkedIn Profile',N'HTTP',NULL,12,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'ContextMenu',N'{GooglePlusProfileUrl}',N'Open GooglePlus Profile',N'HTTP',NULL,13,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'ContextMenu',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,20,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'ContextMenu',N'mailto:{Email}&subject=Thanks for the connection&body=Hello {FirstName},%0A',N'Create an email - Thanks for the connection',N'CMD',NULL,21,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'ContextMenu',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,30,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'ContextMenu',N'https://www.google.com/search?as_q={Company}',N'Google Search - {Company}',N'HTTP',NULL,31,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'ContextMenu',N'https://www.google.com/search?as_q={FirstName}+{LastName}',N'Google Search - {FirstName} {LastName}',N'HTTP',NULL,32,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'SelectionChange',N'xls71',N'uspSourceContactsByContactID',N'PROCEDURE',N'ContactDetails _TopWindow',NULL,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewWebsites',NULL,N'SelectionChange',N'xls71',N'uspContactsByDomain',N'PROCEDURE',N'WebsiteDetails _TopWindow',NULL,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewCountries',NULL,N'SelectionChange',N'xls71',N'uspContactsByCountryName',N'PROCEDURE',N'CountryDetails _TopWindow',21,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'Contacts',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,50,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'Countries',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,50,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'SourceContacts',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,50,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'Websites',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,50,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'http',N'Google - API',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,50,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'http',N'LinkedIn - API',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,50,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'http',N'Yahoo! - API',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,50,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'Google - CSV',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,50,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'LinkedIn - CSV',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,50,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'Outlook - CSV',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,50,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'Yahoo! - HTM',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,50,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,50,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewCountries',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,50,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewSourceContacts',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,50,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewWebsites',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,50,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'Contacts',NULL,N'Actions',N'http://www.savetodb.com/social-contact-database.htm/{DataLanguage}/',N'Online Help, Questions, and Suggestions',N'HTTP',NULL,51,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'Countries',NULL,N'Actions',N'http://www.savetodb.com/social-contact-database.htm/{DataLanguage}/',N'Online Help, Questions, and Suggestions',N'HTTP',NULL,51,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'SourceContacts',NULL,N'Actions',N'http://www.savetodb.com/social-contact-database.htm/{DataLanguage}/',N'Online Help, Questions, and Suggestions',N'HTTP',NULL,51,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'dbo71',N'Websites',NULL,N'Actions',N'http://www.savetodb.com/social-contact-database.htm/{DataLanguage}/',N'Online Help, Questions, and Suggestions',N'HTTP',NULL,51,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'http',N'Google - API',NULL,N'Actions',N'http://www.savetodb.com/social-contact-database.htm/{DataLanguage}/',N'Online Help, Questions, and Suggestions',N'HTTP',NULL,51,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'http',N'LinkedIn - API',NULL,N'Actions',N'http://www.savetodb.com/social-contact-database.htm/{DataLanguage}/',N'Online Help, Questions, and Suggestions',N'HTTP',NULL,51,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'http',N'Yahoo! - API',NULL,N'Actions',N'http://www.savetodb.com/social-contact-database.htm/{DataLanguage}/',N'Online Help, Questions, and Suggestions',N'HTTP',NULL,51,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'Google - CSV',NULL,N'Actions',N'http://www.savetodb.com/social-contact-database.htm/{DataLanguage}/',N'Online Help, Questions, and Suggestions',N'HTTP',NULL,51,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'LinkedIn - CSV',NULL,N'Actions',N'http://www.savetodb.com/social-contact-database.htm/{DataLanguage}/',N'Online Help, Questions, and Suggestions',N'HTTP',NULL,51,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'Outlook - CSV',NULL,N'Actions',N'http://www.savetodb.com/social-contact-database.htm/{DataLanguage}/',N'Online Help, Questions, and Suggestions',N'HTTP',NULL,51,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'text',N'Yahoo! - HTM',NULL,N'Actions',N'http://www.savetodb.com/social-contact-database.htm/{DataLanguage}/',N'Online Help, Questions, and Suggestions',N'HTTP',NULL,51,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'Actions',N'http://www.savetodb.com/social-contact-database.htm/{DataLanguage}/',N'Online Help, Questions, and Suggestions',N'HTTP',NULL,51,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewCountries',NULL,N'Actions',N'http://www.savetodb.com/social-contact-database.htm/{DataLanguage}/',N'Online Help, Questions, and Suggestions',N'HTTP',NULL,51,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewSourceContacts',NULL,N'Actions',N'http://www.savetodb.com/social-contact-database.htm/{DataLanguage}/',N'Online Help, Questions, and Suggestions',N'HTTP',NULL,51,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewWebsites',NULL,N'Actions',N'http://www.savetodb.com/social-contact-database.htm/{DataLanguage}/',N'Online Help, Questions, and Suggestions',N'HTTP',NULL,51,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'Actions',NULL,N'MenuSeparator',N'MENUSEPARATOR',NULL,40,NULL,1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[EVENT_NAME],[HANDLER_SCHEMA],[HANDLER_NAME],[HANDLER_TYPE],[TARGET_WORKSHEET],[MENU_ORDER],[EDIT_PARAMETERS],[IS_ACTIVE]) VALUES (N'xls71',N'viewContacts',NULL,N'Actions',N'xls71',N'viewExportOutlookCSV',N'VIEW',N'_SaveAs contacts.csv;CodePage=1252;Format=CSV;Separator=,',41,NULL,1);

INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'xls71',N'viewContacts',N'en',N'Contacts',NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'xls71',N'viewSourceContacts',N'en',N'Source Contacts',NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'xls71',N'viewWebsites',N'en',N'Websites',NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'xls71',N'viewCountries',N'en',N'Countries',NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'xls71',N'uspExcelEvent_Actions_dbo71_Countries_SetChecked',N'en',N'Update Records as Checked',NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'xls71',N'uspExcelEvent_Actions_dbo71_Websites_SetChecked',N'en',N'Update Records as Checked',NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'xls71',N'uspExcelEvent_Actions_dbo71_Contacts_SetAsConfirmed',N'en',N'Update Records as Checked',NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'xls71',N'uspSourceContactsByContactID',N'en',N'Source Contacts for Contact ID',NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'xls71',N'uspContactsByDomain',N'en',N'Contacts for Domain',NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'xls71',N'uspContactsByCountryName',N'en',N'Contacts for Country',NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'dbo71',N'SourceContacts',N'en',N'Source Contacts Table',NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'dbo71',N'Countries',N'en',N'Countries Table',NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'dbo71',N'Websites',N'en',N'Websites Table',NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'dbo71',N'Contacts',N'en',N'Contacts Table',NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'etl71',N'uspDiscoverExistingContacts',N'en',NULL,NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'etl71',N'uspUpdateContacts',N'en',N'Update Data',NULL,NULL);
INSERT INTO [dbo01].[ObjectTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC],[TRANSLATED_COMMENT]) VALUES (N'xls71',N'viewExportOutlookCSV',N'en',N'Export Contacts as Outlook CSV',NULL,NULL);

INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'Company',N'en',N'Company',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'ContactCategory',N'en',N'ContactCategory',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'ContactStatus',N'en',N'ContactStatus',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'Country',N'en',N'Country',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'Domain',N'en',N'Domain',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'Email',N'en',N'Email',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'FirstName',N'en',N'FirstName',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'GooglePlusProfileUrl',N'en',N'GooglePlusProfileUrl',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'ID',N'en',N'ID',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'Industry',N'en',N'Industry',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'IsDoubleName',N'en',N'IsDoubleName',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'LastName',N'en',N'LastName',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'LinkedInHeadline',N'en',N'LinkedInHeadline',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'LinkedInProfileUrl',N'en',N'LinkedInProfileUrl',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'LocationName',N'en',N'LocationName',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'Mobile',N'en',N'Mobile',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'Number1',N'en',N'Number1',N'You may change the column name in the dbo01.ColumnTranslation.TRANSLATED_NAME field');
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'Number2',N'en',N'Number2',N'You may change the column name in the dbo01.ColumnTranslation.TRANSLATED_NAME field');
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'Number3',N'en',N'Number3',N'You may change the column name in the dbo01.ColumnTranslation.TRANSLATED_NAME field');
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'Phone',N'en',N'Phone',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'String1',N'en',N'String1',N'You may change the column name in the dbo01.ColumnTranslation.TRANSLATED_NAME field');
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'String2',N'en',N'String2',N'You may change the column name in the dbo01.ColumnTranslation.TRANSLATED_NAME field');
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'String3',N'en',N'String3',N'You may change the column name in the dbo01.ColumnTranslation.TRANSLATED_NAME field');
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'Title',N'en',N'Title',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewContacts',N'Website',N'en',N'Website',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'ContactID',N'en',N'ContactID',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'IsJunk',N'en',N'IsJunk',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'MergeDate',N'en',N'MergeDate',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'Source',N'en',N'Source',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'SourceCompany',N'en',N'SourceCompany',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'SourceCountryCode',N'en',N'SourceCountryCode',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'SourceEmail',N'en',N'SourceEmail',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'SourceFirstName',N'en',N'SourceFirstName',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'SourceHeadline',N'en',N'SourceHeadline',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'SourceID',N'en',N'SourceID',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'SourceIndustry',N'en',N'SourceIndustry',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'SourceLastName',N'en',N'SourceLastName',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'SourceLocationName',N'en',N'SourceLocationName',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'SourceMobile',N'en',N'SourceMobile',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'SourcePhone',N'en',N'SourcePhone',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'SourceProfileUrl',N'en',N'SourceProfileUrl',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'SourceSecondaryEmail',N'en',N'SourceSecondaryEmail',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewSourceContacts',N'SourceTitle',N'en',N'SourceTitle',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewWebsites',N'Checked',N'en',N'Checked',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewWebsites',N'Comment',N'en',N'Comment',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewWebsites',N'ContactCategoryCount',N'en',N'ContactCategoryCount',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewWebsites',N'Domain',N'en',N'Domain',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewWebsites',N'DomainContactCount',N'en',N'DomainContactCount',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewWebsites',N'HasErrors',N'en',N'HasErrors',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewWebsites',N'ID',N'en',N'ID',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewWebsites',N'SourceDomainContactCount',N'en',N'SourceDomainContactCount',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewWebsites',N'Website',N'en',N'Website',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewWebsites',N'WebsiteContactCount',N'en',N'WebsiteContactCount',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewCountries',N'Checked',N'en',N'Checked',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewCountries',N'ContactCount',N'en',N'ContactCount',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewCountries',N'CountryCode',N'en',N'CountryCode',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewCountries',N'CountryName',N'en',N'CountryName',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'viewCountries',N'HasErrors',N'en',N'HasErrors',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'uspSourceContactsByContactID',N'ID',N'en',N'ID',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'uspContactsByDomain',N'Domain',N'en',N'Domain',NULL);
INSERT INTO [dbo01].[ColumnTranslation] ([TABLE_SCHEMA],[TABLE_NAME],[COLUMN_NAME],[LANGUAGE_NAME],[TRANSLATED_NAME],[TRANSLATED_DESC]) VALUES (N'xls71',N'uspContactsByCountryName',N'CountryName',N'en',N'CountryName',NULL);

INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'text',N'LinkedIn - CSV',N'<table name="text.LinkedIn - CSV"><columnFormats><column name="" property="ListObjectName" value="LinkedInCSV_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium2" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="True" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="Title" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Title" property="Address" value="$C$4" type="String" /><column name="First Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="First Name" property="Address" value="$D$4" type="String" /><column name="Middle Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Middle Name" property="Address" value="$E$4" type="String" /><column name="Last Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Last Name" property="Address" value="$F$4" type="String" /><column name="Suffix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Suffix" property="Address" value="$G$4" type="String" /><column name="E-mail Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail Address" property="Address" value="$H$4" type="String" /><column name="E-mail 2 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 Address" property="Address" value="$I$4" type="String" /><column name="E-mail 3 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Address" property="Address" value="$J$4" type="String" /><column name="Business Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street" property="Address" value="$K$4" type="String" /><column name="Business Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 2" property="Address" value="$L$4" type="String" /><column name="Business Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 3" property="Address" value="$M$4" type="String" /><column name="Business City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business City" property="Address" value="$N$4" type="String" /><column name="Business State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business State" property="Address" value="$O$4" type="String" /><column name="Business Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Postal Code" property="Address" value="$P$4" type="String" /><column name="Business Country" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Country" property="Address" value="$Q$4" type="String" /><column name="Home Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street" property="Address" value="$R$4" type="String" /><column name="Home Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 2" property="Address" value="$S$4" type="String" /><column name="Home Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 3" property="Address" value="$T$4" type="String" /><column name="Home City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home City" property="Address" value="$U$4" type="String" /><column name="Home State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home State" property="Address" value="$V$4" type="String" /><column name="Home Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Postal Code" property="Address" value="$W$4" type="String" /><column name="Home Country" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Country" property="Address" value="$X$4" type="String" /><column name="Other Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street" property="Address" value="$Y$4" type="String" /><column name="Other Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 2" property="Address" value="$Z$4" type="String" /><column name="Other Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 3" property="Address" value="$AA$4" type="String" /><column name="Other City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other City" property="Address" value="$AB$4" type="String" /><column name="Other State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other State" property="Address" value="$AC$4" type="String" /><column name="Other Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Postal Code" property="Address" value="$AD$4" type="String" /><column name="Other Country" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Country" property="Address" value="$AE$4" type="String" /><column name="Company" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company" property="Address" value="$AF$4" type="String" /><column name="Department" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Department" property="Address" value="$AG$4" type="String" /><column name="Job Title" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Job Title" property="Address" value="$AH$4" type="String" /><column name="Assistant''s Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Assistant''s Phone" property="Address" value="$AI$4" type="String" /><column name="Business Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Fax" property="Address" value="$AJ$4" type="String" /><column name="Business Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Phone" property="Address" value="$AK$4" type="String" /><column name="Business Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Phone 2" property="Address" value="$AL$4" type="String" /><column name="Callback" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Callback" property="Address" value="$AM$4" type="String" /><column name="Car Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Car Phone" property="Address" value="$AN$4" type="String" /><column name="Company Main Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company Main Phone" property="Address" value="$AO$4" type="String" /><column name="Home Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Fax" property="Address" value="$AP$4" type="String" /><column name="Home Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone" property="Address" value="$AQ$4" type="String" /><column name="Home Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone 2" property="Address" value="$AR$4" type="String" /><column name="ISDN" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ISDN" property="Address" value="$AS$4" type="String" /><column name="Mobile Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mobile Phone" property="Address" value="$AT$4" type="String" /><column name="Other Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Fax" property="Address" value="$AU$4" type="String" /><column name="Other Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Phone" property="Address" value="$AV$4" type="String" /><column name="Pager" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Pager" property="Address" value="$AW$4" type="String" /><column name="Primary Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Primary Phone" property="Address" value="$AX$4" type="String" /><column name="Radio Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Radio Phone" property="Address" value="$AY$4" type="String" /><column name="TTY/TDD Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="TTY/TDD Phone" property="Address" value="$AZ$4" type="String" /><column name="Telex" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Telex" property="Address" value="$BA$4" type="String" /><column name="Assistant''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Assistant''s Name" property="Address" value="$BB$4" type="String" /><column name="Birthday" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Birthday" property="Address" value="$BC$4" type="String" /><column name="Manager''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Manager''s Name" property="Address" value="$BD$4" type="String" /><column name="Notes" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Notes" property="Address" value="$BE$4" type="String" /><column name="Other Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Address PO Box" property="Address" value="$BF$4" type="String" /><column name="Spouse" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Spouse" property="Address" value="$BG$4" type="String" /><column name="Web Page" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Web Page" property="Address" value="$BH$4" type="String" /><column name="Personal Web Page" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Personal Web Page" property="Address" value="$BI$4" type="String" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="Address" value="$BJ$4" type="String" /><column name="Source" property="FormulaR1C1" value="=&quot;LinkedInCSV&quot;" type="String" /><column name="Source" property="ColumnWidth" value="11.29" type="Double" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="Address" value="$BK$4" type="String" /><column name="SourceID" property="FormulaR1C1" value="=[@[E-mail Address]]" type="String" /><column name="SourceID" property="ColumnWidth" value="36.57" type="Double" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="Address" value="$BL$4" type="String" /><column name="SourceFirstName" property="FormulaR1C1" value="=SUBSTITUTE(SUBSTITUTE([@[First Name]],&quot;?&quot;,&quot;&quot;),&quot;?&quot;,&quot;&quot;)" type="String" /><column name="SourceFirstName" property="ColumnWidth" value="17.86" type="Double" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="Address" value="$BM$4" type="String" /><column name="SourceLastName" property="FormulaR1C1" value="=SUBSTITUTE(SUBSTITUTE([@[Last Name]],&quot;?&quot;,&quot;&quot;),&quot;?&quot;,&quot;&quot;)" type="String" /><column name="SourceLastName" property="ColumnWidth" value="17.43" type="Double" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="Address" value="$BN$4" type="String" /><column name="SourceEmail" property="FormulaR1C1" value="=[@[E-mail Address]]" type="String" /><column name="SourceEmail" property="ColumnWidth" value="32.14" type="Double" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="Address" value="$BO$4" type="String" /><column name="SourceCompany" property="FormulaR1C1" value="=TRIM(IF(LEN([@Company])&gt;0,SUBSTITUTE(SUBSTITUTE([@Company],&quot;?&quot;,&quot;&quot;),&quot;?&quot;,&quot;&quot;),&quot;&quot;))" type="String" /><column name="SourceCompany" property="ColumnWidth" value="41.57" type="Double" /><column name="SourceJobTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceJobTitle" property="Address" value="$BP$4" type="String" /><column name="SourceJobTitle" property="FormulaR1C1" value="=IF(LEN([@[Job Title]])&gt;0,SUBSTITUTE(SUBSTITUTE([@[Job Title]],&quot;?&quot;,&quot;&quot;),&quot;?&quot;,&quot;&quot;),&quot;&quot;)" type="String" /><column name="SourceJobTitle" property="ColumnWidth" value="45.29" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="2" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats><views><view name="All"><column name="" property="ListObjectName" value="LinkedInCSV_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="First Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Middle Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Last Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Suffix" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 3 Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Street" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Street 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Street 3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business City" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business State" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Postal Code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Country" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Street" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Street 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Street 3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home City" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home State" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Postal Code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Country" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Street" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Street 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Street 3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other City" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other State" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Postal Code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Country" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Company" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Department" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Job Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Assistant''s Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Fax" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Phone 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Callback" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Car Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Company Main Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Fax" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Phone 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ISDN" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Mobile Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Fax" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Pager" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Primary Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Radio Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="TTY/TDD Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Telex" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Assistant''s Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Birthday" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Manager''s Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Notes" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Address PO Box" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Spouse" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Web Page" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Personal Web Page" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Source Fields"><column name="" property="ListObjectName" value="LinkedInCSV_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="First Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Middle Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Last Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Suffix" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 3 Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Street" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Street 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Street 3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business City" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business State" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Postal Code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Country" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Street" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Street 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Street 3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home City" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home State" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Postal Code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Country" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Street" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Street 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Street 3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other City" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other State" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Postal Code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Country" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Company" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Department" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Job Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Assistant''s Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Fax" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Phone 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Callback" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Car Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Company Main Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Fax" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Phone 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ISDN" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Mobile Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Fax" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Pager" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Primary Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Radio Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="TTY/TDD Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Telex" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Assistant''s Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Birthday" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Manager''s Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Notes" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Address PO Box" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Spouse" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Web Page" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Personal Web Page" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceJobTitle" property="EntireColumn.Hidden" value="True" type="Boolean" /></view><view name="Source Main Fields"><column name="" property="ListObjectName" value="LinkedInCSV_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Title" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="First Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Middle Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Last Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Suffix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Country" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Country" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Country" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Department" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Job Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Assistant''s Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Callback" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Car Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company Main Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ISDN" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mobile Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Pager" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Primary Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Radio Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="TTY/TDD Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Telex" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Assistant''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Birthday" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Manager''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Notes" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Spouse" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Web Page" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Personal Web Page" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceJobTitle" property="EntireColumn.Hidden" value="True" type="Boolean" /></view><view name="Main Fields"><column name="" property="ListObjectName" value="LinkedInCSV_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Title" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="First Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Middle Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Last Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Suffix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Country" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Country" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Country" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Department" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Job Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Assistant''s Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Callback" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Car Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company Main Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ISDN" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mobile Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Pager" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Primary Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Radio Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="TTY/TDD Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Telex" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Assistant''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Birthday" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Manager''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Notes" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Spouse" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Web Page" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Personal Web Page" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceJobTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Export Fields"><column name="" property="ListObjectName" value="LinkedInCSV_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Title" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="First Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Middle Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Last Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Suffix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Country" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Country" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Country" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Department" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Job Title" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Assistant''s Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Callback" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Car Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company Main Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ISDN" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mobile Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Pager" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Primary Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Radio Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="TTY/TDD Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Telex" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Assistant''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Birthday" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Manager''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Notes" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Spouse" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Web Page" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Personal Web Page" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceJobTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /></view></views></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'http',N'LinkedIn - API',N'<table name="http.LinkedIn - API"><columnFormats><column name="" property="ListObjectName" value="LinkedIn_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium2" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="True" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="id" property="Address" value="$C$4" type="String" /><column name="first-name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="first-name" property="Address" value="$D$4" type="String" /><column name="last-name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="last-name" property="Address" value="$E$4" type="String" /><column name="headline" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="headline" property="Address" value="$F$4" type="String" /><column name="picture-url" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="picture-url" property="Address" value="$G$4" type="String" /><column name="api-standard-profile-request.url" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.url" property="Address" value="$H$4" type="String" /><column name="api-standard-profile-request.headers.total" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.headers.total" property="Address" value="$I$4" type="String" /><column name="api-standard-profile-request.headers.http-header.name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.headers.http-header.name" property="Address" value="$J$4" type="String" /><column name="api-standard-profile-request.headers.http-header.value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.headers.http-header.value" property="Address" value="$K$4" type="String" /><column name="site-standard-profile-request.url" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="site-standard-profile-request.url" property="Address" value="$L$4" type="String" /><column name="location.name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="location.name" property="Address" value="$M$4" type="String" /><column name="location.country.code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="location.country.code" property="Address" value="$N$4" type="String" /><column name="industry" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="industry" property="Address" value="$O$4" type="String" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="Address" value="$P$4" type="String" /><column name="Source" property="FormulaR1C1" value="=IF([@id]=&quot;private&quot;,&quot;&quot;,&quot;LinkedIn&quot;)" type="String" /><column name="Source" property="ColumnWidth" value="8.86" type="Double" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="Address" value="$Q$4" type="String" /><column name="SourceID" property="FormulaR1C1" value="=IF([@id]=&quot;private&quot;,&quot;&quot;,[@id])" type="String" /><column name="SourceID" property="ColumnWidth" value="14.14" type="Double" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="Address" value="$R$4" type="String" /><column name="SourceFirstName" property="FormulaR1C1" value="=IF([@SourceID]=&quot;&quot;,&quot;&quot;,SUBSTITUTE(SUBSTITUTE(SUBSTITUTE([@[first-name]],&quot;?&quot;,&quot;&quot;),&quot;?&quot;,&quot;&quot;),&quot;?&quot;,&quot;&quot;))" type="String" /><column name="SourceFirstName" property="ColumnWidth" value="20.43" type="Double" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="Address" value="$S$4" type="String" /><column name="SourceLastName" property="FormulaR1C1" value="=IF([@SourceID]=&quot;&quot;,&quot;&quot;,SUBSTITUTE(SUBSTITUTE(SUBSTITUTE([@[last-name]],&quot;?&quot;,&quot;&quot;),&quot;?&quot;,&quot;&quot;),&quot;?&quot;,&quot;&quot;))" type="String" /><column name="SourceLastName" property="ColumnWidth" value="43.86" type="Double" /><column name="SourceHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceHeadline" property="Address" value="$T$4" type="String" /><column name="SourceHeadline" property="FormulaR1C1" value="=[@headline]&amp;&quot;&quot;" type="String" /><column name="SourceHeadline" property="ColumnWidth" value="54.57" type="Double" /><column name="SourceLocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLocationName" property="Address" value="$U$4" type="String" /><column name="SourceLocationName" property="FormulaR1C1" value="=[@[location.name]]&amp;&quot;&quot;" type="String" /><column name="SourceLocationName" property="ColumnWidth" value="44.71" type="Double" /><column name="SourceCountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCountryCode" property="Address" value="$V$4" type="String" /><column name="SourceCountryCode" property="FormulaR1C1" value="=[@[location.country.code]]&amp;&quot;&quot;" type="String" /><column name="SourceCountryCode" property="ColumnWidth" value="20.43" type="Double" /><column name="SourceIndustry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceIndustry" property="Address" value="$W$4" type="String" /><column name="SourceIndustry" property="FormulaR1C1" value="=[@industry]&amp;&quot;&quot;" type="String" /><column name="SourceIndustry" property="ColumnWidth" value="34.43" type="Double" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceProfileUrl" property="Address" value="$X$4" type="String" /><column name="SourceProfileUrl" property="FormulaR1C1" value="=[@[site-standard-profile-request.url]]&amp;&quot;&quot;" type="String" /><column name="SourceProfileUrl" property="ColumnWidth" value="71.14" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="2" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats><views><view name="All"><column name="" property="ListObjectName" value="LinkedIn_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="id" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="first-name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="last-name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="headline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="picture-url" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="api-standard-profile-request.url" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="api-standard-profile-request.headers.total" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="api-standard-profile-request.headers.http-header.name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="api-standard-profile-request.headers.http-header.value" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="site-standard-profile-request.url" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="location.name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="location.country.code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="industry" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Source Fields"><column name="" property="ListObjectName" value="LinkedIn_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="id" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="first-name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="last-name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="headline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="picture-url" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="api-standard-profile-request.url" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="api-standard-profile-request.headers.total" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="api-standard-profile-request.headers.http-header.name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="api-standard-profile-request.headers.http-header.value" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="site-standard-profile-request.url" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="location.name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="location.country.code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="industry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceHeadline" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLocationName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceCountryCode" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceIndustry" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="True" type="Boolean" /></view><view name="Source Main Fields"><column name="" property="ListObjectName" value="LinkedIn_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="first-name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="last-name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="headline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="picture-url" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.url" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.headers.total" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.headers.http-header.name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.headers.http-header.value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="site-standard-profile-request.url" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="location.name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="location.country.code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="industry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceHeadline" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLocationName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceCountryCode" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceIndustry" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="True" type="Boolean" /></view><view name="Main Fields"><column name="" property="ListObjectName" value="LinkedIn_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="first-name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="last-name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="headline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="picture-url" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.url" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.headers.total" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.headers.http-header.name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.headers.http-header.value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="site-standard-profile-request.url" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="location.name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="location.country.code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="industry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceIndustry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Export Fields"><column name="" property="ListObjectName" value="LinkedIn_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="first-name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="last-name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="headline" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="picture-url" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.url" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.headers.total" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.headers.http-header.name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="api-standard-profile-request.headers.http-header.value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="site-standard-profile-request.url" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="location.name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="location.country.code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="industry" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceIndustry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /></view></views></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'text',N'Yahoo! - HTM',N'<table name="text.Yahoo! - HTM"><columnFormats><column name="" property="ListObjectName" value="YahooHTM_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium2" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="True" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="name" property="Address" value="$C$4" type="String" /><column name="email" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="email" property="Address" value="$D$4" type="String" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="Address" value="$E$4" type="String" /><column name="Source" property="FormulaR1C1" value="=&quot;YahooHTM&quot;" type="String" /><column name="Source" property="ColumnWidth" value="9.71" type="Double" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="Address" value="$F$4" type="String" /><column name="SourceID" property="FormulaR1C1" value="=[@email]" type="String" /><column name="SourceID" property="ColumnWidth" value="21.86" type="Double" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="Address" value="$G$4" type="String" /><column name="SourceFirstName" property="FormulaR1C1" value="=TRIM(LEFT([@name],LEN([@name])-LEN([@SourceLastName])))" type="String" /><column name="SourceFirstName" property="ColumnWidth" value="17.86" type="Double" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="Address" value="$H$4" type="String" /><column name="SourceLastName" property="FormulaR1C1" value="=IFERROR(RIGHT([@name],LEN([@name])-FIND(&quot;|&quot;,SUBSTITUTE([@name],&quot; &quot;,&quot;|&quot;,LEN([@name])-LEN(SUBSTITUTE([@name],&quot; &quot;,&quot;&quot;))))),&quot;&quot;)" type="String" /><column name="SourceLastName" property="ColumnWidth" value="17.43" type="Double" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="Address" value="$I$4" type="String" /><column name="SourceEmail" property="FormulaR1C1" value="=[@SourceID]" type="String" /><column name="SourceEmail" property="ColumnWidth" value="29" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="2" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats><views><view name="All"><column name="" property="ListObjectName" value="YahooHTM_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="email" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Source Fields"><column name="" property="ListObjectName" value="YahooHTM_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="email" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /></view><view name="Source Main Fields"><column name="" property="ListObjectName" value="YahooHTM_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="email" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /></view><view name="Main Fields"><column name="" property="ListObjectName" value="YahooHTM_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="email" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Export Fields"><column name="" property="ListObjectName" value="YahooHTM_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="email" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /></view></views></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'http',N'Yahoo! - API',N'<table name="http.Yahoo! - API"><columnFormats><column name="" property="ListObjectName" value="Yahoo_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium2" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="True" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="yahoo:uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="yahoo:uri" property="Address" value="$C$4" type="String" /><column name="yahoo:created" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="yahoo:created" property="Address" value="$D$4" type="String" /><column name="yahoo:created" property="NumberFormat" value="m/d/yyyy h:mm" type="String" /><column name="yahoo:updated" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="yahoo:updated" property="Address" value="$E$4" type="String" /><column name="yahoo:updated" property="NumberFormat" value="m/d/yyyy h:mm" type="String" /><column name="isConnection" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="isConnection" property="Address" value="$F$4" type="String" /><column name="id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="id" property="Address" value="$G$4" type="String" /><column name="fields.yahoo:uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.yahoo:uri" property="Address" value="$H$4" type="String" /><column name="fields.yahoo:created" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.yahoo:created" property="Address" value="$I$4" type="String" /><column name="fields.yahoo:created" property="NumberFormat" value="m/d/yyyy h:mm" type="String" /><column name="fields.yahoo:updated" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.yahoo:updated" property="Address" value="$J$4" type="String" /><column name="fields.yahoo:updated" property="NumberFormat" value="m/d/yyyy h:mm" type="String" /><column name="fields.id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.id" property="Address" value="$K$4" type="String" /><column name="fields.value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.value" property="Address" value="$L$4" type="String" /><column name="fields.editedBy" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.editedBy" property="Address" value="$M$4" type="String" /><column name="fields.value.givenName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.value.givenName" property="Address" value="$N$4" type="String" /><column name="fields.value.familyName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.value.familyName" property="Address" value="$O$4" type="String" /><column name="categories.yahoo:uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="categories.yahoo:uri" property="Address" value="$P$4" type="String" /><column name="categories.yahoo:created" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="categories.yahoo:created" property="Address" value="$Q$4" type="String" /><column name="categories.yahoo:created" property="NumberFormat" value="m/d/yyyy h:mm" type="String" /><column name="categories.yahoo:updated" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="categories.yahoo:updated" property="Address" value="$R$4" type="String" /><column name="categories.yahoo:updated" property="NumberFormat" value="m/d/yyyy h:mm" type="String" /><column name="categories.id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="categories.id" property="Address" value="$S$4" type="String" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="Address" value="$T$4" type="String" /><column name="Source" property="FormulaR1C1" value="=&quot;Yahoo&quot;" type="String" /><column name="Source" property="ColumnWidth" value="8.57" type="Double" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="Address" value="$U$4" type="String" /><column name="SourceID" property="FormulaR1C1" value="=RIGHT([@[yahoo:uri]],LEN([@[yahoo:uri]])-FIND(&quot;|&quot;,SUBSTITUTE([@[yahoo:uri]],&quot;/&quot;,&quot;|&quot;,LEN([@[yahoo:uri]])-LEN(SUBSTITUTE([@[yahoo:uri]],&quot;/&quot;,&quot;&quot;)))))" type="String" /><column name="SourceID" property="ColumnWidth" value="10.43" type="Double" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="Address" value="$V$4" type="String" /><column name="SourceFirstName" property="FormulaR1C1" value="=IF(LEN([@[fields.value.givenName]])&gt;0,[@[fields.value.givenName]],&quot;&quot;)" type="String" /><column name="SourceFirstName" property="ColumnWidth" value="17.86" type="Double" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="Address" value="$W$4" type="String" /><column name="SourceLastName" property="FormulaR1C1" value="=IF(LEN([@[fields.value.familyName]])&gt;0,[@[fields.value.familyName]],&quot;&quot;)" type="String" /><column name="SourceLastName" property="ColumnWidth" value="17.43" type="Double" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="Address" value="$X$4" type="String" /><column name="SourceEmail" property="FormulaR1C1" value="=[@[fields.value]]" type="String" /><column name="SourceEmail" property="ColumnWidth" value="25.29" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="2" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats><views><view name="All"><column name="" property="ListObjectName" value="Yahoo_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="yahoo:uri" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="yahoo:created" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="yahoo:updated" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="isConnection" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="id" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.yahoo:uri" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.yahoo:created" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.yahoo:updated" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.id" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.value" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.editedBy" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.value.givenName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.value.familyName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="categories.yahoo:uri" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="categories.yahoo:created" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="categories.yahoo:updated" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="categories.id" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Source Fields"><column name="" property="ListObjectName" value="Yahoo_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="yahoo:uri" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="yahoo:created" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="yahoo:updated" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="isConnection" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="id" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.yahoo:uri" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.yahoo:created" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.yahoo:updated" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.id" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.value" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.editedBy" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.value.givenName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.value.familyName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="categories.yahoo:uri" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="categories.yahoo:created" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="categories.yahoo:updated" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="categories.id" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /></view><view name="Source Main Fields"><column name="" property="ListObjectName" value="Yahoo_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="yahoo:uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="yahoo:created" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="yahoo:updated" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="isConnection" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.yahoo:uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.yahoo:created" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.yahoo:updated" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.value" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.editedBy" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.value.givenName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.value.familyName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="categories.yahoo:uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="categories.yahoo:created" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="categories.yahoo:updated" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="categories.id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /></view><view name="Main Fields"><column name="" property="ListObjectName" value="Yahoo_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="yahoo:uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="yahoo:created" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="yahoo:updated" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="isConnection" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.yahoo:uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.yahoo:created" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.yahoo:updated" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.value" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.editedBy" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.value.givenName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="fields.value.familyName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="categories.yahoo:uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="categories.yahoo:created" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="categories.yahoo:updated" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="categories.id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Export Fields"><column name="" property="ListObjectName" value="Yahoo_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="yahoo:uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="yahoo:created" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="yahoo:updated" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="isConnection" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.yahoo:uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.yahoo:created" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.yahoo:updated" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.editedBy" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.value.givenName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="fields.value.familyName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="categories.yahoo:uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="categories.yahoo:created" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="categories.yahoo:updated" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="categories.id" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /></view></views></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'text',N'Google - CSV',N'<table name="text.Google - CSV"><columnFormats><column name="" property="ListObjectName" value="GoogleCSV_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium2" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="True" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Name" property="Address" value="$C$4" type="String" /><column name="Given Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Given Name" property="Address" value="$D$4" type="String" /><column name="Additional Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Additional Name" property="Address" value="$E$4" type="String" /><column name="Family Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Family Name" property="Address" value="$F$4" type="String" /><column name="Yomi Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Yomi Name" property="Address" value="$G$4" type="String" /><column name="Given Name Yomi" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Given Name Yomi" property="Address" value="$H$4" type="String" /><column name="Additional Name Yomi" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Additional Name Yomi" property="Address" value="$I$4" type="String" /><column name="Family Name Yomi" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Family Name Yomi" property="Address" value="$J$4" type="String" /><column name="Name Prefix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Name Prefix" property="Address" value="$K$4" type="String" /><column name="Name Suffix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Name Suffix" property="Address" value="$L$4" type="String" /><column name="Initials" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Initials" property="Address" value="$M$4" type="String" /><column name="Nickname" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Nickname" property="Address" value="$N$4" type="String" /><column name="Short Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Short Name" property="Address" value="$O$4" type="String" /><column name="Maiden Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Maiden Name" property="Address" value="$P$4" type="String" /><column name="Birthday" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Birthday" property="Address" value="$Q$4" type="String" /><column name="Gender" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Gender" property="Address" value="$R$4" type="String" /><column name="Location" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Location" property="Address" value="$S$4" type="String" /><column name="Billing Information" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Billing Information" property="Address" value="$T$4" type="String" /><column name="Directory Server" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Directory Server" property="Address" value="$U$4" type="String" /><column name="Mileage" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mileage" property="Address" value="$V$4" type="String" /><column name="Occupation" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Occupation" property="Address" value="$W$4" type="String" /><column name="Hobby" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Hobby" property="Address" value="$X$4" type="String" /><column name="Sensitivity" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Sensitivity" property="Address" value="$Y$4" type="String" /><column name="Priority" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Priority" property="Address" value="$Z$4" type="String" /><column name="Subject" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Subject" property="Address" value="$AA$4" type="String" /><column name="Notes" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Notes" property="Address" value="$AB$4" type="String" /><column name="Group Membership" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Group Membership" property="Address" value="$AC$4" type="String" /><column name="E-mail 1 - Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 1 - Type" property="Address" value="$AD$4" type="String" /><column name="E-mail 1 - Value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 1 - Value" property="Address" value="$AE$4" type="String" /><column name="E-mail 2 - Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 - Type" property="Address" value="$AF$4" type="String" /><column name="E-mail 2 - Value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 - Value" property="Address" value="$AG$4" type="String" /><column name="Website 1 - Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Website 1 - Type" property="Address" value="$AH$4" type="String" /><column name="Website 1 - Value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Website 1 - Value" property="Address" value="$AI$4" type="String" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="Address" value="$AJ$4" type="String" /><column name="Source" property="FormulaR1C1" value="=&quot;GoogleCSV&quot;" type="String" /><column name="Source" property="ColumnWidth" value="10" type="Double" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="Address" value="$AK$4" type="String" /><column name="SourceID" property="FormulaR1C1" value="=IF(ISERROR(FIND(&quot; ::: &quot;,[@[E-mail 1 - Value]])),[@[E-mail 1 - Value]]&amp;&quot;&quot;,LEFT([@[E-mail 1 - Value]],FIND(&quot; ::: &quot;,[@[E-mail 1 - Value]])))" type="String" /><column name="SourceID" property="ColumnWidth" value="21.86" type="Double" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="Address" value="$AL$4" type="String" /><column name="SourceFirstName" property="FormulaR1C1" value="=[@[Given Name]]&amp;IF([@[Additional Name]]=&quot;&quot;,&quot;&quot;,&quot; &quot;&amp;[@[Additional Name]])" type="String" /><column name="SourceFirstName" property="ColumnWidth" value="17.86" type="Double" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="Address" value="$AM$4" type="String" /><column name="SourceLastName" property="FormulaR1C1" value="=[@[Family Name]]&amp;&quot;&quot;" type="String" /><column name="SourceLastName" property="ColumnWidth" value="17.43" type="Double" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="Address" value="$AN$4" type="String" /><column name="SourceEmail" property="FormulaR1C1" value="=IF(ISERROR(FIND(&quot; ::: &quot;,[@[E-mail 1 - Value]])),[@[E-mail 1 - Value]]&amp;&quot;&quot;,LEFT([@[E-mail 1 - Value]],FIND(&quot; ::: &quot;,[@[E-mail 1 - Value]])))" type="String" /><column name="SourceEmail" property="ColumnWidth" value="19.29" type="Double" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceSecondaryEmail" property="Address" value="$AO$4" type="String" /><column name="SourceSecondaryEmail" property="FormulaR1C1" value="=IF(OR([@[E-mail 2 - Value]]=&quot;&quot;,UPPER([@[E-mail 2 - Value]])=UPPER([@SourceEmail])),&quot;&quot;,[@[E-mail 2 - Value]])" type="String" /><column name="SourceSecondaryEmail" property="ColumnWidth" value="23" type="Double" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceProfileUrl" property="Address" value="$AP$4" type="String" /><column name="SourceProfileUrl" property="FormulaR1C1" value="=[@[Website 1 - Value]]&amp;&quot;&quot;" type="String" /><column name="SourceProfileUrl" property="ColumnWidth" value="27.57" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats><views><view name="All"><column name="" property="ListObjectName" value="GoogleCSV_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Given Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Additional Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Family Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Yomi Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Given Name Yomi" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Additional Name Yomi" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Family Name Yomi" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Name Prefix" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Name Suffix" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Initials" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Nickname" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Short Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Maiden Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Birthday" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Gender" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Location" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Billing Information" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Directory Server" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Mileage" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Occupation" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Hobby" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Sensitivity" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Priority" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Subject" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Notes" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Group Membership" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 1 - Type" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 1 - Value" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 - Type" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 - Value" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website 1 - Type" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website 1 - Value" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Source Fields"><column name="" property="ListObjectName" value="GoogleCSV_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Given Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Additional Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Family Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Yomi Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Given Name Yomi" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Additional Name Yomi" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Family Name Yomi" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Name Prefix" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Name Suffix" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Initials" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Nickname" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Short Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Maiden Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Birthday" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Gender" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Location" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Billing Information" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Directory Server" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Mileage" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Occupation" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Hobby" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Sensitivity" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Priority" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Subject" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Notes" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Group Membership" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 1 - Type" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 1 - Value" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 - Type" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 - Value" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website 1 - Type" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website 1 - Value" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="True" type="Boolean" /></view><view name="Source Main Fields"><column name="" property="ListObjectName" value="GoogleCSV_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Given Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Additional Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Family Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Yomi Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Given Name Yomi" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Additional Name Yomi" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Family Name Yomi" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Name Prefix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Name Suffix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Initials" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Nickname" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Short Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Maiden Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Birthday" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Gender" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Location" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Billing Information" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Directory Server" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mileage" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Occupation" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Hobby" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Sensitivity" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Priority" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Subject" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Notes" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Group Membership" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 1 - Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 1 - Value" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 - Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 - Value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Website 1 - Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Website 1 - Value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="True" type="Boolean" /></view><view name="Main Fields"><column name="" property="ListObjectName" value="GoogleCSV_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Given Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Additional Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Family Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Yomi Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Given Name Yomi" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Additional Name Yomi" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Family Name Yomi" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Name Prefix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Name Suffix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Initials" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Nickname" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Short Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Maiden Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Birthday" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Gender" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Location" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Billing Information" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Directory Server" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mileage" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Occupation" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Hobby" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Sensitivity" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Priority" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Subject" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Notes" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Group Membership" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 1 - Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 1 - Value" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 - Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 - Value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Website 1 - Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Website 1 - Value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Export Fields"><column name="" property="ListObjectName" value="GoogleCSV_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Given Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Additional Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Family Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Yomi Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Given Name Yomi" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Additional Name Yomi" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Family Name Yomi" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Name Prefix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Name Suffix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Initials" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Nickname" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Short Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Maiden Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Birthday" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Gender" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Location" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Billing Information" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Directory Server" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mileage" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Occupation" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Hobby" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Sensitivity" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Priority" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Subject" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Notes" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Group Membership" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 1 - Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 1 - Value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 - Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 - Value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Website 1 - Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Website 1 - Value" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /></view></views></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'http',N'Google - API',N'<table name="http.Google - API"><columnFormats><column name="" property="ListObjectName" value="Google_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium2" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="True" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="_RowNum" property="VerticalAlignment" value="-4160" type="Double" /><column name="id.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="id.$t" property="Address" value="$C$4" type="String" /><column name="id.$t" property="VerticalAlignment" value="-4160" type="Double" /><column name="updated.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="updated.$t" property="Address" value="$D$4" type="String" /><column name="updated.$t" property="NumberFormat" value="m/d/yyyy h:mm" type="String" /><column name="updated.$t" property="VerticalAlignment" value="-4160" type="Double" /><column name="title.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="title.$t" property="Address" value="$E$4" type="String" /><column name="title.$t" property="VerticalAlignment" value="-4160" type="Double" /><column name="link.href" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="link.href" property="Address" value="$F$4" type="String" /><column name="link.href" property="VerticalAlignment" value="-4160" type="Double" /><column name="gd$organization.primary" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$organization.primary" property="Address" value="$G$4" type="String" /><column name="gd$organization.primary" property="VerticalAlignment" value="-4160" type="Double" /><column name="gd$organization.gd$orgName.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$organization.gd$orgName.$t" property="Address" value="$H$4" type="String" /><column name="gd$organization.gd$orgName.$t" property="VerticalAlignment" value="-4160" type="Double" /><column name="gd$email.address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$email.address" property="Address" value="$I$4" type="String" /><column name="gd$email.address" property="VerticalAlignment" value="-4160" type="Double" /><column name="gd$email.primary" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$email.primary" property="Address" value="$J$4" type="String" /><column name="gd$email.primary" property="VerticalAlignment" value="-4160" type="Double" /><column name="gd$phoneNumber.uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$phoneNumber.uri" property="Address" value="$K$4" type="String" /><column name="gd$phoneNumber.uri" property="VerticalAlignment" value="-4160" type="Double" /><column name="gd$phoneNumber.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$phoneNumber.$t" property="Address" value="$L$4" type="String" /><column name="gd$phoneNumber.$t" property="VerticalAlignment" value="-4160" type="Double" /><column name="content.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="content.$t" property="Address" value="$M$4" type="String" /><column name="content.$t" property="VerticalAlignment" value="-4160" type="Double" /><column name="gd$organization.gd$orgTitle.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$organization.gd$orgTitle.$t" property="Address" value="$N$4" type="String" /><column name="gd$organization.gd$orgTitle.$t" property="VerticalAlignment" value="-4160" type="Double" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="Address" value="$O$4" type="String" /><column name="Source" property="FormulaR1C1" value="=&quot;Google&quot;" type="String" /><column name="Source" property="ColumnWidth" value="8.57" type="Double" /><column name="Source" property="VerticalAlignment" value="-4160" type="Double" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="Address" value="$P$4" type="String" /><column name="SourceID" property="FormulaR1C1" value="=RIGHT([@[id.$t]],LEN([@[id.$t]])-FIND(&quot;|&quot;,SUBSTITUTE([@[id.$t]],&quot;/&quot;,&quot;|&quot;,LEN([@[id.$t]])-LEN(SUBSTITUTE([@[id.$t]],&quot;/&quot;,&quot;&quot;)))))" type="String" /><column name="SourceID" property="ColumnWidth" value="17.14" type="Double" /><column name="SourceID" property="VerticalAlignment" value="-4160" type="Double" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="Address" value="$Q$4" type="String" /><column name="SourceFirstName" property="FormulaR1C1" value="=TRIM(SUBSTITUTE(LEFT([@[title.$t]],LEN([@[title.$t]])-LEN([@SourceLastName])),&quot;?&quot;,&quot;&quot;))" type="String" /><column name="SourceFirstName" property="ColumnWidth" value="25.29" type="Double" /><column name="SourceFirstName" property="VerticalAlignment" value="-4160" type="Double" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="Address" value="$R$4" type="String" /><column name="SourceLastName" property="FormulaR1C1" value="=IFERROR(SUBSTITUTE(RIGHT([@[title.$t]],LEN([@[title.$t]])-FIND(&quot;|&quot;,SUBSTITUTE([@[title.$t]],&quot; &quot;,&quot;|&quot;,LEN([@[title.$t]])-LEN(SUBSTITUTE([@[title.$t]],&quot; &quot;,&quot;&quot;))))),&quot;?&quot;,&quot;&quot;),&quot;&quot;)" type="String" /><column name="SourceLastName" property="ColumnWidth" value="17.43" type="Double" /><column name="SourceLastName" property="VerticalAlignment" value="-4160" type="Double" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="Address" value="$S$4" type="String" /><column name="SourceEmail" property="FormulaR1C1" value="=[@[gd$email.address]]&amp;&quot;&quot;" type="String" /><column name="SourceEmail" property="ColumnWidth" value="31.71" type="Double" /><column name="SourceEmail" property="VerticalAlignment" value="-4160" type="Double" /><column name="SourceMobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceMobile" property="Address" value="$T$4" type="String" /><column name="SourceMobile" property="FormulaR1C1" value="=[@[gd$phoneNumber.$t]]&amp;&quot;&quot;" type="String" /><column name="SourceMobile" property="ColumnWidth" value="17.29" type="Double" /><column name="SourceMobile" property="VerticalAlignment" value="-4160" type="Double" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="Address" value="$U$4" type="String" /><column name="SourceCompany" property="FormulaR1C1" value="=[@[gd$organization.gd$orgName.$t]]&amp;&quot;&quot;" type="String" /><column name="SourceCompany" property="ColumnWidth" value="17.86" type="Double" /><column name="SourceCompany" property="VerticalAlignment" value="-4160" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="2" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats><views><view name="All"><column name="" property="ListObjectName" value="Google_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="id.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="updated.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="title.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="link.href" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$organization.primary" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$organization.gd$orgName.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$email.address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$email.primary" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$phoneNumber.uri" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$phoneNumber.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="content.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$organization.gd$orgTitle.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Source Fields"><column name="" property="ListObjectName" value="Google_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="id.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="updated.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="title.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="link.href" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$organization.primary" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$organization.gd$orgName.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$email.address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$email.primary" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$phoneNumber.uri" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$phoneNumber.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="content.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$organization.gd$orgTitle.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="True" type="Boolean" /></view><view name="Source Main Fields"><column name="" property="ListObjectName" value="Google_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="id.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="updated.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="title.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="link.href" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$organization.primary" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$organization.gd$orgName.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$email.address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$email.primary" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$phoneNumber.uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$phoneNumber.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="content.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$organization.gd$orgTitle.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="True" type="Boolean" /></view><view name="Main Fields"><column name="" property="ListObjectName" value="Google_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="id.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="updated.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="title.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="link.href" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$organization.primary" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$organization.gd$orgName.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$email.address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="gd$email.primary" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$phoneNumber.uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$phoneNumber.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="content.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$organization.gd$orgTitle.$t" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Export Fields"><column name="" property="ListObjectName" value="Google_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="id.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="updated.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="title.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="link.href" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$organization.primary" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$organization.gd$orgName.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$email.address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$email.primary" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$phoneNumber.uri" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$phoneNumber.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="content.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="gd$organization.gd$orgTitle.$t" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /></view></views></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'text',N'Outlook - CSV',N'<table name="text.Outlook - CSV"><columnFormats><column name="" property="ListObjectName" value="OutlookCSV_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium2" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="True" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="Title" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Title" property="Address" value="$C$4" type="String" /><column name="First Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="First Name" property="Address" value="$D$4" type="String" /><column name="Middle Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Middle Name" property="Address" value="$E$4" type="String" /><column name="Last Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Last Name" property="Address" value="$F$4" type="String" /><column name="Suffix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Suffix" property="Address" value="$G$4" type="String" /><column name="Company" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company" property="Address" value="$H$4" type="String" /><column name="Department" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Department" property="Address" value="$I$4" type="String" /><column name="Job Title" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Job Title" property="Address" value="$J$4" type="String" /><column name="Business Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street" property="Address" value="$K$4" type="String" /><column name="Business Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 2" property="Address" value="$L$4" type="String" /><column name="Business Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 3" property="Address" value="$M$4" type="String" /><column name="Business City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business City" property="Address" value="$N$4" type="String" /><column name="Business State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business State" property="Address" value="$O$4" type="String" /><column name="Business Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Postal Code" property="Address" value="$P$4" type="String" /><column name="Business Country/Region" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Country/Region" property="Address" value="$Q$4" type="String" /><column name="Home Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street" property="Address" value="$R$4" type="String" /><column name="Home Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 2" property="Address" value="$S$4" type="String" /><column name="Home Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 3" property="Address" value="$T$4" type="String" /><column name="Home City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home City" property="Address" value="$U$4" type="String" /><column name="Home State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home State" property="Address" value="$V$4" type="String" /><column name="Home Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Postal Code" property="Address" value="$W$4" type="String" /><column name="Home Country/Region" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Country/Region" property="Address" value="$X$4" type="String" /><column name="Other Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street" property="Address" value="$Y$4" type="String" /><column name="Other Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 2" property="Address" value="$Z$4" type="String" /><column name="Other Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 3" property="Address" value="$AA$4" type="String" /><column name="Other City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other City" property="Address" value="$AB$4" type="String" /><column name="Other State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other State" property="Address" value="$AC$4" type="String" /><column name="Other Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Postal Code" property="Address" value="$AD$4" type="String" /><column name="Other Country/Region" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Country/Region" property="Address" value="$AE$4" type="String" /><column name="Assistant''s Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Assistant''s Phone" property="Address" value="$AF$4" type="String" /><column name="Business Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Fax" property="Address" value="$AG$4" type="String" /><column name="Business Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Phone" property="Address" value="$AH$4" type="String" /><column name="Business Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Phone 2" property="Address" value="$AI$4" type="String" /><column name="Callback" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Callback" property="Address" value="$AJ$4" type="String" /><column name="Car Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Car Phone" property="Address" value="$AK$4" type="String" /><column name="Company Main Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company Main Phone" property="Address" value="$AL$4" type="String" /><column name="Home Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Fax" property="Address" value="$AM$4" type="String" /><column name="Home Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone" property="Address" value="$AN$4" type="String" /><column name="Home Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone 2" property="Address" value="$AO$4" type="String" /><column name="ISDN" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ISDN" property="Address" value="$AP$4" type="String" /><column name="Mobile Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mobile Phone" property="Address" value="$AQ$4" type="String" /><column name="Other Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Fax" property="Address" value="$AR$4" type="String" /><column name="Other Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Phone" property="Address" value="$AS$4" type="String" /><column name="Pager" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Pager" property="Address" value="$AT$4" type="String" /><column name="Primary Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Primary Phone" property="Address" value="$AU$4" type="String" /><column name="Radio Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Radio Phone" property="Address" value="$AV$4" type="String" /><column name="TTY/TDD Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="TTY/TDD Phone" property="Address" value="$AW$4" type="String" /><column name="Telex" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Telex" property="Address" value="$AX$4" type="String" /><column name="Account" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Account" property="Address" value="$AY$4" type="String" /><column name="Anniversary" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Anniversary" property="Address" value="$AZ$4" type="String" /><column name="Assistant''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Assistant''s Name" property="Address" value="$BA$4" type="String" /><column name="Billing Information" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Billing Information" property="Address" value="$BB$4" type="String" /><column name="Birthday" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Birthday" property="Address" value="$BC$4" type="String" /><column name="Business Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Address PO Box" property="Address" value="$BD$4" type="String" /><column name="Categories" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Categories" property="Address" value="$BE$4" type="String" /><column name="Children" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Children" property="Address" value="$BF$4" type="String" /><column name="Directory Server" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Directory Server" property="Address" value="$BG$4" type="String" /><column name="E-mail Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail Address" property="Address" value="$BH$4" type="String" /><column name="E-mail Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail Type" property="Address" value="$BI$4" type="String" /><column name="E-mail Display Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail Display Name" property="Address" value="$BJ$4" type="String" /><column name="E-mail 2 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 Address" property="Address" value="$BK$4" type="String" /><column name="E-mail 2 Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 Type" property="Address" value="$BL$4" type="String" /><column name="E-mail 2 Display Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 Display Name" property="Address" value="$BM$4" type="String" /><column name="E-mail 3 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Address" property="Address" value="$BN$4" type="String" /><column name="E-mail 3 Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Type" property="Address" value="$BO$4" type="String" /><column name="E-mail 3 Display Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Display Name" property="Address" value="$BP$4" type="String" /><column name="Gender" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Gender" property="Address" value="$BQ$4" type="String" /><column name="Government ID Number" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Government ID Number" property="Address" value="$BR$4" type="String" /><column name="Hobby" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Hobby" property="Address" value="$BS$4" type="String" /><column name="Home Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Address PO Box" property="Address" value="$BT$4" type="String" /><column name="Initials" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Initials" property="Address" value="$BU$4" type="String" /><column name="Internet Free Busy" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Internet Free Busy" property="Address" value="$BV$4" type="String" /><column name="Keywords" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Keywords" property="Address" value="$BW$4" type="String" /><column name="Language" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Language" property="Address" value="$BX$4" type="String" /><column name="Location" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Location" property="Address" value="$BY$4" type="String" /><column name="Manager''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Manager''s Name" property="Address" value="$BZ$4" type="String" /><column name="Mileage" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mileage" property="Address" value="$CA$4" type="String" /><column name="Notes" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Notes" property="Address" value="$CB$4" type="String" /><column name="Office Location" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Office Location" property="Address" value="$CC$4" type="String" /><column name="Organizational ID Number" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Organizational ID Number" property="Address" value="$CD$4" type="String" /><column name="Other Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Address PO Box" property="Address" value="$CE$4" type="String" /><column name="Priority" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Priority" property="Address" value="$CF$4" type="String" /><column name="Private" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Private" property="Address" value="$CG$4" type="String" /><column name="Profession" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Profession" property="Address" value="$CH$4" type="String" /><column name="Referred By" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Referred By" property="Address" value="$CI$4" type="String" /><column name="Sensitivity" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Sensitivity" property="Address" value="$CJ$4" type="String" /><column name="Spouse" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Spouse" property="Address" value="$CK$4" type="String" /><column name="User 1" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 1" property="Address" value="$CL$4" type="String" /><column name="User 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 2" property="Address" value="$CM$4" type="String" /><column name="User 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 3" property="Address" value="$CN$4" type="String" /><column name="User 4" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 4" property="Address" value="$CO$4" type="String" /><column name="Web Page" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Web Page" property="Address" value="$CP$4" type="String" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="Address" value="$CQ$4" type="String" /><column name="Source" property="FormulaR1C1" value="=&quot;OutlookCSV&quot;" type="String" /><column name="Source" property="ColumnWidth" value="10.86" type="Double" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="Address" value="$CR$4" type="String" /><column name="SourceID" property="FormulaR1C1" value="=IF([@[E-mail Address]]&gt;&quot;&quot;,[@[E-mail Address]],SUBSTITUTE(SUBSTITUTE(SUBSTITUTE(SUBSTITUTE(SUBSTITUTE([@SourcePhone],&quot;+&quot;,&quot;&quot;),&quot;(&quot;,&quot;&quot;),&quot;)&quot;,&quot;&quot;),&quot; &quot;,&quot;&quot;),&quot;-&quot;,&quot;&quot;))" type="String" /><column name="SourceID" property="ColumnWidth" value="21.86" type="Double" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="Address" value="$CS$4" type="String" /><column name="SourceFirstName" property="FormulaR1C1" value="=[@[First Name]]&amp;IF(LEN([@[Middle Name]])&gt;0,&quot; &quot;&amp;[@[Middle Name]],&quot;&quot;)" type="String" /><column name="SourceFirstName" property="ColumnWidth" value="17.86" type="Double" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="Address" value="$CT$4" type="String" /><column name="SourceLastName" property="FormulaR1C1" value="=[@[Last Name]]&amp;&quot;&quot;" type="String" /><column name="SourceLastName" property="ColumnWidth" value="17.43" type="Double" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="Address" value="$CU$4" type="String" /><column name="SourceEmail" property="FormulaR1C1" value="=[@[E-mail Address]]&amp;&quot;&quot;" type="String" /><column name="SourceEmail" property="ColumnWidth" value="21.86" type="Double" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceSecondaryEmail" property="Address" value="$CV$4" type="String" /><column name="SourceSecondaryEmail" property="FormulaR1C1" value="=[@[E-mail 2 Address]]&amp;&quot;&quot;" type="String" /><column name="SourceSecondaryEmail" property="ColumnWidth" value="23" type="Double" /><column name="SourceMobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceMobile" property="Address" value="$CW$4" type="String" /><column name="SourceMobile" property="FormulaR1C1" value="=[@[Mobile Phone]]&amp;&quot;&quot;" type="String" /><column name="SourceMobile" property="ColumnWidth" value="15.14" type="Double" /><column name="SourcePhone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourcePhone" property="Address" value="$CX$4" type="String" /><column name="SourcePhone" property="FormulaR1C1" value="=IF([@[Business Phone]]&gt;&quot;&quot;,[@[Business Phone]],IF([@[Primary Phone]]&gt;&quot;&quot;,[@[Primary Phone]],IF([@[Home Phone]]&gt;&quot;&quot;,[@[Home Phone]],IF([@[Mobile Phone]]&gt;&quot;&quot;,[@[Mobile Phone]],&quot;&quot;))))" type="String" /><column name="SourcePhone" property="ColumnWidth" value="16.14" type="Double" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="Address" value="$CY$4" type="String" /><column name="SourceCompany" property="FormulaR1C1" value="=[@Company]&amp;&quot;&quot;" type="String" /><column name="SourceCompany" property="ColumnWidth" value="17" type="Double" /><column name="SourceJobTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceJobTitle" property="Address" value="$CZ$4" type="String" /><column name="SourceJobTitle" property="FormulaR1C1" value="=[@[Job Title]]&amp;&quot;&quot;" type="String" /><column name="SourceJobTitle" property="ColumnWidth" value="15.71" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="2" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats><views><view name="All"><column name="" property="ListObjectName" value="OutlookCSV_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="First Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Middle Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Last Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Suffix" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Company" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Department" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Job Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Street" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Street 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Street 3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business City" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business State" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Postal Code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Country/Region" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Street" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Street 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Street 3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home City" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home State" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Postal Code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Country/Region" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Street" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Street 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Street 3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other City" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other State" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Postal Code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Country/Region" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Assistant''s Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Fax" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Phone 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Callback" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Car Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Company Main Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Fax" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Phone 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ISDN" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Mobile Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Fax" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Pager" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Primary Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Radio Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="TTY/TDD Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Telex" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Account" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Anniversary" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Assistant''s Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Billing Information" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Birthday" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Address PO Box" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Categories" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Children" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Directory Server" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail Type" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail Display Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 Type" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 Display Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 3 Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 3 Type" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 3 Display Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Gender" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Government ID Number" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Hobby" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Address PO Box" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Initials" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Internet Free Busy" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Keywords" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Language" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Location" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Manager''s Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Mileage" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Notes" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Office Location" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Organizational ID Number" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Address PO Box" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Priority" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Private" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Profession" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Referred By" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Sensitivity" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Spouse" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="User 1" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="User 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="User 3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="User 4" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Web Page" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Source Fields"><column name="" property="ListObjectName" value="OutlookCSV_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="First Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Middle Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Last Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Suffix" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Company" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Department" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Job Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Street" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Street 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Street 3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business City" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business State" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Postal Code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Country/Region" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Street" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Street 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Street 3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home City" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home State" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Postal Code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Country/Region" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Street" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Street 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Street 3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other City" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other State" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Postal Code" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Country/Region" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Assistant''s Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Fax" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Phone 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Callback" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Car Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Company Main Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Fax" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Phone 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ISDN" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Mobile Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Fax" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Pager" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Primary Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Radio Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="TTY/TDD Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Telex" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Account" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Anniversary" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Assistant''s Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Billing Information" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Birthday" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Address PO Box" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Categories" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Children" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Directory Server" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail Type" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail Display Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 Type" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 2 Display Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 3 Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 3 Type" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail 3 Display Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Gender" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Government ID Number" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Hobby" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Home Address PO Box" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Initials" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Internet Free Busy" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Keywords" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Language" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Location" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Manager''s Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Mileage" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Notes" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Office Location" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Organizational ID Number" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Address PO Box" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Priority" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Private" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Profession" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Referred By" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Sensitivity" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Spouse" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="User 1" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="User 2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="User 3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="User 4" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Web Page" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourcePhone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceJobTitle" property="EntireColumn.Hidden" value="True" type="Boolean" /></view><view name="Source Main Fields"><column name="" property="ListObjectName" value="OutlookCSV_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Title" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="First Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Middle Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Last Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Suffix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Department" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Job Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Country/Region" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Country/Region" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Country/Region" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Assistant''s Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Callback" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Car Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company Main Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ISDN" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mobile Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Pager" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Primary Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Radio Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="TTY/TDD Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Telex" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Account" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Anniversary" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Assistant''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Billing Information" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Birthday" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Categories" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Children" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Directory Server" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail Display Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 Display Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Display Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Gender" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Government ID Number" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Hobby" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Initials" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Internet Free Busy" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Keywords" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Language" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Location" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Manager''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mileage" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Notes" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Office Location" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Organizational ID Number" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Priority" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Private" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Profession" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Referred By" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Sensitivity" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Spouse" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 1" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 4" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Web Page" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourcePhone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceJobTitle" property="EntireColumn.Hidden" value="True" type="Boolean" /></view><view name="Main Fields"><column name="" property="ListObjectName" value="OutlookCSV_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Title" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="First Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Middle Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Last Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Suffix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Department" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Job Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Country/Region" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Country/Region" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Country/Region" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Assistant''s Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Business Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Callback" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Car Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company Main Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ISDN" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mobile Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Other Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Pager" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Primary Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Radio Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="TTY/TDD Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Telex" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Account" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Anniversary" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Assistant''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Billing Information" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Birthday" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Categories" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Children" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Directory Server" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail Address" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="E-mail Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail Display Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 Display Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Display Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Gender" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Government ID Number" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Hobby" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Initials" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Internet Free Busy" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Keywords" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Language" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Location" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Manager''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mileage" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Notes" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Office Location" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Organizational ID Number" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Priority" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Private" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Profession" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Referred By" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Sensitivity" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Spouse" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 1" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 4" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Web Page" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourcePhone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Export Fields"><column name="" property="ListObjectName" value="OutlookCSV_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Title" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="First Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Middle Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Last Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Suffix" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Department" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Job Title" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Country/Region" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Country/Region" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Street 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other City" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other State" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Postal Code" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Country/Region" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Assistant''s Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Callback" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Car Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Company Main Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Phone 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ISDN" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mobile Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Fax" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Pager" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Primary Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Radio Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="TTY/TDD Phone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Telex" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Account" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Anniversary" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Assistant''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Billing Information" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Birthday" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Business Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Categories" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Children" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Directory Server" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail Display Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 2 Display Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Address" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Type" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="E-mail 3 Display Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Gender" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Government ID Number" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Hobby" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Home Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Initials" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Internet Free Busy" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Keywords" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Language" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Location" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Manager''s Name" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Mileage" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Notes" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Office Location" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Organizational ID Number" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Other Address PO Box" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Priority" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Private" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Profession" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Referred By" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Sensitivity" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Spouse" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 1" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 2" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 3" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="User 4" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Web Page" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourcePhone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /></view></views></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'dbo71',N'Countries',N'<table name="dbo71.Countries"><columnFormats><column name="" property="ListObjectName" value="Countries_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium2" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="True" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="CountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="CountryCode" property="Address" value="$C$4" type="String" /><column name="CountryCode" property="ColumnWidth" value="14.14" type="Double" /><column name="CountryCode" property="Validation.Type" value="6" type="Double" /><column name="CountryCode" property="Validation.AlertStyle" value="1" type="Double" /><column name="CountryCode" property="Validation.Operator" value="8" type="Double" /><column name="CountryCode" property="Validation.Formula1" value="2" type="String" /><column name="CountryCode" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="CountryCode" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="CountryCode" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="CountryCode" property="Validation.ErrorMessage" value="The column requires values of the char(2) datatype." type="String" /><column name="CountryCode" property="Validation.ShowInput" value="True" type="Boolean" /><column name="CountryCode" property="Validation.ShowError" value="True" type="Boolean" /><column name="CountryName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="CountryName" property="Address" value="$D$4" type="String" /><column name="CountryName" property="ColumnWidth" value="19.29" type="Double" /><column name="CountryName" property="Validation.Type" value="6" type="Double" /><column name="CountryName" property="Validation.AlertStyle" value="1" type="Double" /><column name="CountryName" property="Validation.Operator" value="8" type="Double" /><column name="CountryName" property="Validation.Formula1" value="255" type="String" /><column name="CountryName" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="CountryName" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="CountryName" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="CountryName" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="CountryName" property="Validation.ShowInput" value="True" type="Boolean" /><column name="CountryName" property="Validation.ShowError" value="True" type="Boolean" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="Address" value="$E$4" type="String" /><column name="Checked" property="ColumnWidth" value="10.14" type="Double" /><column name="Checked" property="Validation.Type" value="1" type="Double" /><column name="Checked" property="Validation.AlertStyle" value="1" type="Double" /><column name="Checked" property="Validation.Operator" value="1" type="Double" /><column name="Checked" property="Validation.Formula1" value="0" type="String" /><column name="Checked" property="Validation.Formula2" value="255" type="String" /><column name="Checked" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Checked" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Checked" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="Checked" property="Validation.ErrorMessage" value="The column requires values of the tinyint datatype." type="String" /><column name="Checked" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Checked" property="Validation.ShowError" value="True" type="Boolean" /><column name="CountryCode" property="FormatConditions(1).AppliesTo.Address" value="$C$4:$C$90" type="String" /><column name="CountryCode" property="FormatConditions(1).Type" value="2" type="Double" /><column name="CountryCode" property="FormatConditions(1).Priority" value="1" type="Double" /><column name="CountryCode" property="FormatConditions(1).Formula1" value="=ISBLANK(C4)" type="String" /><column name="CountryCode" property="FormatConditions(1).NumberFormat" value="General" type="String" /><column name="CountryCode" property="FormatConditions(1).Interior.Color" value="65535" type="Double" /><column name="CountryCode" property="FormatConditions(1).Interior.PatternThemeColor" value="-1" type="Double" /><column name="SortFields(1)" property="KeyfieldName" value="CountryName" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="1" type="Double" /><column name="SortFields(1)" property="DataOption" value="0" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats><views><view name="By CountryCode"><column name="" property="ListObjectName" value="Countries_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="CountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="CountryName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SortFields(1)" property="KeyfieldName" value="CountryCode" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="1" type="Double" /><column name="SortFields(1)" property="DataOption" value="2" type="Double" /></view><view name="By CountryName"><column name="" property="ListObjectName" value="Countries_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="CountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="CountryName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SortFields(1)" property="KeyfieldName" value="CountryName" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="1" type="Double" /><column name="SortFields(1)" property="DataOption" value="2" type="Double" /></view></views></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'xls71',N'viewCountries',N'<table name="xls71.viewCountries"><columnFormats><column name="" property="ListObjectName" value="Countries_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium15" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="CountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="CountryCode" property="Address" value="$C$4" type="String" /><column name="CountryCode" property="ColumnWidth" value="14.14" type="Double" /><column name="CountryCode" property="Validation.Type" value="6" type="Double" /><column name="CountryCode" property="Validation.AlertStyle" value="1" type="Double" /><column name="CountryCode" property="Validation.Operator" value="8" type="Double" /><column name="CountryCode" property="Validation.Formula1" value="2" type="String" /><column name="CountryCode" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="CountryCode" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="CountryCode" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="CountryCode" property="Validation.ErrorMessage" value="The column requires values of the char(2) datatype." type="String" /><column name="CountryCode" property="Validation.ShowInput" value="True" type="Boolean" /><column name="CountryCode" property="Validation.ShowError" value="True" type="Boolean" /><column name="CountryName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="CountryName" property="Address" value="$D$4" type="String" /><column name="CountryName" property="ColumnWidth" value="19.29" type="Double" /><column name="CountryName" property="Validation.Type" value="6" type="Double" /><column name="CountryName" property="Validation.AlertStyle" value="1" type="Double" /><column name="CountryName" property="Validation.Operator" value="8" type="Double" /><column name="CountryName" property="Validation.Formula1" value="255" type="String" /><column name="CountryName" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="CountryName" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="CountryName" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="CountryName" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="CountryName" property="Validation.ShowInput" value="True" type="Boolean" /><column name="CountryName" property="Validation.ShowError" value="True" type="Boolean" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="Address" value="$E$4" type="String" /><column name="Checked" property="ColumnWidth" value="10.14" type="Double" /><column name="Checked" property="HorizontalAlignment" value="-4108" type="Double" /><column name="Checked" property="Font.Size" value="10" type="Double" /><column name="Checked" property="Validation.Type" value="1" type="Double" /><column name="Checked" property="Validation.AlertStyle" value="1" type="Double" /><column name="Checked" property="Validation.Operator" value="1" type="Double" /><column name="Checked" property="Validation.Formula1" value="0" type="String" /><column name="Checked" property="Validation.Formula2" value="255" type="String" /><column name="Checked" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Checked" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Checked" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="Checked" property="Validation.ErrorMessage" value="The column requires values of the tinyint datatype." type="String" /><column name="Checked" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Checked" property="Validation.ShowError" value="True" type="Boolean" /><column name="ContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCount" property="Address" value="$F$4" type="String" /><column name="ContactCount" property="ColumnWidth" value="14.71" type="Double" /><column name="ContactCount" property="Validation.Type" value="1" type="Double" /><column name="ContactCount" property="Validation.AlertStyle" value="1" type="Double" /><column name="ContactCount" property="Validation.Operator" value="1" type="Double" /><column name="ContactCount" property="Validation.Formula1" value="-2147483648" type="String" /><column name="ContactCount" property="Validation.Formula2" value="2147483647" type="String" /><column name="ContactCount" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="ContactCount" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="ContactCount" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="ContactCount" property="Validation.ErrorMessage" value="The column requires values of the int datatype." type="String" /><column name="ContactCount" property="Validation.ShowInput" value="True" type="Boolean" /><column name="ContactCount" property="Validation.ShowError" value="True" type="Boolean" /><column name="HasErrors" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="HasErrors" property="Address" value="$G$4" type="String" /><column name="HasErrors" property="ColumnWidth" value="10.86" type="Double" /><column name="HasErrors" property="HorizontalAlignment" value="-4108" type="Double" /><column name="HasErrors" property="Font.Size" value="10" type="Double" /><column name="HasErrors" property="Validation.Type" value="1" type="Double" /><column name="HasErrors" property="Validation.AlertStyle" value="1" type="Double" /><column name="HasErrors" property="Validation.Operator" value="1" type="Double" /><column name="HasErrors" property="Validation.Formula1" value="-2147483648" type="String" /><column name="HasErrors" property="Validation.Formula2" value="2147483647" type="String" /><column name="HasErrors" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="HasErrors" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="HasErrors" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="HasErrors" property="Validation.ErrorMessage" value="The column requires values of the int datatype." type="String" /><column name="HasErrors" property="Validation.ShowInput" value="True" type="Boolean" /><column name="HasErrors" property="Validation.ShowError" value="True" type="Boolean" /><column name="CountryCode" property="FormatConditions(1).AppliesTo.Address" value="$C$4:$C$90" type="String" /><column name="CountryCode" property="FormatConditions(1).Type" value="2" type="Double" /><column name="CountryCode" property="FormatConditions(1).Priority" value="4" type="Double" /><column name="CountryCode" property="FormatConditions(1).Formula1" value="=ISBLANK(C4)" type="String" /><column name="CountryCode" property="FormatConditions(1).Interior.Color" value="65535" type="Double" /><column name="CountryCode" property="FormatConditions(1).Interior.PatternThemeColor" value="-1" type="Double" /><column name="Checked" property="FormatConditions(1).AppliesTo.Address" value="$E$4:$E$90" type="String" /><column name="Checked" property="FormatConditions(1).Type" value="6" type="Double" /><column name="Checked" property="FormatConditions(1).Priority" value="3" type="Double" /><column name="Checked" property="FormatConditions(1).ShowIconOnly" value="True" type="Boolean" /><column name="Checked" property="FormatConditions(1).IconSet.ID" value="8" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(1).Type" value="3" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(1).Operator" value="7" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(2).Value" value="0.5" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(2).Operator" value="7" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(3).Value" value="1" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(3).Operator" value="7" type="Double" /><column name="ContactCount" property="FormatConditions(1).AppliesTo.Address" value="$F$4:$F$90" type="String" /><column name="ContactCount" property="FormatConditions(1).Type" value="3" type="Double" /><column name="ContactCount" property="FormatConditions(1).Priority" value="1" type="Double" /><column name="ContactCount" property="FormatConditions(1).ColorScaleCriteria(1).Type" value="1" type="Double" /><column name="ContactCount" property="FormatConditions(1).ColorScaleCriteria(1).FormatColor.Color" value="7039480" type="Double" /><column name="ContactCount" property="FormatConditions(1).ColorScaleCriteria(2).Type" value="5" type="Double" /><column name="ContactCount" property="FormatConditions(1).ColorScaleCriteria(2).Value" value="50" type="Double" /><column name="ContactCount" property="FormatConditions(1).ColorScaleCriteria(2).FormatColor.Color" value="8711167" type="Double" /><column name="ContactCount" property="FormatConditions(1).ColorScaleCriteria(3).Type" value="2" type="Double" /><column name="ContactCount" property="FormatConditions(1).ColorScaleCriteria(3).FormatColor.Color" value="8109667" type="Double" /><column name="HasErrors" property="FormatConditions(1).AppliesTo.Address" value="$G$4:$G$90" type="String" /><column name="HasErrors" property="FormatConditions(1).Type" value="6" type="Double" /><column name="HasErrors" property="FormatConditions(1).Priority" value="2" type="Double" /><column name="HasErrors" property="FormatConditions(1).ReverseOrder" value="True" type="Boolean" /><column name="HasErrors" property="FormatConditions(1).ShowIconOnly" value="True" type="Boolean" /><column name="HasErrors" property="FormatConditions(1).IconSet.ID" value="8" type="Double" /><column name="HasErrors" property="FormatConditions(1).IconCriteria(1).Type" value="3" type="Double" /><column name="HasErrors" property="FormatConditions(1).IconCriteria(1).Operator" value="7" type="Double" /><column name="HasErrors" property="FormatConditions(1).IconCriteria(2).Value" value="0.5" type="Double" /><column name="HasErrors" property="FormatConditions(1).IconCriteria(2).Operator" value="7" type="Double" /><column name="HasErrors" property="FormatConditions(1).IconCriteria(3).Value" value="1" type="Double" /><column name="HasErrors" property="FormatConditions(1).IconCriteria(3).Operator" value="7" type="Double" /><column name="SortFields(1)" property="KeyfieldName" value="CountryName" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="1" type="Double" /><column name="SortFields(1)" property="DataOption" value="0" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats><views><view name="By Country Code"><column name="" property="ListObjectName" value="Countries_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="CountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="CountryName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="HasErrors" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SortFields(1)" property="KeyfieldName" value="CountryCode" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="1" type="Double" /><column name="SortFields(1)" property="DataOption" value="2" type="Double" /></view><view name="By Country Name"><column name="" property="ListObjectName" value="Countries_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="CountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="CountryName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="HasErrors" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SortFields(1)" property="KeyfieldName" value="CountryName" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="1" type="Double" /><column name="SortFields(1)" property="DataOption" value="2" type="Double" /></view><view name="By Contact Count"><column name="" property="ListObjectName" value="Countries_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="CountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="CountryName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="HasErrors" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SortFields(1)" property="KeyfieldName" value="ContactCount" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="2" type="Double" /><column name="SortFields(1)" property="DataOption" value="2" type="Double" /><column name="SortFields(2)" property="KeyfieldName" value="CountryName" type="String" /><column name="SortFields(2)" property="SortOn" value="0" type="Double" /><column name="SortFields(2)" property="Order" value="1" type="Double" /><column name="SortFields(2)" property="DataOption" value="2" type="Double" /></view><view name="New Countries"><column name="" property="ListObjectName" value="Countries_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="CountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="CountryName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="HasErrors" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="AutoFilter.Criteria1" value="=" type="String" /><column name="SortFields(1)" property="KeyfieldName" value="CountryName" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="1" type="Double" /><column name="SortFields(1)" property="DataOption" value="2" type="Double" /></view><view name="With Errors"><column name="" property="ListObjectName" value="Countries_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="CountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="CountryName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="HasErrors" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="HasErrors" property="AutoFilter.Criteria1" value="&lt;&gt;" type="String" /><column name="SortFields(1)" property="KeyfieldName" value="CountryName" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="1" type="Double" /><column name="SortFields(1)" property="DataOption" value="2" type="Double" /></view></views></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'dbo71',N'Websites',N'<table name="dbo71.Websites"><columnFormats><column name="" property="ListObjectName" value="Countries_Table113" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium15" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="ID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ID" property="Address" value="$C$4" type="String" /><column name="ID" property="ColumnWidth" value="4.43" type="Double" /><column name="ID" property="Validation.Type" value="1" type="Double" /><column name="ID" property="Validation.AlertStyle" value="1" type="Double" /><column name="ID" property="Validation.Operator" value="1" type="Double" /><column name="ID" property="Validation.Formula1" value="-32768" type="String" /><column name="ID" property="Validation.Formula2" value="32767" type="String" /><column name="ID" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="ID" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="ID" property="Validation.ShowInput" value="True" type="Boolean" /><column name="ID" property="Validation.ShowError" value="True" type="Boolean" /><column name="Domain" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Domain" property="Address" value="$D$4" type="String" /><column name="Domain" property="ColumnWidth" value="21.86" type="Double" /><column name="Domain" property="Validation.Type" value="6" type="Double" /><column name="Domain" property="Validation.AlertStyle" value="1" type="Double" /><column name="Domain" property="Validation.Operator" value="8" type="Double" /><column name="Domain" property="Validation.Formula1" value="255" type="String" /><column name="Domain" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Domain" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Domain" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Domain" property="Validation.ShowError" value="True" type="Boolean" /><column name="Website" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website" property="Address" value="$E$4" type="String" /><column name="Website" property="ColumnWidth" value="12.86" type="Double" /><column name="Website" property="Validation.Type" value="6" type="Double" /><column name="Website" property="Validation.AlertStyle" value="1" type="Double" /><column name="Website" property="Validation.Operator" value="8" type="Double" /><column name="Website" property="Validation.Formula1" value="255" type="String" /><column name="Website" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Website" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Website" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Website" property="Validation.ShowError" value="True" type="Boolean" /><column name="Comment" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Comment" property="Address" value="$F$4" type="String" /><column name="Comment" property="ColumnWidth" value="16.86" type="Double" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="Address" value="$G$4" type="String" /><column name="Checked" property="ColumnWidth" value="10.14" type="Double" /><column name="Checked" property="HorizontalAlignment" value="-4108" type="Double" /><column name="Checked" property="Font.Size" value="10" type="Double" /><column name="Checked" property="Validation.Type" value="1" type="Double" /><column name="Checked" property="Validation.AlertStyle" value="1" type="Double" /><column name="Checked" property="Validation.Operator" value="1" type="Double" /><column name="Checked" property="Validation.Formula1" value="0" type="String" /><column name="Checked" property="Validation.Formula2" value="255" type="String" /><column name="Checked" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Checked" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Checked" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Checked" property="Validation.ShowError" value="True" type="Boolean" /><column name="Checked" property="FormatConditions(1).AppliesTo.Address" value="$G$4:$G$71" type="String" /><column name="Checked" property="FormatConditions(1).Type" value="6" type="Double" /><column name="Checked" property="FormatConditions(1).Priority" value="1" type="Double" /><column name="Checked" property="FormatConditions(1).ShowIconOnly" value="True" type="Boolean" /><column name="Checked" property="FormatConditions(1).IconSet.ID" value="8" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(1).Type" value="3" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(1).Operator" value="7" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(2).Value" value="0.5" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(2).Operator" value="7" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(3).Value" value="1" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(3).Operator" value="7" type="Double" /><column name="SortFields(1)" property="KeyfieldName" value="Domain" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="1" type="Double" /><column name="SortFields(1)" property="DataOption" value="2" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'xls71',N'viewWebsites',N'<table name="xls71.viewWebsites"><columnFormats><column name="" property="ListObjectName" value="Websites_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium15" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="ID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ID" property="Address" value="$C$4" type="String" /><column name="ID" property="ColumnWidth" value="4.43" type="Double" /><column name="ID" property="Validation.Type" value="1" type="Double" /><column name="ID" property="Validation.AlertStyle" value="1" type="Double" /><column name="ID" property="Validation.Operator" value="1" type="Double" /><column name="ID" property="Validation.Formula1" value="-32768" type="String" /><column name="ID" property="Validation.Formula2" value="32767" type="String" /><column name="ID" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="ID" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="ID" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="ID" property="Validation.ErrorMessage" value="The column requires values of the smallint datatype." type="String" /><column name="ID" property="Validation.ShowInput" value="True" type="Boolean" /><column name="ID" property="Validation.ShowError" value="True" type="Boolean" /><column name="Domain" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Domain" property="Address" value="$D$4" type="String" /><column name="Domain" property="ColumnWidth" value="36.14" type="Double" /><column name="Domain" property="Validation.Type" value="6" type="Double" /><column name="Domain" property="Validation.AlertStyle" value="1" type="Double" /><column name="Domain" property="Validation.Operator" value="8" type="Double" /><column name="Domain" property="Validation.Formula1" value="255" type="String" /><column name="Domain" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Domain" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Domain" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="Domain" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="Domain" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Domain" property="Validation.ShowError" value="True" type="Boolean" /><column name="Website" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website" property="Address" value="$E$4" type="String" /><column name="Website" property="ColumnWidth" value="41.71" type="Double" /><column name="Website" property="Validation.Type" value="6" type="Double" /><column name="Website" property="Validation.AlertStyle" value="1" type="Double" /><column name="Website" property="Validation.Operator" value="8" type="Double" /><column name="Website" property="Validation.Formula1" value="255" type="String" /><column name="Website" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Website" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Website" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="Website" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="Website" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Website" property="Validation.ShowError" value="True" type="Boolean" /><column name="Comment" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Comment" property="Address" value="$F$4" type="String" /><column name="Comment" property="ColumnWidth" value="20.29" type="Double" /><column name="Comment" property="Validation.Type" value="6" type="Double" /><column name="Comment" property="Validation.AlertStyle" value="1" type="Double" /><column name="Comment" property="Validation.Operator" value="8" type="Double" /><column name="Comment" property="Validation.Formula1" value="255" type="String" /><column name="Comment" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Comment" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Comment" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="Comment" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="Comment" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Comment" property="Validation.ShowError" value="True" type="Boolean" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="Address" value="$G$4" type="String" /><column name="Checked" property="ColumnWidth" value="10.14" type="Double" /><column name="Checked" property="HorizontalAlignment" value="-4108" type="Double" /><column name="Checked" property="Font.Size" value="10" type="Double" /><column name="Checked" property="Validation.Type" value="1" type="Double" /><column name="Checked" property="Validation.AlertStyle" value="1" type="Double" /><column name="Checked" property="Validation.Operator" value="1" type="Double" /><column name="Checked" property="Validation.Formula1" value="0" type="String" /><column name="Checked" property="Validation.Formula2" value="255" type="String" /><column name="Checked" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Checked" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Checked" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="Checked" property="Validation.ErrorMessage" value="The column requires values of the tinyint datatype." type="String" /><column name="Checked" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Checked" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceDomainContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceDomainContactCount" property="Address" value="$H$4" type="String" /><column name="SourceDomainContactCount" property="ColumnWidth" value="28" type="Double" /><column name="SourceDomainContactCount" property="Validation.Type" value="1" type="Double" /><column name="SourceDomainContactCount" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceDomainContactCount" property="Validation.Operator" value="1" type="Double" /><column name="SourceDomainContactCount" property="Validation.Formula1" value="-2147483648" type="String" /><column name="SourceDomainContactCount" property="Validation.Formula2" value="2147483647" type="String" /><column name="SourceDomainContactCount" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceDomainContactCount" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceDomainContactCount" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceDomainContactCount" property="Validation.ErrorMessage" value="The column requires values of the int datatype." type="String" /><column name="SourceDomainContactCount" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceDomainContactCount" property="Validation.ShowError" value="True" type="Boolean" /><column name="DomainContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="DomainContactCount" property="Address" value="$I$4" type="String" /><column name="DomainContactCount" property="ColumnWidth" value="21.71" type="Double" /><column name="DomainContactCount" property="Validation.Type" value="1" type="Double" /><column name="DomainContactCount" property="Validation.AlertStyle" value="1" type="Double" /><column name="DomainContactCount" property="Validation.Operator" value="1" type="Double" /><column name="DomainContactCount" property="Validation.Formula1" value="-2147483648" type="String" /><column name="DomainContactCount" property="Validation.Formula2" value="2147483647" type="String" /><column name="DomainContactCount" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="DomainContactCount" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="DomainContactCount" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="DomainContactCount" property="Validation.ErrorMessage" value="The column requires values of the int datatype." type="String" /><column name="DomainContactCount" property="Validation.ShowInput" value="True" type="Boolean" /><column name="DomainContactCount" property="Validation.ShowError" value="True" type="Boolean" /><column name="WebsiteContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="WebsiteContactCount" property="Address" value="$J$4" type="String" /><column name="WebsiteContactCount" property="ColumnWidth" value="22.43" type="Double" /><column name="WebsiteContactCount" property="Validation.Type" value="1" type="Double" /><column name="WebsiteContactCount" property="Validation.AlertStyle" value="1" type="Double" /><column name="WebsiteContactCount" property="Validation.Operator" value="1" type="Double" /><column name="WebsiteContactCount" property="Validation.Formula1" value="-2147483648" type="String" /><column name="WebsiteContactCount" property="Validation.Formula2" value="2147483647" type="String" /><column name="WebsiteContactCount" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="WebsiteContactCount" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="WebsiteContactCount" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="WebsiteContactCount" property="Validation.ErrorMessage" value="The column requires values of the int datatype." type="String" /><column name="WebsiteContactCount" property="Validation.ShowInput" value="True" type="Boolean" /><column name="WebsiteContactCount" property="Validation.ShowError" value="True" type="Boolean" /><column name="ContactCategoryCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCategoryCount" property="Address" value="$K$4" type="String" /><column name="ContactCategoryCount" property="ColumnWidth" value="22.71" type="Double" /><column name="ContactCategoryCount" property="Validation.Type" value="1" type="Double" /><column name="ContactCategoryCount" property="Validation.AlertStyle" value="1" type="Double" /><column name="ContactCategoryCount" property="Validation.Operator" value="1" type="Double" /><column name="ContactCategoryCount" property="Validation.Formula1" value="-2147483648" type="String" /><column name="ContactCategoryCount" property="Validation.Formula2" value="2147483647" type="String" /><column name="ContactCategoryCount" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="ContactCategoryCount" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="ContactCategoryCount" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="ContactCategoryCount" property="Validation.ErrorMessage" value="The column requires values of the int datatype." type="String" /><column name="ContactCategoryCount" property="Validation.ShowInput" value="True" type="Boolean" /><column name="ContactCategoryCount" property="Validation.ShowError" value="True" type="Boolean" /><column name="HasErrors" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="HasErrors" property="Address" value="$L$4" type="String" /><column name="HasErrors" property="ColumnWidth" value="10.14" type="Double" /><column name="HasErrors" property="HorizontalAlignment" value="-4108" type="Double" /><column name="HasErrors" property="Font.Size" value="10" type="Double" /><column name="HasErrors" property="Validation.Type" value="1" type="Double" /><column name="HasErrors" property="Validation.AlertStyle" value="1" type="Double" /><column name="HasErrors" property="Validation.Operator" value="1" type="Double" /><column name="HasErrors" property="Validation.Formula1" value="-2147483648" type="String" /><column name="HasErrors" property="Validation.Formula2" value="2147483647" type="String" /><column name="HasErrors" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="HasErrors" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="HasErrors" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="HasErrors" property="Validation.ErrorMessage" value="The column requires values of the int datatype." type="String" /><column name="HasErrors" property="Validation.ShowInput" value="True" type="Boolean" /><column name="HasErrors" property="Validation.ShowError" value="True" type="Boolean" /><column name="ID" property="FormatConditions(1).AppliesTo.Address" value="$C$4:$C$71" type="String" /><column name="ID" property="FormatConditions(1).Type" value="2" type="Double" /><column name="ID" property="FormatConditions(1).Priority" value="6" type="Double" /><column name="ID" property="FormatConditions(1).Formula1" value="=ISBLANK(C4)" type="String" /><column name="ID" property="FormatConditions(1).Interior.Color" value="65535" type="Double" /><column name="ID" property="FormatConditions(1).Interior.PatternThemeColor" value="-1" type="Double" /><column name="Checked" property="FormatConditions(1).AppliesTo.Address" value="$G$4:$G$71" type="String" /><column name="Checked" property="FormatConditions(1).Type" value="6" type="Double" /><column name="Checked" property="FormatConditions(1).Priority" value="25" type="Double" /><column name="Checked" property="FormatConditions(1).ShowIconOnly" value="True" type="Boolean" /><column name="Checked" property="FormatConditions(1).IconSet.ID" value="8" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(1).Type" value="3" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(1).Operator" value="7" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(2).Value" value="0.5" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(2).Operator" value="7" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(3).Value" value="1" type="Double" /><column name="Checked" property="FormatConditions(1).IconCriteria(3).Operator" value="7" type="Double" /><column name="SourceDomainContactCount" property="FormatConditions(1).AppliesTo.Address" value="$H$4:$H$71" type="String" /><column name="SourceDomainContactCount" property="FormatConditions(1).Type" value="3" type="Double" /><column name="SourceDomainContactCount" property="FormatConditions(1).Priority" value="27" type="Double" /><column name="SourceDomainContactCount" property="FormatConditions(1).ColorScaleCriteria(1).Type" value="1" type="Double" /><column name="SourceDomainContactCount" property="FormatConditions(1).ColorScaleCriteria(1).FormatColor.Color" value="7039480" type="Double" /><column name="SourceDomainContactCount" property="FormatConditions(1).ColorScaleCriteria(2).Type" value="5" type="Double" /><column name="SourceDomainContactCount" property="FormatConditions(1).ColorScaleCriteria(2).Value" value="50" type="Double" /><column name="SourceDomainContactCount" property="FormatConditions(1).ColorScaleCriteria(2).FormatColor.Color" value="8711167" type="Double" /><column name="SourceDomainContactCount" property="FormatConditions(1).ColorScaleCriteria(3).Type" value="2" type="Double" /><column name="SourceDomainContactCount" property="FormatConditions(1).ColorScaleCriteria(3).FormatColor.Color" value="8109667" type="Double" /><column name="DomainContactCount" property="FormatConditions(1).AppliesTo.Address" value="$I$4:$I$71" type="String" /><column name="DomainContactCount" property="FormatConditions(1).Type" value="3" type="Double" /><column name="DomainContactCount" property="FormatConditions(1).Priority" value="28" type="Double" /><column name="DomainContactCount" property="FormatConditions(1).ColorScaleCriteria(1).Type" value="1" type="Double" /><column name="DomainContactCount" property="FormatConditions(1).ColorScaleCriteria(1).FormatColor.Color" value="7039480" type="Double" /><column name="DomainContactCount" property="FormatConditions(1).ColorScaleCriteria(2).Type" value="5" type="Double" /><column name="DomainContactCount" property="FormatConditions(1).ColorScaleCriteria(2).Value" value="50" type="Double" /><column name="DomainContactCount" property="FormatConditions(1).ColorScaleCriteria(2).FormatColor.Color" value="8711167" type="Double" /><column name="DomainContactCount" property="FormatConditions(1).ColorScaleCriteria(3).Type" value="2" type="Double" /><column name="DomainContactCount" property="FormatConditions(1).ColorScaleCriteria(3).FormatColor.Color" value="8109667" type="Double" /><column name="WebsiteContactCount" property="FormatConditions(1).AppliesTo.Address" value="$J$4:$J$71" type="String" /><column name="WebsiteContactCount" property="FormatConditions(1).Type" value="3" type="Double" /><column name="WebsiteContactCount" property="FormatConditions(1).Priority" value="29" type="Double" /><column name="WebsiteContactCount" property="FormatConditions(1).ColorScaleCriteria(1).Type" value="1" type="Double" /><column name="WebsiteContactCount" property="FormatConditions(1).ColorScaleCriteria(1).FormatColor.Color" value="7039480" type="Double" /><column name="WebsiteContactCount" property="FormatConditions(1).ColorScaleCriteria(2).Type" value="5" type="Double" /><column name="WebsiteContactCount" property="FormatConditions(1).ColorScaleCriteria(2).Value" value="50" type="Double" /><column name="WebsiteContactCount" property="FormatConditions(1).ColorScaleCriteria(2).FormatColor.Color" value="8711167" type="Double" /><column name="WebsiteContactCount" property="FormatConditions(1).ColorScaleCriteria(3).Type" value="2" type="Double" /><column name="WebsiteContactCount" property="FormatConditions(1).ColorScaleCriteria(3).FormatColor.Color" value="8109667" type="Double" /><column name="HasErrors" property="FormatConditions(1).AppliesTo.Address" value="$L$4:$L$71" type="String" /><column name="HasErrors" property="FormatConditions(1).Type" value="6" type="Double" /><column name="HasErrors" property="FormatConditions(1).Priority" value="26" type="Double" /><column name="HasErrors" property="FormatConditions(1).ReverseOrder" value="True" type="Boolean" /><column name="HasErrors" property="FormatConditions(1).ShowIconOnly" value="True" type="Boolean" /><column name="HasErrors" property="FormatConditions(1).IconSet.ID" value="8" type="Double" /><column name="HasErrors" property="FormatConditions(1).IconCriteria(1).Type" value="3" type="Double" /><column name="HasErrors" property="FormatConditions(1).IconCriteria(1).Operator" value="7" type="Double" /><column name="HasErrors" property="FormatConditions(1).IconCriteria(2).Value" value="0.5" type="Double" /><column name="HasErrors" property="FormatConditions(1).IconCriteria(2).Operator" value="7" type="Double" /><column name="HasErrors" property="FormatConditions(1).IconCriteria(3).Value" value="1" type="Double" /><column name="HasErrors" property="FormatConditions(1).IconCriteria(3).Operator" value="7" type="Double" /><column name="SortFields(1)" property="KeyfieldName" value="ID" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="1" type="Double" /><column name="SortFields(1)" property="DataOption" value="0" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats><views><view name="By Domain"><column name="" property="ListObjectName" value="Countries_Table113" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Domain" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Comment" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceDomainContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="DomainContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="WebsiteContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCategoryCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="HasErrors" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SortFields(1)" property="KeyfieldName" value="Domain" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="1" type="Double" /><column name="SortFields(1)" property="DataOption" value="2" type="Double" /></view><view name="By Contact Count"><column name="" property="ListObjectName" value="Countries_Table113" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Domain" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Comment" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceDomainContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="DomainContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="WebsiteContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCategoryCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="HasErrors" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SortFields(1)" property="KeyfieldName" value="DomainContactCount" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="2" type="Double" /><column name="SortFields(1)" property="DataOption" value="2" type="Double" /><column name="SortFields(2)" property="KeyfieldName" value="Domain" type="String" /><column name="SortFields(2)" property="SortOn" value="0" type="Double" /><column name="SortFields(2)" property="Order" value="1" type="Double" /><column name="SortFields(2)" property="DataOption" value="2" type="Double" /></view><view name="By ID"><column name="" property="ListObjectName" value="Countries_Table113" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Domain" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Comment" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceDomainContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="DomainContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="WebsiteContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCategoryCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="HasErrors" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SortFields(1)" property="KeyfieldName" value="ID" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="1" type="Double" /><column name="SortFields(1)" property="DataOption" value="2" type="Double" /></view><view name="New Websites"><column name="" property="ListObjectName" value="Countries_Table113" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Domain" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Comment" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceDomainContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="DomainContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="WebsiteContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCategoryCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="HasErrors" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="AutoFilter.Criteria1" value="=" type="String" /><column name="SortFields(1)" property="KeyfieldName" value="DomainContactCount" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="2" type="Double" /><column name="SortFields(1)" property="DataOption" value="0" type="Double" /><column name="SortFields(2)" property="KeyfieldName" value="Domain" type="String" /><column name="SortFields(2)" property="SortOn" value="0" type="Double" /><column name="SortFields(2)" property="Order" value="1" type="Double" /><column name="SortFields(2)" property="DataOption" value="0" type="Double" /></view><view name="With Errors"><column name="" property="ListObjectName" value="Countries_Table113" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Domain" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Comment" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Checked" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceDomainContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="DomainContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="WebsiteContactCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCategoryCount" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="HasErrors" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="HasErrors" property="AutoFilter.Criteria1" value="&lt;&gt;" type="String" /><column name="SortFields(1)" property="KeyfieldName" value="DomainContactCount" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="2" type="Double" /><column name="SortFields(1)" property="DataOption" value="0" type="Double" /><column name="SortFields(2)" property="KeyfieldName" value="Domain" type="String" /><column name="SortFields(2)" property="SortOn" value="0" type="Double" /><column name="SortFields(2)" property="Order" value="1" type="Double" /><column name="SortFields(2)" property="DataOption" value="0" type="Double" /></view></views></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'xls71',N'viewSourceContacts',N'<table name="xls71.viewSourceContacts"><columnFormats><column name="" property="ListObjectName" value="SourceContacts_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium15" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="True" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="Address" value="$C$4" type="String" /><column name="Source" property="ColumnWidth" value="9" type="Double" /><column name="Source" property="Validation.Type" value="6" type="Double" /><column name="Source" property="Validation.AlertStyle" value="1" type="Double" /><column name="Source" property="Validation.Operator" value="8" type="Double" /><column name="Source" property="Validation.Formula1" value="50" type="String" /><column name="Source" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Source" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Source" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="Source" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(50) datatype." type="String" /><column name="Source" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Source" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="Address" value="$D$4" type="String" /><column name="SourceID" property="Validation.Type" value="6" type="Double" /><column name="SourceID" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceID" property="Validation.Operator" value="8" type="Double" /><column name="SourceID" property="Validation.Formula1" value="50" type="String" /><column name="SourceID" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceID" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceID" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceID" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(50) datatype." type="String" /><column name="SourceID" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceID" property="Validation.ShowError" value="True" type="Boolean" /><column name="MergeDate" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="MergeDate" property="Address" value="$E$4" type="String" /><column name="MergeDate" property="NumberFormat" value="m/d/yyyy h:mm" type="String" /><column name="MergeDate" property="Validation.Type" value="4" type="Double" /><column name="MergeDate" property="Validation.AlertStyle" value="1" type="Double" /><column name="MergeDate" property="Validation.Operator" value="5" type="Double" /><column name="MergeDate" property="Validation.Formula1" value="01.01.1900" type="String" /><column name="MergeDate" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="MergeDate" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="MergeDate" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="MergeDate" property="Validation.ErrorMessage" value="The column requires values of the datetime datatype." type="String" /><column name="MergeDate" property="Validation.ShowInput" value="True" type="Boolean" /><column name="MergeDate" property="Validation.ShowError" value="True" type="Boolean" /><column name="IsJunk" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="IsJunk" property="Address" value="$F$4" type="String" /><column name="IsJunk" property="ColumnWidth" value="8" type="Double" /><column name="IsJunk" property="Validation.Type" value="1" type="Double" /><column name="IsJunk" property="Validation.AlertStyle" value="1" type="Double" /><column name="IsJunk" property="Validation.Operator" value="1" type="Double" /><column name="IsJunk" property="Validation.Formula1" value="0" type="String" /><column name="IsJunk" property="Validation.Formula2" value="255" type="String" /><column name="IsJunk" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="IsJunk" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="IsJunk" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="IsJunk" property="Validation.ErrorMessage" value="The column requires values of the tinyint datatype." type="String" /><column name="IsJunk" property="Validation.ShowInput" value="True" type="Boolean" /><column name="IsJunk" property="Validation.ShowError" value="True" type="Boolean" /><column name="ContactID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactID" property="Address" value="$G$4" type="String" /><column name="ContactID" property="ColumnWidth" value="11.14" type="Double" /><column name="ContactID" property="Validation.Type" value="1" type="Double" /><column name="ContactID" property="Validation.AlertStyle" value="1" type="Double" /><column name="ContactID" property="Validation.Operator" value="1" type="Double" /><column name="ContactID" property="Validation.Formula1" value="-2147483648" type="String" /><column name="ContactID" property="Validation.Formula2" value="2147483647" type="String" /><column name="ContactID" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="ContactID" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="ContactID" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="ContactID" property="Validation.ErrorMessage" value="The column requires values of the int datatype." type="String" /><column name="ContactID" property="Validation.ShowInput" value="True" type="Boolean" /><column name="ContactID" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="Address" value="$H$4" type="String" /><column name="SourceFirstName" property="ColumnWidth" value="25.29" type="Double" /><column name="SourceFirstName" property="Validation.Type" value="6" type="Double" /><column name="SourceFirstName" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceFirstName" property="Validation.Operator" value="8" type="Double" /><column name="SourceFirstName" property="Validation.Formula1" value="255" type="String" /><column name="SourceFirstName" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceFirstName" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceFirstName" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceFirstName" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceFirstName" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceFirstName" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="Address" value="$I$4" type="String" /><column name="SourceLastName" property="ColumnWidth" value="17.43" type="Double" /><column name="SourceLastName" property="Validation.Type" value="6" type="Double" /><column name="SourceLastName" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceLastName" property="Validation.Operator" value="8" type="Double" /><column name="SourceLastName" property="Validation.Formula1" value="255" type="String" /><column name="SourceLastName" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceLastName" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceLastName" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceLastName" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceLastName" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceLastName" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="Address" value="$J$4" type="String" /><column name="SourceEmail" property="ColumnWidth" value="31.71" type="Double" /><column name="SourceEmail" property="Validation.Type" value="6" type="Double" /><column name="SourceEmail" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceEmail" property="Validation.Operator" value="8" type="Double" /><column name="SourceEmail" property="Validation.Formula1" value="255" type="String" /><column name="SourceEmail" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceEmail" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceEmail" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceEmail" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceEmail" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceEmail" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceSecondaryEmail" property="Address" value="$K$4" type="String" /><column name="SourceSecondaryEmail" property="ColumnWidth" value="23" type="Double" /><column name="SourceSecondaryEmail" property="Validation.Type" value="6" type="Double" /><column name="SourceSecondaryEmail" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceSecondaryEmail" property="Validation.Operator" value="8" type="Double" /><column name="SourceSecondaryEmail" property="Validation.Formula1" value="255" type="String" /><column name="SourceSecondaryEmail" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceSecondaryEmail" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceSecondaryEmail" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceSecondaryEmail" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceSecondaryEmail" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceSecondaryEmail" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceMobile" property="Address" value="$L$4" type="String" /><column name="SourceMobile" property="ColumnWidth" value="17.29" type="Double" /><column name="SourceMobile" property="Validation.Type" value="6" type="Double" /><column name="SourceMobile" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceMobile" property="Validation.Operator" value="8" type="Double" /><column name="SourceMobile" property="Validation.Formula1" value="255" type="String" /><column name="SourceMobile" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceMobile" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceMobile" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceMobile" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceMobile" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceMobile" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourcePhone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourcePhone" property="Address" value="$M$4" type="String" /><column name="SourcePhone" property="ColumnWidth" value="14.29" type="Double" /><column name="SourcePhone" property="Validation.Type" value="6" type="Double" /><column name="SourcePhone" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourcePhone" property="Validation.Operator" value="8" type="Double" /><column name="SourcePhone" property="Validation.Formula1" value="255" type="String" /><column name="SourcePhone" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourcePhone" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourcePhone" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourcePhone" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourcePhone" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourcePhone" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="Address" value="$N$4" type="String" /><column name="SourceCompany" property="ColumnWidth" value="17.86" type="Double" /><column name="SourceCompany" property="Validation.Type" value="6" type="Double" /><column name="SourceCompany" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceCompany" property="Validation.Operator" value="8" type="Double" /><column name="SourceCompany" property="Validation.Formula1" value="255" type="String" /><column name="SourceCompany" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceCompany" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceCompany" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceCompany" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceCompany" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceCompany" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceJobTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceJobTitle" property="Address" value="$O$4" type="String" /><column name="SourceJobTitle" property="ColumnWidth" value="20.43" type="Double" /><column name="SourceJobTitle" property="Validation.Type" value="6" type="Double" /><column name="SourceJobTitle" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceJobTitle" property="Validation.Operator" value="8" type="Double" /><column name="SourceJobTitle" property="Validation.Formula1" value="255" type="String" /><column name="SourceJobTitle" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceJobTitle" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceJobTitle" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceJobTitle" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceJobTitle" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceJobTitle" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceHeadline" property="Address" value="$P$4" type="String" /><column name="SourceHeadline" property="ColumnWidth" value="12.57" type="Double" /><column name="SourceHeadline" property="Validation.Type" value="6" type="Double" /><column name="SourceHeadline" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceHeadline" property="Validation.Operator" value="8" type="Double" /><column name="SourceHeadline" property="Validation.Formula1" value="255" type="String" /><column name="SourceHeadline" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceHeadline" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceHeadline" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceHeadline" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceHeadline" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceHeadline" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceIndustry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceIndustry" property="Address" value="$Q$4" type="String" /><column name="SourceIndustry" property="ColumnWidth" value="16.71" type="Double" /><column name="SourceIndustry" property="Validation.Type" value="6" type="Double" /><column name="SourceIndustry" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceIndustry" property="Validation.Operator" value="8" type="Double" /><column name="SourceIndustry" property="Validation.Formula1" value="255" type="String" /><column name="SourceIndustry" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceIndustry" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceIndustry" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceIndustry" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceIndustry" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceIndustry" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceLocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLocationName" property="Address" value="$R$4" type="String" /><column name="SourceLocationName" property="ColumnWidth" value="16" type="Double" /><column name="SourceLocationName" property="Validation.Type" value="6" type="Double" /><column name="SourceLocationName" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceLocationName" property="Validation.Operator" value="8" type="Double" /><column name="SourceLocationName" property="Validation.Formula1" value="255" type="String" /><column name="SourceLocationName" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceLocationName" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceLocationName" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceLocationName" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceLocationName" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceLocationName" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceCountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCountryCode" property="Address" value="$S$4" type="String" /><column name="SourceCountryCode" property="ColumnWidth" value="21.57" type="Double" /><column name="SourceCountryCode" property="Validation.Type" value="6" type="Double" /><column name="SourceCountryCode" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceCountryCode" property="Validation.Operator" value="8" type="Double" /><column name="SourceCountryCode" property="Validation.Formula1" value="2" type="String" /><column name="SourceCountryCode" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceCountryCode" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceCountryCode" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceCountryCode" property="Validation.ErrorMessage" value="The column requires values of the char(2) datatype." type="String" /><column name="SourceCountryCode" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceCountryCode" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceProfileUrl" property="Address" value="$T$4" type="String" /><column name="SourceProfileUrl" property="ColumnWidth" value="20.43" type="Double" /><column name="SourceProfileUrl" property="Validation.Type" value="6" type="Double" /><column name="SourceProfileUrl" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceProfileUrl" property="Validation.Operator" value="8" type="Double" /><column name="SourceProfileUrl" property="Validation.Formula1" value="255" type="String" /><column name="SourceProfileUrl" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceProfileUrl" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceProfileUrl" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceProfileUrl" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceProfileUrl" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceProfileUrl" property="Validation.ShowError" value="True" type="Boolean" /><column name="Source" property="FormatConditions(1).ColumnsCount" value="2" type="Double" /><column name="Source" property="FormatConditions(1).AppliesTo.Address" value="$C$4:$D$4" type="String" /><column name="Source" property="FormatConditions(1).Type" value="2" type="Double" /><column name="Source" property="FormatConditions(1).Priority" value="1" type="Double" /><column name="Source" property="FormatConditions(1).Formula1" value="=ISBLANK(C4)" type="String" /><column name="Source" property="FormatConditions(1).Interior.Color" value="65535" type="Double" /><column name="Source" property="FormatConditions(1).Interior.PatternThemeColor" value="-1" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="2" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats><views><view name="All"><column name="" property="ListObjectName" value="SourceContacts" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="MergeDate" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="IsJunk" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourcePhone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceIndustry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Main Fields"><column name="" property="ListObjectName" value="SourceContacts" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="MergeDate" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="IsJunk" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourcePhone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceIndustry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /></view></views></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'xls71',N'viewContacts',N'<table name="xls71.viewContacts"><columnFormats><column name="" property="ListObjectName" value="Contacts_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium15" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="True" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="ID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ID" property="Address" value="$C$4" type="String" /><column name="ID" property="ColumnWidth" value="4.43" type="Double" /><column name="ContactStatus" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactStatus" property="Address" value="$D$4" type="String" /><column name="ContactStatus" property="ColumnWidth" value="2.14" type="Double" /><column name="ContactStatus" property="HorizontalAlignment" value="-4108" type="Double" /><column name="ContactStatus" property="Font.Size" value="10" type="Double" /><column name="IsDoubleName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="IsDoubleName" property="Address" value="$E$4" type="String" /><column name="IsDoubleName" property="ColumnWidth" value="3.14" type="Double" /><column name="ContactCategory" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCategory" property="Address" value="$F$4" type="String" /><column name="ContactCategory" property="ColumnWidth" value="17.29" type="Double" /><column name="FirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="FirstName" property="Address" value="$G$4" type="String" /><column name="FirstName" property="ColumnWidth" value="25.29" type="Double" /><column name="LastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LastName" property="Address" value="$H$4" type="String" /><column name="LastName" property="ColumnWidth" value="29.71" type="Double" /><column name="Email" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Email" property="Address" value="$I$4" type="String" /><column name="Email" property="ColumnWidth" value="31.71" type="Double" /><column name="Mobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Mobile" property="Address" value="$J$4" type="String" /><column name="Mobile" property="ColumnWidth" value="17.29" type="Double" /><column name="Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Phone" property="Address" value="$K$4" type="String" /><column name="Phone" property="ColumnWidth" value="8.57" type="Double" /><column name="Country" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Country" property="Address" value="$L$4" type="String" /><column name="Country" property="ColumnWidth" value="19.29" type="Double" /><column name="Domain" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Domain" property="Address" value="$M$4" type="String" /><column name="Website" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website" property="Address" value="$N$4" type="String" /><column name="Website" property="ColumnWidth" value="27.71" type="Double" /><column name="Company" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Company" property="Address" value="$O$4" type="String" /><column name="Company" property="ColumnWidth" value="29" type="Double" /><column name="JobTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="JobTitle" property="Address" value="$P$4" type="String" /><column name="JobTitle" property="ColumnWidth" value="19.14" type="Double" /><column name="LinkedInHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LinkedInHeadline" property="Address" value="$Q$4" type="String" /><column name="LinkedInHeadline" property="ColumnWidth" value="22.29" type="Double" /><column name="LinkedInProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LinkedInProfileUrl" property="Address" value="$R$4" type="String" /><column name="LinkedInProfileUrl" property="ColumnWidth" value="28.43" type="Double" /><column name="GooglePlusProfileUrl" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="GooglePlusProfileUrl" property="Address" value="$S$4" type="String" /><column name="Industry" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Industry" property="Address" value="$T$4" type="String" /><column name="LocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LocationName" property="Address" value="$U$4" type="String" /><column name="LocationName" property="ColumnWidth" value="19.14" type="Double" /><column name="String1" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="String1" property="Address" value="$V$4" type="String" /><column name="String1" property="ColumnWidth" value="19.14" type="Double" /><column name="String2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="String2" property="Address" value="$W$4" type="String" /><column name="String2" property="ColumnWidth" value="19.14" type="Double" /><column name="String3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="String3" property="Address" value="$X$4" type="String" /><column name="String3" property="ColumnWidth" value="19.14" type="Double" /><column name="Number1" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Number1" property="Address" value="$Y$4" type="String" /><column name="Number1" property="ColumnWidth" value="19.14" type="Double" /><column name="Number2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Number2" property="Address" value="$Z$4" type="String" /><column name="Number2" property="ColumnWidth" value="19.14" type="Double" /><column name="Number3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Number3" property="Address" value="$AA$4" type="String" /><column name="Number3" property="ColumnWidth" value="19.14" type="Double" /><column name="ContactStatus" property="FormatConditions(1).AppliesTo.Address" value="$D$4" type="String" /><column name="ContactStatus" property="FormatConditions(1).Type" value="6" type="Double" /><column name="ContactStatus" property="FormatConditions(1).Priority" value="30" type="Double" /><column name="ContactStatus" property="FormatConditions(1).ShowIconOnly" value="True" type="Boolean" /><column name="ContactStatus" property="FormatConditions(1).IconSet.ID" value="13" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(1).Type" value="3" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(1).Operator" value="7" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(2).Value" value="1" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(2).Operator" value="7" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(3).Value" value="2" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(3).Operator" value="7" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(4).Value" value="3" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(4).Operator" value="7" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="2" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats><views><view name="All"><column name="" property="ListObjectName" value="SourceContacts8" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactStatus" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="IsDoubleName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCategory" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="FirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Email" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Mobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Domain" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Company" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LinkedInHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LinkedInProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="GooglePlusProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Main Fields"><column name="" property="ListObjectName" value="SourceContacts8" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactStatus" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="IsDoubleName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCategory" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="FirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Email" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Mobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Country" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Domain" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Website" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Company" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LinkedInHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LinkedInProfileUrl" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="GooglePlusProfileUrl" property="EntireColumn.Hidden" value="True" type="Boolean" /></view><view name="New and Updated"><column name="" property="ListObjectName" value="Contacts_Table" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactStatus" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="IsDoubleName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCategory" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="FirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Email" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Mobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Country" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Domain" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Website" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Company" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LinkedInHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LinkedInProfileUrl" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="GooglePlusProfileUrl" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Industry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactStatus" property="AutoFilter.Criteria1" value="=2" type="String" /></view></views></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'xls71',N'uspSourceContactsByContactID',N'<table name="xls71.uspSourceContactsByContactID"><columnFormats><column name="" property="ListObjectName" value="ContactDetails_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium2" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="True" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$3" type="String" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="Address" value="$C$3" type="String" /><column name="Source" property="ColumnWidth" value="11.29" type="Double" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="Address" value="$D$3" type="String" /><column name="MergeDate" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="MergeDate" property="Address" value="$E$3" type="String" /><column name="MergeDate" property="NumberFormat" value="m/d/yyyy" type="String" /><column name="IsJunk" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="IsJunk" property="Address" value="$F$3" type="String" /><column name="IsJunk" property="ColumnWidth" value="8" type="Double" /><column name="IsJunk" property="HorizontalAlignment" value="-4108" type="Double" /><column name="IsJunk" property="Font.Size" value="10" type="Double" /><column name="ContactID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactID" property="Address" value="$G$3" type="String" /><column name="ContactID" property="ColumnWidth" value="11.14" type="Double" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="Address" value="$H$3" type="String" /><column name="SourceFirstName" property="ColumnWidth" value="19" type="Double" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="Address" value="$I$3" type="String" /><column name="SourceLastName" property="ColumnWidth" value="18" type="Double" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="Address" value="$J$3" type="String" /><column name="SourceEmail" property="ColumnWidth" value="27.29" type="Double" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceSecondaryEmail" property="Address" value="$K$3" type="String" /><column name="SourceMobile" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceMobile" property="Address" value="$L$3" type="String" /><column name="SourcePhone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourcePhone" property="Address" value="$M$3" type="String" /><column name="SourceCompany" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceCompany" property="Address" value="$N$3" type="String" /><column name="SourceHeadline" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceHeadline" property="Address" value="$O$3" type="String" /><column name="SourceIndustry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceIndustry" property="Address" value="$P$3" type="String" /><column name="SourceIndustry" property="ColumnWidth" value="51.86" type="Double" /><column name="SourceLocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLocationName" property="Address" value="$Q$3" type="String" /><column name="SourceLocationName" property="ColumnWidth" value="34.43" type="Double" /><column name="SourceCountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCountryCode" property="Address" value="$R$3" type="String" /><column name="SourceCountryCode" property="ColumnWidth" value="43" type="Double" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceProfileUrl" property="Address" value="$S$3" type="String" /><column name="SourceProfileUrl" property="ColumnWidth" value="20.43" type="Double" /><column name="SourceJobTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceJobTitle" property="Address" value="$T$3" type="String" /><column name="SourceJobTitle" property="ColumnWidth" value="20.43" type="Double" /><column name="IsJunk" property="FormatConditions(1).AppliesTo.Address" value="$F$3" type="String" /><column name="IsJunk" property="FormatConditions(1).Type" value="6" type="Double" /><column name="IsJunk" property="FormatConditions(1).Priority" value="24" type="Double" /><column name="IsJunk" property="FormatConditions(1).ReverseOrder" value="True" type="Boolean" /><column name="IsJunk" property="FormatConditions(1).ShowIconOnly" value="True" type="Boolean" /><column name="IsJunk" property="FormatConditions(1).IconSet.ID" value="8" type="Double" /><column name="IsJunk" property="FormatConditions(1).IconCriteria(1).Type" value="3" type="Double" /><column name="IsJunk" property="FormatConditions(1).IconCriteria(1).Operator" value="7" type="Double" /><column name="IsJunk" property="FormatConditions(1).IconCriteria(2).Value" value="0.5" type="Double" /><column name="IsJunk" property="FormatConditions(1).IconCriteria(2).Operator" value="7" type="Double" /><column name="IsJunk" property="FormatConditions(1).IconCriteria(3).Value" value="1" type="Double" /><column name="IsJunk" property="FormatConditions(1).IconCriteria(3).Operator" value="7" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="1" type="Double" /><column name="" property="PageSetup.Zoom" value="100" type="Double" /></columnFormats><views><view name="All"><column name="" property="ListObjectName" value="ContactDetails_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="MergeDate" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="IsJunk" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourcePhone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceIndustry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Main Fields"><column name="" property="ListObjectName" value="ContactDetails_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="MergeDate" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="IsJunk" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourcePhone" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceTitle" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceIndustry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /></view></views></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'xls71',N'uspContactsByCountryName',N'<table name="xls71.uspContactsByCountryName"><columnFormats><column name="" property="ListObjectName" value="CountryDetails_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium2" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="True" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$3" type="String" /><column name="ID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ID" property="Address" value="$C$3" type="String" /><column name="ID" property="ColumnWidth" value="4.43" type="Double" /><column name="ContactStatus" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactStatus" property="Address" value="$D$3" type="String" /><column name="ContactStatus" property="ColumnWidth" value="14.86" type="Double" /><column name="ContactStatus" property="HorizontalAlignment" value="-4108" type="Double" /><column name="ContactStatus" property="VerticalAlignment" value="-4160" type="Double" /><column name="ContactStatus" property="Font.Size" value="10" type="Double" /><column name="IsDoubleName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="IsDoubleName" property="Address" value="$E$3" type="String" /><column name="IsDoubleName" property="ColumnWidth" value="15.86" type="Double" /><column name="ContactCategory" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCategory" property="Address" value="$F$3" type="String" /><column name="ContactCategory" property="ColumnWidth" value="17.29" type="Double" /><column name="FirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="FirstName" property="Address" value="$G$3" type="String" /><column name="FirstName" property="ColumnWidth" value="11.71" type="Double" /><column name="LastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LastName" property="Address" value="$H$3" type="String" /><column name="LastName" property="ColumnWidth" value="21.71" type="Double" /><column name="Email" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Email" property="Address" value="$I$3" type="String" /><column name="Email" property="ColumnWidth" value="30" type="Double" /><column name="Mobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Mobile" property="Address" value="$J$3" type="String" /><column name="Mobile" property="ColumnWidth" value="9" type="Double" /><column name="Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Phone" property="Address" value="$K$3" type="String" /><column name="Phone" property="ColumnWidth" value="8.29" type="Double" /><column name="Country" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Country" property="Address" value="$L$3" type="String" /><column name="Country" property="ColumnWidth" value="11.29" type="Double" /><column name="Domain" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Domain" property="Address" value="$M$3" type="String" /><column name="Domain" property="ColumnWidth" value="11.43" type="Double" /><column name="Website" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website" property="Address" value="$N$3" type="String" /><column name="Website" property="ColumnWidth" value="10.14" type="Double" /><column name="Company" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Company" property="Address" value="$O$3" type="String" /><column name="Company" property="ColumnWidth" value="27.57" type="Double" /><column name="LinkedInHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LinkedInHeadline" property="Address" value="$P$3" type="String" /><column name="LinkedInHeadline" property="ColumnWidth" value="18.43" type="Double" /><column name="Industry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Industry" property="Address" value="$Q$3" type="String" /><column name="Industry" property="ColumnWidth" value="47.43" type="Double" /><column name="LocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LocationName" property="Address" value="$R$3" type="String" /><column name="LocationName" property="ColumnWidth" value="9.86" type="Double" /><column name="LinkedInProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LinkedInProfileUrl" property="Address" value="$S$3" type="String" /><column name="LinkedInProfileUrl" property="ColumnWidth" value="15.43" type="Double" /><column name="GooglePlusProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="GooglePlusProfileUrl" property="Address" value="$T$3" type="String" /><column name="GooglePlusProfileUrl" property="ColumnWidth" value="70.71" type="Double" /><column name="String1" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="String1" property="Address" value="$U$3" type="String" /><column name="String1" property="ColumnWidth" value="21.71" type="Double" /><column name="String2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="String2" property="Address" value="$V$3" type="String" /><column name="String2" property="ColumnWidth" value="21.71" type="Double" /><column name="String3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="String3" property="Address" value="$W$3" type="String" /><column name="String3" property="ColumnWidth" value="21.71" type="Double" /><column name="Number1" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Number1" property="Address" value="$X$3" type="String" /><column name="Number1" property="ColumnWidth" value="21.71" type="Double" /><column name="Number2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Number2" property="Address" value="$Y$3" type="String" /><column name="Number2" property="ColumnWidth" value="21.71" type="Double" /><column name="Number3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Number3" property="Address" value="$Z$3" type="String" /><column name="Number3" property="ColumnWidth" value="21.71" type="Double" /><column name="JobTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="JobTitle" property="Address" value="$AA$3" type="String" /><column name="JobTitle" property="ColumnWidth" value="21.71" type="Double" /><column name="ContactStatus" property="FormatConditions(1).AppliesTo.Address" value="$D$3" type="String" /><column name="ContactStatus" property="FormatConditions(1).Type" value="6" type="Double" /><column name="ContactStatus" property="FormatConditions(1).Priority" value="31" type="Double" /><column name="ContactStatus" property="FormatConditions(1).ShowIconOnly" value="True" type="Boolean" /><column name="ContactStatus" property="FormatConditions(1).IconSet.ID" value="13" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(1).Type" value="3" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(1).Operator" value="7" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(2).Value" value="1" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(2).Operator" value="7" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(3).Value" value="2" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(3).Operator" value="7" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(4).Value" value="3" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(4).Operator" value="7" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="2" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'xls71',N'uspContactsByDomain',N'<table name="xls71.uspContactsByDomain"><columnFormats><column name="" property="ListObjectName" value="WebsiteDetails_Table1" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium2" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="True" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$3" type="String" /><column name="ID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ID" property="Address" value="$C$3" type="String" /><column name="ID" property="ColumnWidth" value="4.43" type="Double" /><column name="ContactStatus" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactStatus" property="Address" value="$D$3" type="String" /><column name="ContactStatus" property="ColumnWidth" value="3" type="Double" /><column name="ContactStatus" property="HorizontalAlignment" value="-4108" type="Double" /><column name="ContactStatus" property="VerticalAlignment" value="-4160" type="Double" /><column name="ContactStatus" property="Font.Size" value="10" type="Double" /><column name="IsDoubleName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="IsDoubleName" property="Address" value="$E$3" type="String" /><column name="IsDoubleName" property="ColumnWidth" value="15.86" type="Double" /><column name="ContactCategory" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCategory" property="Address" value="$F$3" type="String" /><column name="ContactCategory" property="ColumnWidth" value="17.29" type="Double" /><column name="FirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="FirstName" property="Address" value="$G$3" type="String" /><column name="FirstName" property="ColumnWidth" value="11.71" type="Double" /><column name="LastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LastName" property="Address" value="$H$3" type="String" /><column name="LastName" property="ColumnWidth" value="11.29" type="Double" /><column name="Email" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Email" property="Address" value="$I$3" type="String" /><column name="Email" property="ColumnWidth" value="29.29" type="Double" /><column name="Mobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Mobile" property="Address" value="$J$3" type="String" /><column name="Mobile" property="ColumnWidth" value="9" type="Double" /><column name="Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Phone" property="Address" value="$K$3" type="String" /><column name="Phone" property="ColumnWidth" value="8.29" type="Double" /><column name="Country" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Country" property="Address" value="$L$3" type="String" /><column name="Country" property="ColumnWidth" value="12.14" type="Double" /><column name="Domain" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Domain" property="Address" value="$M$3" type="String" /><column name="Domain" property="ColumnWidth" value="19.14" type="Double" /><column name="Website" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website" property="Address" value="$N$3" type="String" /><column name="Website" property="ColumnWidth" value="24.57" type="Double" /><column name="Company" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Company" property="Address" value="$O$3" type="String" /><column name="Company" property="ColumnWidth" value="70.71" type="Double" /><column name="LinkedInHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LinkedInHeadline" property="Address" value="$P$3" type="String" /><column name="LinkedInHeadline" property="ColumnWidth" value="26.14" type="Double" /><column name="Industry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Industry" property="Address" value="$Q$3" type="String" /><column name="Industry" property="ColumnWidth" value="39.57" type="Double" /><column name="LocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LocationName" property="Address" value="$R$3" type="String" /><column name="LocationName" property="ColumnWidth" value="17.86" type="Double" /><column name="LinkedInProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LinkedInProfileUrl" property="Address" value="$S$3" type="String" /><column name="LinkedInProfileUrl" property="ColumnWidth" value="29" type="Double" /><column name="GooglePlusProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="GooglePlusProfileUrl" property="Address" value="$T$3" type="String" /><column name="GooglePlusProfileUrl" property="ColumnWidth" value="70.71" type="Double" /><column name="String1" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="String1" property="Address" value="$U$3" type="String" /><column name="String1" property="ColumnWidth" value="21.71" type="Double" /><column name="String2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="String2" property="Address" value="$V$3" type="String" /><column name="String2" property="ColumnWidth" value="21.71" type="Double" /><column name="String3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="String3" property="Address" value="$W$3" type="String" /><column name="String3" property="ColumnWidth" value="21.71" type="Double" /><column name="Number1" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Number1" property="Address" value="$X$3" type="String" /><column name="Number1" property="ColumnWidth" value="21.71" type="Double" /><column name="Number2" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Number2" property="Address" value="$Y$3" type="String" /><column name="Number2" property="ColumnWidth" value="21.71" type="Double" /><column name="Number3" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Number3" property="Address" value="$Z$3" type="String" /><column name="Number3" property="ColumnWidth" value="21.71" type="Double" /><column name="JobTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="JobTitle" property="Address" value="$AA$3" type="String" /><column name="JobTitle" property="ColumnWidth" value="21.71" type="Double" /><column name="ContactStatus" property="FormatConditions(1).AppliesTo.Address" value="$D$3" type="String" /><column name="ContactStatus" property="FormatConditions(1).Type" value="6" type="Double" /><column name="ContactStatus" property="FormatConditions(1).Priority" value="32" type="Double" /><column name="ContactStatus" property="FormatConditions(1).ShowIconOnly" value="True" type="Boolean" /><column name="ContactStatus" property="FormatConditions(1).IconSet.ID" value="13" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(1).Type" value="3" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(1).Operator" value="7" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(2).Value" value="1" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(2).Operator" value="7" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(3).Value" value="2" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(3).Operator" value="7" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(4).Value" value="3" type="Double" /><column name="ContactStatus" property="FormatConditions(1).IconCriteria(4).Operator" value="7" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'dbo71',N'Contacts',N'<table name="dbo71.Contacts"><columnFormats><column name="" property="ListObjectName" value="SourceContacts8" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium2" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="True" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="ID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ID" property="Address" value="$C$4" type="String" /><column name="ID" property="ColumnWidth" value="4.43" type="Double" /><column name="ID" property="Validation.Type" value="1" type="Double" /><column name="ID" property="Validation.AlertStyle" value="1" type="Double" /><column name="ID" property="Validation.Operator" value="1" type="Double" /><column name="ID" property="Validation.Formula1" value="-2147483648" type="String" /><column name="ID" property="Validation.Formula2" value="2147483647" type="String" /><column name="ID" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="ID" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="ID" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="ID" property="Validation.ErrorMessage" value="The column requires values of the int datatype." type="String" /><column name="ID" property="Validation.ShowInput" value="True" type="Boolean" /><column name="ID" property="Validation.ShowError" value="True" type="Boolean" /><column name="ContactStatus" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactStatus" property="Address" value="$D$4" type="String" /><column name="ContactStatus" property="ColumnWidth" value="14.86" type="Double" /><column name="ContactStatus" property="Validation.Type" value="1" type="Double" /><column name="ContactStatus" property="Validation.AlertStyle" value="1" type="Double" /><column name="ContactStatus" property="Validation.Operator" value="1" type="Double" /><column name="ContactStatus" property="Validation.Formula1" value="0" type="String" /><column name="ContactStatus" property="Validation.Formula2" value="255" type="String" /><column name="ContactStatus" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="ContactStatus" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="ContactStatus" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="ContactStatus" property="Validation.ErrorMessage" value="The column requires values of the tinyint datatype." type="String" /><column name="ContactStatus" property="Validation.ShowInput" value="True" type="Boolean" /><column name="ContactStatus" property="Validation.ShowError" value="True" type="Boolean" /><column name="ContactCategory" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactCategory" property="Address" value="$E$4" type="String" /><column name="ContactCategory" property="ColumnWidth" value="17.29" type="Double" /><column name="ContactCategory" property="Validation.Type" value="6" type="Double" /><column name="ContactCategory" property="Validation.AlertStyle" value="1" type="Double" /><column name="ContactCategory" property="Validation.Operator" value="8" type="Double" /><column name="ContactCategory" property="Validation.Formula1" value="255" type="String" /><column name="ContactCategory" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="ContactCategory" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="ContactCategory" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="ContactCategory" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="ContactCategory" property="Validation.ShowInput" value="True" type="Boolean" /><column name="ContactCategory" property="Validation.ShowError" value="True" type="Boolean" /><column name="FirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="FirstName" property="Address" value="$F$4" type="String" /><column name="FirstName" property="ColumnWidth" value="22.86" type="Double" /><column name="FirstName" property="Validation.Type" value="6" type="Double" /><column name="FirstName" property="Validation.AlertStyle" value="1" type="Double" /><column name="FirstName" property="Validation.Operator" value="8" type="Double" /><column name="FirstName" property="Validation.Formula1" value="255" type="String" /><column name="FirstName" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="FirstName" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="FirstName" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="FirstName" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="FirstName" property="Validation.ShowInput" value="True" type="Boolean" /><column name="FirstName" property="Validation.ShowError" value="True" type="Boolean" /><column name="LastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="LastName" property="Address" value="$G$4" type="String" /><column name="LastName" property="ColumnWidth" value="12.71" type="Double" /><column name="LastName" property="Validation.Type" value="6" type="Double" /><column name="LastName" property="Validation.AlertStyle" value="1" type="Double" /><column name="LastName" property="Validation.Operator" value="8" type="Double" /><column name="LastName" property="Validation.Formula1" value="255" type="String" /><column name="LastName" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="LastName" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="LastName" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="LastName" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="LastName" property="Validation.ShowInput" value="True" type="Boolean" /><column name="LastName" property="Validation.ShowError" value="True" type="Boolean" /><column name="Email" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Email" property="Address" value="$H$4" type="String" /><column name="Email" property="ColumnWidth" value="31.71" type="Double" /><column name="Email" property="Validation.Type" value="6" type="Double" /><column name="Email" property="Validation.AlertStyle" value="1" type="Double" /><column name="Email" property="Validation.Operator" value="8" type="Double" /><column name="Email" property="Validation.Formula1" value="255" type="String" /><column name="Email" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Email" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Email" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="Email" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="Email" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Email" property="Validation.ShowError" value="True" type="Boolean" /><column name="Mobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Mobile" property="Address" value="$I$4" type="String" /><column name="Mobile" property="ColumnWidth" value="17.43" type="Double" /><column name="Mobile" property="Validation.Type" value="6" type="Double" /><column name="Mobile" property="Validation.AlertStyle" value="1" type="Double" /><column name="Mobile" property="Validation.Operator" value="8" type="Double" /><column name="Mobile" property="Validation.Formula1" value="255" type="String" /><column name="Mobile" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Mobile" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Mobile" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="Mobile" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="Mobile" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Mobile" property="Validation.ShowError" value="True" type="Boolean" /><column name="Phone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Phone" property="Address" value="$J$4" type="String" /><column name="Phone" property="ColumnWidth" value="16" type="Double" /><column name="Phone" property="Validation.Type" value="6" type="Double" /><column name="Phone" property="Validation.AlertStyle" value="1" type="Double" /><column name="Phone" property="Validation.Operator" value="8" type="Double" /><column name="Phone" property="Validation.Formula1" value="255" type="String" /><column name="Phone" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Phone" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Phone" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="Phone" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="Phone" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Phone" property="Validation.ShowError" value="True" type="Boolean" /><column name="Website" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Website" property="Address" value="$K$4" type="String" /><column name="Website" property="ColumnWidth" value="15" type="Double" /><column name="Website" property="Validation.Type" value="6" type="Double" /><column name="Website" property="Validation.AlertStyle" value="1" type="Double" /><column name="Website" property="Validation.Operator" value="8" type="Double" /><column name="Website" property="Validation.Formula1" value="255" type="String" /><column name="Website" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Website" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Website" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="Website" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="Website" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Website" property="Validation.ShowError" value="True" type="Boolean" /><column name="Company" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Company" property="Address" value="$L$4" type="String" /><column name="Company" property="ColumnWidth" value="15.71" type="Double" /><column name="Company" property="Validation.Type" value="6" type="Double" /><column name="Company" property="Validation.AlertStyle" value="1" type="Double" /><column name="Company" property="Validation.Operator" value="8" type="Double" /><column name="Company" property="Validation.Formula1" value="255" type="String" /><column name="Company" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Company" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Company" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="Company" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="Company" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Company" property="Validation.ShowError" value="True" type="Boolean" /><column name="Title" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Title" property="Address" value="$M$4" type="String" /><column name="Title" property="ColumnWidth" value="24.71" type="Double" /><column name="Title" property="Validation.Type" value="6" type="Double" /><column name="Title" property="Validation.AlertStyle" value="1" type="Double" /><column name="Title" property="Validation.Operator" value="8" type="Double" /><column name="Title" property="Validation.Formula1" value="255" type="String" /><column name="Title" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Title" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Title" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="Title" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="Title" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Title" property="Validation.ShowError" value="True" type="Boolean" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="2" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="2" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats></table>');
INSERT INTO [dbo01].[TableFormats] ([TABLE_SCHEMA],[TABLE_NAME],[TABLE_EXCEL_FORMAT_XML]) VALUES (N'dbo71',N'SourceContacts',N'<table name="dbo71.SourceContacts"><columnFormats><column name="" property="ListObjectName" value="SourceContacts" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium2" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="True" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="Address" value="$C$4" type="String" /><column name="Source" property="ColumnWidth" value="8.57" type="Double" /><column name="Source" property="Validation.Type" value="6" type="Double" /><column name="Source" property="Validation.AlertStyle" value="1" type="Double" /><column name="Source" property="Validation.Operator" value="8" type="Double" /><column name="Source" property="Validation.Formula1" value="50" type="String" /><column name="Source" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="Source" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="Source" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="Source" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(50) datatype." type="String" /><column name="Source" property="Validation.ShowInput" value="True" type="Boolean" /><column name="Source" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="SourceID" property="Address" value="$D$4" type="String" /><column name="SourceID" property="Validation.Type" value="6" type="Double" /><column name="SourceID" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceID" property="Validation.Operator" value="8" type="Double" /><column name="SourceID" property="Validation.Formula1" value="50" type="String" /><column name="SourceID" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceID" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceID" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceID" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(50) datatype." type="String" /><column name="SourceID" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceID" property="Validation.ShowError" value="True" type="Boolean" /><column name="MergeDate" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="MergeDate" property="Address" value="$E$4" type="String" /><column name="MergeDate" property="NumberFormat" value="m/d/yyyy h:mm" type="String" /><column name="MergeDate" property="Validation.Type" value="4" type="Double" /><column name="MergeDate" property="Validation.AlertStyle" value="1" type="Double" /><column name="MergeDate" property="Validation.Operator" value="5" type="Double" /><column name="MergeDate" property="Validation.Formula1" value="01.01.1900" type="String" /><column name="MergeDate" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="MergeDate" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="MergeDate" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="MergeDate" property="Validation.ErrorMessage" value="The column requires values of the datetime datatype." type="String" /><column name="MergeDate" property="Validation.ShowInput" value="True" type="Boolean" /><column name="MergeDate" property="Validation.ShowError" value="True" type="Boolean" /><column name="IsJunk" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="IsJunk" property="Address" value="$F$4" type="String" /><column name="IsJunk" property="ColumnWidth" value="8" type="Double" /><column name="IsJunk" property="Validation.Type" value="1" type="Double" /><column name="IsJunk" property="Validation.AlertStyle" value="1" type="Double" /><column name="IsJunk" property="Validation.Operator" value="1" type="Double" /><column name="IsJunk" property="Validation.Formula1" value="0" type="String" /><column name="IsJunk" property="Validation.Formula2" value="255" type="String" /><column name="IsJunk" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="IsJunk" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="IsJunk" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="IsJunk" property="Validation.ErrorMessage" value="The column requires values of the tinyint datatype." type="String" /><column name="IsJunk" property="Validation.ShowInput" value="True" type="Boolean" /><column name="IsJunk" property="Validation.ShowError" value="True" type="Boolean" /><column name="ContactID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactID" property="Address" value="$G$4" type="String" /><column name="ContactID" property="ColumnWidth" value="11.14" type="Double" /><column name="ContactID" property="Validation.Type" value="1" type="Double" /><column name="ContactID" property="Validation.AlertStyle" value="1" type="Double" /><column name="ContactID" property="Validation.Operator" value="1" type="Double" /><column name="ContactID" property="Validation.Formula1" value="-2147483648" type="String" /><column name="ContactID" property="Validation.Formula2" value="2147483647" type="String" /><column name="ContactID" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="ContactID" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="ContactID" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="ContactID" property="Validation.ErrorMessage" value="The column requires values of the int datatype." type="String" /><column name="ContactID" property="Validation.ShowInput" value="True" type="Boolean" /><column name="ContactID" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="Address" value="$H$4" type="String" /><column name="SourceFirstName" property="ColumnWidth" value="25.29" type="Double" /><column name="SourceFirstName" property="Validation.Type" value="6" type="Double" /><column name="SourceFirstName" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceFirstName" property="Validation.Operator" value="8" type="Double" /><column name="SourceFirstName" property="Validation.Formula1" value="255" type="String" /><column name="SourceFirstName" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceFirstName" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceFirstName" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceFirstName" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceFirstName" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceFirstName" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="Address" value="$I$4" type="String" /><column name="SourceLastName" property="ColumnWidth" value="17.43" type="Double" /><column name="SourceLastName" property="Validation.Type" value="6" type="Double" /><column name="SourceLastName" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceLastName" property="Validation.Operator" value="8" type="Double" /><column name="SourceLastName" property="Validation.Formula1" value="255" type="String" /><column name="SourceLastName" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceLastName" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceLastName" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceLastName" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceLastName" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceLastName" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="Address" value="$J$4" type="String" /><column name="SourceEmail" property="ColumnWidth" value="31.71" type="Double" /><column name="SourceEmail" property="Validation.Type" value="6" type="Double" /><column name="SourceEmail" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceEmail" property="Validation.Operator" value="8" type="Double" /><column name="SourceEmail" property="Validation.Formula1" value="255" type="String" /><column name="SourceEmail" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceEmail" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceEmail" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceEmail" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceEmail" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceEmail" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceSecondaryEmail" property="Address" value="$K$4" type="String" /><column name="SourceSecondaryEmail" property="ColumnWidth" value="23" type="Double" /><column name="SourceSecondaryEmail" property="Validation.Type" value="6" type="Double" /><column name="SourceSecondaryEmail" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceSecondaryEmail" property="Validation.Operator" value="8" type="Double" /><column name="SourceSecondaryEmail" property="Validation.Formula1" value="255" type="String" /><column name="SourceSecondaryEmail" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceSecondaryEmail" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceSecondaryEmail" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceSecondaryEmail" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceSecondaryEmail" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceSecondaryEmail" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceMobile" property="Address" value="$L$4" type="String" /><column name="SourceMobile" property="ColumnWidth" value="17.29" type="Double" /><column name="SourceMobile" property="Validation.Type" value="6" type="Double" /><column name="SourceMobile" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceMobile" property="Validation.Operator" value="8" type="Double" /><column name="SourceMobile" property="Validation.Formula1" value="255" type="String" /><column name="SourceMobile" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceMobile" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceMobile" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceMobile" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceMobile" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceMobile" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourcePhone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourcePhone" property="Address" value="$M$4" type="String" /><column name="SourcePhone" property="ColumnWidth" value="14.29" type="Double" /><column name="SourcePhone" property="Validation.Type" value="6" type="Double" /><column name="SourcePhone" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourcePhone" property="Validation.Operator" value="8" type="Double" /><column name="SourcePhone" property="Validation.Formula1" value="255" type="String" /><column name="SourcePhone" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourcePhone" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourcePhone" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourcePhone" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourcePhone" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourcePhone" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="Address" value="$N$4" type="String" /><column name="SourceCompany" property="ColumnWidth" value="17.86" type="Double" /><column name="SourceCompany" property="Validation.Type" value="6" type="Double" /><column name="SourceCompany" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceCompany" property="Validation.Operator" value="8" type="Double" /><column name="SourceCompany" property="Validation.Formula1" value="255" type="String" /><column name="SourceCompany" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceCompany" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceCompany" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceCompany" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceCompany" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceCompany" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceTitle" property="Address" value="$O$4" type="String" /><column name="SourceTitle" property="ColumnWidth" value="12.57" type="Double" /><column name="SourceTitle" property="Validation.Type" value="6" type="Double" /><column name="SourceTitle" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceTitle" property="Validation.Operator" value="8" type="Double" /><column name="SourceTitle" property="Validation.Formula1" value="255" type="String" /><column name="SourceTitle" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceTitle" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceTitle" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceTitle" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceTitle" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceTitle" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceHeadline" property="Address" value="$P$4" type="String" /><column name="SourceHeadline" property="ColumnWidth" value="16.71" type="Double" /><column name="SourceHeadline" property="Validation.Type" value="6" type="Double" /><column name="SourceHeadline" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceHeadline" property="Validation.Operator" value="8" type="Double" /><column name="SourceHeadline" property="Validation.Formula1" value="255" type="String" /><column name="SourceHeadline" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceHeadline" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceHeadline" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceHeadline" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceHeadline" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceHeadline" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceIndustry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceIndustry" property="Address" value="$Q$4" type="String" /><column name="SourceIndustry" property="ColumnWidth" value="16" type="Double" /><column name="SourceIndustry" property="Validation.Type" value="6" type="Double" /><column name="SourceIndustry" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceIndustry" property="Validation.Operator" value="8" type="Double" /><column name="SourceIndustry" property="Validation.Formula1" value="255" type="String" /><column name="SourceIndustry" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceIndustry" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceIndustry" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceIndustry" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceIndustry" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceIndustry" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceLocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLocationName" property="Address" value="$R$4" type="String" /><column name="SourceLocationName" property="ColumnWidth" value="21.57" type="Double" /><column name="SourceLocationName" property="Validation.Type" value="6" type="Double" /><column name="SourceLocationName" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceLocationName" property="Validation.Operator" value="8" type="Double" /><column name="SourceLocationName" property="Validation.Formula1" value="255" type="String" /><column name="SourceLocationName" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceLocationName" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceLocationName" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceLocationName" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceLocationName" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceLocationName" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceCountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCountryCode" property="Address" value="$S$4" type="String" /><column name="SourceCountryCode" property="ColumnWidth" value="20.43" type="Double" /><column name="SourceCountryCode" property="Validation.Type" value="6" type="Double" /><column name="SourceCountryCode" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceCountryCode" property="Validation.Operator" value="8" type="Double" /><column name="SourceCountryCode" property="Validation.Formula1" value="2" type="String" /><column name="SourceCountryCode" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceCountryCode" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceCountryCode" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceCountryCode" property="Validation.ErrorMessage" value="The column requires values of the char(2) datatype." type="String" /><column name="SourceCountryCode" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceCountryCode" property="Validation.ShowError" value="True" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceProfileUrl" property="Address" value="$T$4" type="String" /><column name="SourceProfileUrl" property="ColumnWidth" value="17.43" type="Double" /><column name="SourceProfileUrl" property="Validation.Type" value="6" type="Double" /><column name="SourceProfileUrl" property="Validation.AlertStyle" value="1" type="Double" /><column name="SourceProfileUrl" property="Validation.Operator" value="8" type="Double" /><column name="SourceProfileUrl" property="Validation.Formula1" value="255" type="String" /><column name="SourceProfileUrl" property="Validation.IgnoreBlank" value="True" type="Boolean" /><column name="SourceProfileUrl" property="Validation.InCellDropdown" value="True" type="Boolean" /><column name="SourceProfileUrl" property="Validation.ErrorTitle" value="Datatype Control" type="String" /><column name="SourceProfileUrl" property="Validation.ErrorMessage" value="The column requires values of the nvarchar(255) datatype." type="String" /><column name="SourceProfileUrl" property="Validation.ShowInput" value="True" type="Boolean" /><column name="SourceProfileUrl" property="Validation.ShowError" value="True" type="Boolean" /><column name="Source" property="FormatConditions(1).AppliesTo.Address" value="$C$4:$C$59" type="String" /><column name="Source" property="FormatConditions(1).Type" value="2" type="Double" /><column name="Source" property="FormatConditions(1).Priority" value="1" type="Double" /><column name="Source" property="FormatConditions(1).Formula1" value="=ISBLANK(C4)" type="String" /><column name="Source" property="FormatConditions(1).Interior.Color" value="65535" type="Double" /><column name="Source" property="FormatConditions(1).Interior.PatternThemeColor" value="-1" type="Double" /><column name="SourceID" property="FormatConditions(1).AppliesTo.Address" value="$D$4:$D$59" type="String" /><column name="SourceID" property="FormatConditions(1).Type" value="2" type="Double" /><column name="SourceID" property="FormatConditions(1).Priority" value="2" type="Double" /><column name="SourceID" property="FormatConditions(1).Formula1" value="=ISBLANK(D4)" type="String" /><column name="SourceID" property="FormatConditions(1).Interior.Color" value="65535" type="Double" /><column name="SourceID" property="FormatConditions(1).Interior.PatternThemeColor" value="-1" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="2" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="2" type="Double" /><column name="" property="PageSetup.PaperSize" value="1" type="Double" /><column name="" property="PageSetup.LeftMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.RightMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.TopMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.BottomMargin" value="25.511811023622" type="Double" /><column name="" property="PageSetup.HeaderMargin" value="22.6771653543307" type="Double" /><column name="" property="PageSetup.FooterMargin" value="22.6771653543307" type="Double" /></columnFormats><views><view name="All"><column name="" property="ListObjectName" value="SourceContacts" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="MergeDate" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="IsJunk" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourcePhone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceIndustry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /></view><view name="Main Fields"><column name="" property="ListObjectName" value="SourceContacts" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="Source" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="MergeDate" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="IsJunk" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ContactID" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceFirstName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLastName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceSecondaryEmail" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceMobile" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourcePhone" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCompany" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceTitle" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceHeadline" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceIndustry" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceLocationName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceCountryCode" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SourceProfileUrl" property="EntireColumn.Hidden" value="False" type="Boolean" /></view></views></table>');
GO

print N'The application has been installed';
GO
